@@ -56,7 +56,7 @@ static GCC_INLINE ub4 perl_hash_function(
     )
 {
     register ub4  hash_value_int = 0;
-    register ub1 * s_end = s_ptr+length;
+    register ub1 * const s_end = s_ptr+length;
 
     while (s_ptr < s_end)
     {
@@ -80,34 +80,22 @@ static GCC_INLINE ub4 perl_hash_function(
  * replaced_with_cached macro above.
  * */
 static GCC_INLINE void fc_solve_cache_stacks(
-        fc_solve_hard_thread_t * hard_thread,
-        fcs_kv_state_t * new_state
+        fc_solve_hard_thread_t * const hard_thread,
+        fcs_kv_state_t * const new_state
         )
 {
-    int i;
-#if (FCS_STACK_STORAGE == FCS_STACK_STORAGE_INTERNAL_HASH)
-#ifdef FCS_ENABLE_SECONDARY_HASH_VALUE
-    SFO_hash_value_t hash_value_int;
-#endif
-#elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_JUDY)
-    PWord_t * PValue;
-#endif
-    void * cached_stack;
-    char * new_ptr;
-    fc_solve_instance_t * instance = hard_thread->instance;
+    fc_solve_instance_t * const instance = hard_thread->instance;
 #ifndef HARD_CODED_NUM_STACKS
-    DECLARE_AND_SET_GAME_PARAMS();
+    SET_GAME_PARAMS();
 #endif
-    fcs_cards_column_t column;
-    register int col_len;
-    fcs_compact_allocator_t * stacks_allocator;
-    register fcs_state_t * new_state_key = new_state->key;
-    register fcs_state_extra_info_t * new_state_info = new_state->val;
-    register fcs_cards_column_t * current_stack;
+    register fcs_state_t * const new_state_key = new_state->key;
+    register fcs_state_extra_info_t * const new_state_info = new_state->val;
 
-    stacks_allocator = &(hard_thread->allocator);
+    fcs_compact_allocator_t * const stacks_allocator = &(hard_thread->allocator);
 
-    for (i=0, current_stack = new_state_key->stacks ; i < LOCAL_STACKS_NUM ; i++, current_stack++)
+    fcs_cards_column_t * current_stack = new_state_key->stacks;
+
+    for (int i=0 ; i < LOCAL_STACKS_NUM ; i++, current_stack++)
     {
         /*
          * If the stack is not a copy - it is already cached so skip
@@ -118,22 +106,23 @@ static GCC_INLINE void fc_solve_cache_stacks(
             continue;
         }
 
-        column = fcs_state_get_col(*(new_state_key), i);
-        col_len = (fcs_col_len(column)+1);
+        fcs_cards_column_t column = fcs_state_get_col(*(new_state_key), i);
+        const int col_len = (fcs_col_len(column)+1);
 
-        new_ptr = (char*)fcs_compact_alloc_ptr(stacks_allocator, col_len);
+        char * const new_ptr = (char*)fcs_compact_alloc_ptr(stacks_allocator, col_len);
         memcpy(new_ptr, column, col_len);
         *(current_stack) = new_ptr;
 
+        void * cached_stack;
 #if FCS_STACK_STORAGE == FCS_STACK_STORAGE_INTERNAL_HASH
 #ifdef FCS_ENABLE_SECONDARY_HASH_VALUE
         /* Calculate the hash value for the stack */
         /* This hash function was ripped from the Perl source code.
          * (It is not derived work however). */
+        SFO_hash_value_t hash_value_int = 0;
         {
             const char * s_ptr = (char*)(*(current_stack));
             const char * s_end = s_ptr+fcs_col_len(s_ptr)+1;
-            hash_value_int = 0;
             while (s_ptr < s_end)
             {
                 hash_value_int += (hash_value_int << 5) + *(s_ptr++);
@@ -171,7 +160,7 @@ static GCC_INLINE void fc_solve_cache_stacks(
 
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_GOOGLE_DENSE_HASH)
         {
-            column = fcs_state_get_col(new_state->s, i);
+            column = fcs_state_get_col(*new_state_key, i);
 
             replace_with_cached(
                 fc_solve_columns_google_hash_insert(
@@ -186,7 +175,7 @@ static GCC_INLINE void fc_solve_cache_stacks(
         cached_stack =
             fcs_libavl2_stacks_tree_insert(
                 instance->stacks_tree,
-                new_state->s.stacks[i]
+                new_state_key->stacks[i]
             );
 
         replace_with_cached(cached_stack != NULL);
@@ -229,6 +218,7 @@ static GCC_INLINE void fc_solve_cache_stacks(
                 );
         }
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_JUDY)
+        PWord_t * PValue;
         column = fcs_state_get_col(*new_state_key, i);
 
         JHSI(
@@ -264,10 +254,9 @@ static GCC_INLINE void fc_solve_cache_stacks(
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_GLIB_HASH)
 guint fc_solve_hash_function(gconstpointer key)
 {
-    guint hash_value;
     const char * s_ptr = (char*)key;
-    const char * s_end = s_ptr+sizeof(fcs_state_t);
-    hash_value = 0;
+    const char * const s_end = s_ptr+sizeof(fcs_state_t);
+    guint hash_value = 0;
     while (s_ptr < s_end)
     {
         hash_value += (hash_value << 5) + *(s_ptr++);
@@ -300,14 +289,14 @@ guint fc_solve_hash_function(gconstpointer key)
  * */
 
 static GCC_INLINE void on_state_new(
-    fc_solve_instance_t * instance,
-    fc_solve_hard_thread_t * hard_thread,
-    fcs_state_extra_info_t * new_state_info
+    fc_solve_instance_t * const instance,
+    fc_solve_hard_thread_t * const hard_thread,
+    fcs_state_extra_info_t * const new_state_info
 )
 {
-    register fcs_collectible_state_t * parent_state;
+    fcs_collectible_state_t * const parent_state = new_state_info->parent;
     /* The new state was not found in the cache, and it was already inserted */
-    if (likely(parent_state = new_state_info->parent))
+    if (likely(parent_state))
     {
         (FCS_S_NUM_ACTIVE_CHILDREN(parent_state))++;
         /* If parent_val is defined, so is moves_to_parent */
@@ -325,9 +314,9 @@ static GCC_INLINE void on_state_new(
 }
 
 fcs_bool_t fc_solve_check_and_add_state(
-    fc_solve_hard_thread_t * hard_thread,
-    fcs_kv_state_t * new_state,
-    fcs_kv_state_t * existing_state_raw
+    fc_solve_hard_thread_t * const hard_thread,
+    fcs_kv_state_t * const new_state,
+    fcs_kv_state_t * const existing_state_raw
     )
 {
 /*
@@ -340,16 +329,7 @@ fcs_bool_t fc_solve_check_and_add_state(
 
 #define ON_STATE_NEW() on_state_new(instance, hard_thread, new_state->val);
 
-#if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_INTERNAL_HASH)
-#ifdef FCS_ENABLE_SECONDARY_HASH_VALUE
-    SFO_hash_value_t hash_value_int;
-#endif
-#endif
-#if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_INDIRECT)
-    fcs_standalone_state_ptrs_t * pos_ptr;
-    fcs_bool_t found;
-#endif
-    fc_solve_instance_t * instance = hard_thread->instance;
+    fc_solve_instance_t * const instance = hard_thread->instance;
 
     /* #if'ing out because it doesn't belong here. */
 #if 0
@@ -381,10 +361,10 @@ fcs_bool_t fc_solve_check_and_add_state(
 
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_INTERNAL_HASH)
 #ifdef FCS_ENABLE_SECONDARY_HASH_VALUE
+    SFO_hash_value_t hash_value_int = 0;
     {
         const char * s_ptr = (char*)new_state_key;
         const char * s_end = s_ptr+sizeof(*new_state_key);
-        hash_value_int = 0;
         while (s_ptr < s_end)
         {
             hash_value_int += (hash_value_int << 5) + *(s_ptr++);
@@ -451,6 +431,7 @@ fcs_bool_t fc_solve_check_and_add_state(
         }
     }
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_INDIRECT)
+    fcs_standalone_state_ptrs_t * pos_ptr;
     /* Try to see if the state is found in indirect_prev_states */
     if ((pos_ptr = (fcs_standalone_state_ptrs_t *)bsearch(&new_state_key,
                 instance->indirect_prev_states,
@@ -458,6 +439,7 @@ fcs_bool_t fc_solve_check_and_add_state(
                 sizeof(instance->indirect_prev_states[0]),
                 fc_solve_state_compare_indirect)) == NULL)
     {
+        fcs_bool_t found;
         /* It isn't in prev_states, but maybe it's in the sort margin */
         pos_ptr = (fcs_standalone_state_ptrs_t *)fc_solve_bsearch(
             &new_state_key,
@@ -493,12 +475,10 @@ fcs_bool_t fc_solve_check_and_add_state(
                 /* The sort margin is full, let's combine it with the main array */
 
                 instance->indirect_prev_states =
-                    realloc(
+                    SREALLOC(
                         instance->indirect_prev_states,
-                        sizeof(instance->indirect_prev_states[0])
-                        * (instance->num_indirect_prev_states
+                        instance->num_indirect_prev_states
                             + instance->num_prev_states_margin
-                        )
                     );
 
                 fc_solve_merge_large_and_small_sorted_arrays(
