@@ -33,12 +33,14 @@
 #include <stdio.h>
 #include <ctype.h>
 
+#include "set_weights.h"
 #include "fcs_user.h"
 #include "fcs_cl.h"
 #include "split_cmd_line.h"
 
 #include "prefix.h"
 #include "inline.h"
+#include "bool.h"
 
 #include "cmd_line_enum.h"
 
@@ -47,13 +49,9 @@ static GCC_INLINE int read_preset(const char * preset_name, args_man_t * * args_
     int ret_code = 1;
     char * home_dir_presetrc = NULL, * env_var_presetrc = NULL;
     const char * global_presetrc = NULL;
-    const char * path;
     const char * * presetrc_pathes[5];
-    int path_idx;
-    char line[8192];
     FILE * f = NULL;
     char * opened_files_dir = NULL;
-    int read_next_preset = 0;
 
 
     {
@@ -70,7 +68,10 @@ static GCC_INLINE int read_preset(const char * preset_name, args_man_t * * args_
         home_dir = getenv("HOME");
         if (home_dir)
         {
-            home_dir_presetrc = malloc(strlen(home_dir) + 50);
+            home_dir_presetrc = SMALLOC(
+                home_dir_presetrc,
+                strlen(home_dir) + 50
+            );
             sprintf(home_dir_presetrc,
                 "%s/.freecell-solver/presetrc", home_dir
                 );
@@ -80,7 +81,9 @@ static GCC_INLINE int read_preset(const char * preset_name, args_man_t * * args_
 
     global_presetrc = (FREECELL_SOLVER_PKG_DATA_DIR "/presetrc");
 
-    for(path_idx=0;(presetrc_pathes[path_idx] != NULL) ; path_idx++)
+    const char * path;
+    fcs_bool_t read_next_preset = FALSE;
+    for (int path_idx=0;(presetrc_pathes[path_idx] != NULL) ; path_idx++)
     {
         path = (*presetrc_pathes[path_idx]);
         if (path == NULL)
@@ -94,6 +97,7 @@ static GCC_INLINE int read_preset(const char * preset_name, args_man_t * * args_
         }
         while(1)
         {
+            char line[8192];
             if (fgets(line, sizeof(line), f) == NULL)
             {
                 break;
@@ -123,7 +127,7 @@ static GCC_INLINE int read_preset(const char * preset_name, args_man_t * * args_
                 nullify_newline();
                 if (!strcmp(line+5, preset_name))
                 {
-                    read_next_preset = 1;
+                    read_next_preset = TRUE;
                 }
             }
             else if (!strncmp(line, "command=", 8))
@@ -132,6 +136,10 @@ static GCC_INLINE int read_preset(const char * preset_name, args_man_t * * args_
                 {
                     *args_man = fc_solve_args_man_alloc();
                     fc_solve_args_man_chop(*args_man, line+8);
+#if 0
+                    fprintf(stderr, "man_chop for <<<%s>>>\n", line);
+                    fflush(stderr);
+#endif
                     ret_code = 0;
                     goto have_preset;
                 }
@@ -168,6 +176,54 @@ have_preset:
     return ret_code;
 }
 
+DLLEXPORT int freecell_solver_user_cmd_line_read_cmd_line_preset(
+    void * instance,
+    const char * const preset_name,
+    freecell_solver_str_t * known_parameters,
+    char * * error_string,
+    int file_nesting_count,
+    freecell_solver_str_t opened_files_dir
+)
+{
+    int status;
+    args_man_t * preset_args;
+    char * dir = NULL;
+
+    status = read_preset(preset_name, &preset_args, &dir, NULL);
+    if (status != 0)
+    {
+        *error_string = strdup("Could not read preset.");
+        return FCS_CMD_LINE_ERROR_IN_ARG;
+    }
+    else
+    {
+        int ret;
+        int last_arg = 0;
+
+        ret = freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
+            instance,
+            preset_args->argc,
+            (freecell_solver_str_t *)(void *)(preset_args->argv),
+            0,
+            known_parameters,
+            NULL,
+            NULL,
+            error_string,
+            &(last_arg),
+            ((file_nesting_count < 0) ? file_nesting_count : (file_nesting_count-1)),
+            dir ? dir : opened_files_dir
+        );
+
+        if (dir)
+        {
+            free(dir);
+        }
+        fc_solve_args_man_free(preset_args);
+
+        return ret;
+
+    }
+}
 
 DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
     void * instance,
@@ -183,43 +239,44 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
     freecell_solver_str_t opened_files_dir
     )
 {
-    freecell_solver_str_t * arg;
-    freecell_solver_str_t * arg_argc;
-    freecell_solver_str_t * known_param;
     int num_to_skip;
-    int callback_ret;
-    int opt;
     const char * p;
 
     *error_string = NULL;
 
-    arg_argc = &(argv[argc]);
+    freecell_solver_str_t * arg_argc = &(argv[argc]);
 
-    for(arg=&(argv[start_arg]);arg<arg_argc;arg++)
+    freecell_solver_str_t * arg;
+    freecell_solver_str_t * known_param;
+    for (arg = &(argv[start_arg]) ; arg < arg_argc ; arg++)
     {
         /* First check for the parameters that the user recognizes */
-        known_param = known_parameters;
-        while((*known_param) && strcmp(*known_param, (*arg)))
+        if (known_parameters)
         {
-            known_param++;
-        }
-        if ((*known_param) != NULL )
-        {
-            int ret;
-
-            callback_ret = callback(instance, argc, argv, arg-&(argv[0]), &num_to_skip, &ret, callback_context);
-            if (callback_ret == FCS_CMD_LINE_SKIP)
+            known_param = known_parameters;
+            while((*known_param) && strcmp(*known_param, (*arg)))
             {
-                arg += num_to_skip-1;
-                continue;
+                known_param++;
             }
-            else if (callback_ret == FCS_CMD_LINE_STOP)
+            if ((*known_param) != NULL )
             {
-                *last_arg = arg-&(argv[0]);
-                return ret;
+                int ret;
+
+                const int callback_ret = callback(instance, argc, argv, arg-&(argv[0]), &num_to_skip, &ret, callback_context);
+                if (callback_ret == FCS_CMD_LINE_SKIP)
+                {
+                    arg += num_to_skip-1;
+                    continue;
+                }
+                else if (callback_ret == FCS_CMD_LINE_STOP)
+                {
+                    *last_arg = arg-&(argv[0]);
+                    return ret;
+                }
             }
         }
 
+        int opt;
         /* OPT-PARSE-START */
 p = (*arg);
 opt = FCS_OPT_UNRECOGNIZED;
@@ -345,9 +402,37 @@ opt = FCS_OPT_FLARE_NAME;
 break;
 
 case 's':
+
 {
-if (!strncmp(p, "-plan", 5)) {
+if (*(p++) == '-')
+{
+{ switch(*(p++)) {
+case 'c':
+{
+if (!strncmp(p, "hoice", 5)) {
 p += 5;
+opt = FCS_OPT_FLARES_CHOICE;
+
+}
+}
+
+break;
+
+case 'i':
+{
+if (!strncmp(p, "ters-factor", 11)) {
+p += 11;
+opt = FCS_OPT_FLARES_ITERS_FACTOR;
+
+}
+}
+
+break;
+
+case 'p':
+{
+if (!strncmp(p, "lan", 3)) {
+p += 3;
 opt = FCS_OPT_FLARES_PLAN;
 
 }
@@ -358,6 +443,15 @@ break;
 }
 }
 
+}
+
+}
+
+break;
+
+}
+}
+
 }
 }
 
@@ -807,6 +901,17 @@ opt = FCS_OPT_DEPTH_TESTS_ORDER;
 
 break;
 
+case 'f':
+{
+if (!strncmp(p, "if", 2)) {
+p += 2;
+opt = FCS_OPT_FLARES_ITERS_FACTOR;
+
+}
+}
+
+break;
+
 case 'g':
 
 {
@@ -1080,7 +1185,7 @@ break;
         {
             PROCESS_OPT_ARG() ;
 
-            freecell_solver_user_limit_iterations(instance, atoi((*arg)));
+            freecell_solver_user_limit_iterations_long(instance, atol((*arg)));
         }
         break;
 
@@ -1094,7 +1199,7 @@ break;
             ret = freecell_solver_user_set_tests_order(instance, (*arg), &fcs_user_errstr);
             if (ret != 0)
             {
-                char * errstr = malloc(strlen(fcs_user_errstr)+500);
+                char * errstr = SMALLOC(errstr, strlen(fcs_user_errstr)+500);
                 sprintf(
                     errstr,
                     "Error in tests' order!\n%s\n",
@@ -1116,7 +1221,7 @@ break;
             {
                 char * errstr;
 
-                errstr = malloc(500);
+                errstr = SMALLOC(errstr, 500);
                 sprintf(errstr,
                     "Error! The freecells\' number "
                     "exceeds the maximum of %i.\n"
@@ -1138,7 +1243,7 @@ break;
             {
                 char * errstr;
 
-                errstr = malloc(500);
+                errstr = SMALLOC(errstr, 500);
                 sprintf(errstr,
                     "Error! The stacks\' number "
                     "exceeds the maximum of %i.\n"
@@ -1160,7 +1265,7 @@ break;
             {
                 char * errstr;
 
-                errstr = malloc(500);
+                errstr = SMALLOC(errstr, 500);
                 sprintf(errstr,
                     "Error! The decks\' number "
                     "exceeds the maximum of %i.\n"
@@ -1244,7 +1349,7 @@ break;
             {
                 char * errstr;
 
-                errstr = malloc(strlen((*arg))+500);
+                errstr = SMALLOC(errstr, strlen((*arg))+500);
 
                 sprintf(errstr, "Unknown game \"%s\"!\n\n", (*arg));
                 *error_string = errstr;
@@ -1255,7 +1360,7 @@ break;
             {
                 char * errstr;
 
-                errstr = malloc(strlen((*arg))+500);
+                errstr = SMALLOC(errstr, strlen((*arg))+500);
                 sprintf(errstr, "The game \"%s\" exceeds the maximal number "
                         "of freecells in the program.\n"
                         "Modify the file \"config.h\" and recompile, "
@@ -1271,7 +1376,7 @@ break;
             {
                 char * errstr;
 
-                errstr = malloc(strlen((*arg))+500);
+                errstr = SMALLOC(errstr, strlen((*arg))+500);
 
                 sprintf(errstr, "The game \"%s\" exceeds the maximal number "
                         "of stacks in the program.\n"
@@ -1288,7 +1393,7 @@ break;
             {
                 char * errstr;
 
-                errstr = malloc(strlen((*arg))+500);
+                errstr = SMALLOC(errstr, strlen((*arg))+500);
 
                 sprintf(errstr,
                     "The game \"%s\" exceeds the limits of the program.\n"
@@ -1330,7 +1435,7 @@ break;
             {
                 char * errstr;
 
-                errstr = malloc(strlen((*arg))+500);
+                errstr = SMALLOC(errstr, strlen((*arg))+500);
 
                 sprintf(
                     errstr,
@@ -1352,62 +1457,23 @@ break;
             PROCESS_OPT_ARG() ;
 
             {
-                int a;
-                freecell_solver_str_t start_num;
-                freecell_solver_str_t end_num;
-                char * substring_copy;
-
-                start_num = (*arg);
-
-                substring_copy = malloc(strlen(start_num) + 1);
-
+                int i;
                 /* Initialize all the Best Frist Search weights at first
                  * to 0 so
                  * we won't have partial initialization.
                  * */
-                for (a=0 ; a<5 ; a++)
+                double befs_weights[FCS_NUM_BEFS_WEIGHTS];
+
+                fc_solve_set_weights(*arg, befs_weights);
+
+                for (i=0 ; i<FCS_NUM_BEFS_WEIGHTS ; i++)
                 {
                     freecell_solver_user_set_a_star_weight(
                         instance,
-                        a,
-                        0
-                        );
+                        i,
+                        befs_weights[i]
+                    );
                 }
-
-                for (a=0 ; a<5 ; a++)
-                {
-                    while ((*start_num > '9') && (*start_num < '0') && (*start_num != '\0'))
-                    {
-                        start_num++;
-                    }
-                    if (*start_num == '\0')
-                    {
-                        break;
-                    }
-                    end_num = start_num+1;
-                    while ((((*end_num >= '0') && (*end_num <= '9')) || (*end_num == '.')) && (*end_num != '\0'))
-                    {
-                        end_num++;
-                    }
-
-                    strncpy(substring_copy, start_num, end_num-start_num);
-                    substring_copy[end_num-start_num] = '\0';
-                    freecell_solver_user_set_a_star_weight(
-                        instance,
-                        a,
-                        atof(substring_copy)
-                        );
-                    /* Make sure that if the string terminated here -
-                     * we stop.
-                     * */
-                    if (*end_num == '\0')
-                    {
-                        break;
-                    }
-                    start_num=end_num+1;
-                }
-
-                free(substring_copy);
             }
         }
         break;
@@ -1430,9 +1496,9 @@ break;
         {
             PROCESS_OPT_ARG() ;
 
-            freecell_solver_user_limit_num_states_in_collection(
+            freecell_solver_user_limit_num_states_in_collection_long(
                 instance,
-                atoi((*arg))
+                atol((*arg))
             );
         }
         break;
@@ -1544,6 +1610,14 @@ break;
         }
         break;
 
+        case FCS_OPT_FLARES_ITERS_FACTOR: /* STRINGS=-fif|--flares-iters-factor; */
+        {
+            PROCESS_OPT_ARG() ;
+
+            freecell_solver_user_set_flares_iters_factor(instance, atof(*arg));
+        }
+        break;
+
         case FCS_OPT_OPTIMIZATION_TESTS_ORDER: /* STRINGS=-opt-to|--optimization-tests-order; */
         {
             int ret;
@@ -1559,7 +1633,7 @@ break;
 
             if (ret != 0)
             {
-                char * errstr = malloc(strlen(fcs_user_errstr)+500);
+                char * errstr = SMALLOC(errstr, strlen(fcs_user_errstr)+500);
                 sprintf(
                     errstr,
                     "Error in the optimization scan's tests' order!\n%s\n",
@@ -1592,7 +1666,7 @@ break;
             {
                 char * errstr;
 
-                errstr = malloc(strlen((*arg))+500);
+                errstr = SMALLOC(errstr, strlen((*arg))+500);
 
                 sprintf(errstr, "Unknown scans' synergy type \"%s\"!\n", (*arg));
                 *error_string = errstr;
@@ -1654,7 +1728,7 @@ break;
                     len += strlen(s);
                     len++;
 
-                    complete_path = malloc(len);
+                    complete_path = SMALLOC(complete_path, len);
                     sprintf(complete_path, "%s%s", opened_files_dir, s);
                     f = fopen(complete_path, "rt");
                     free(complete_path);
@@ -1676,9 +1750,7 @@ break;
                 /* If we still could not open it return an error */
                 if (f == NULL)
                 {
-                    char * err_str;
-
-                    err_str = malloc(strlen(s)+100);
+                    char * err_str = SMALLOC(err_str, strlen(s)+100);
                     sprintf(err_str,
                             "Could not open file \"%s\"!\nQuitting.\n",
                             s);
@@ -1689,7 +1761,7 @@ break;
                 }
                 fseek(f, 0, SEEK_END);
                 file_len = ftell(f);
-                buffer=malloc(file_len+1);
+                buffer= SMALLOC(buffer, file_len+1);
                 if (buffer == NULL)
                 {
                     *error_string = strdup("Could not allocate enough memory to parse the file. Quitting.\n");
@@ -1753,54 +1825,30 @@ break;
         {
             PROCESS_OPT_ARG() ;
 
+            int ret = freecell_solver_user_cmd_line_read_cmd_line_preset(
+                instance,
+                (*arg),
+                known_parameters,
+                error_string,
+                file_nesting_count,
+                opened_files_dir
+            );
+
+            if (ret == FCS_CMD_LINE_ERROR_IN_ARG)
             {
-                int status;
-                args_man_t * preset_args;
-                char * dir = NULL;
+                char * err_str = SMALLOC(err_str, strlen((*arg)) + 100);
+                sprintf(err_str, "Unable to load the \"%s\" configuration!\n", (*arg));
+                *error_string = err_str;
 
-                status = read_preset((*arg), &preset_args, &dir, NULL);
-                if (status != 0)
-                {
-                    char * err_str;
-                    err_str = malloc(strlen((*arg)) + 100);
-                    sprintf(err_str, "Unable to load the \"%s\" configuration!\n", (*arg));
-                    *error_string = err_str;
-
-                    RET_ERROR_IN_ARG() ;
-                }
-                else
-                {
-                    int ret;
-
-                    ret = freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
-                        instance,
-                        preset_args->argc,
-                        (freecell_solver_str_t *)(void *)(preset_args->argv),
-                        0,
-                        known_parameters,
-                        callback,
-                        callback_context,
-                        error_string,
-                        last_arg,
-                        ((file_nesting_count < 0) ? file_nesting_count : (file_nesting_count-1)),
-                        dir ? dir : opened_files_dir
-                        );
-
-                    if (dir)
-                    {
-                        free(dir);
-                    }
-                    fc_solve_args_man_free(preset_args);
-
-                    if (ret == FCS_CMD_LINE_UNRECOGNIZED_OPTION)
-                    {
-                        /* Do nothing - continue */
-                    }
-                    else if (ret != FCS_CMD_LINE_OK)
-                    {
-                        return ret;
-                    }
-                }
+                RET_ERROR_IN_ARG() ;
+            }
+            else if (ret == FCS_CMD_LINE_UNRECOGNIZED_OPTION)
+            {
+                /* Do nothing - continue */
+            }
+            else if (ret != FCS_CMD_LINE_OK)
+            {
+                return ret;
             }
         }
         break;
@@ -1837,7 +1885,7 @@ break;
 
                 if (ret != 0)
                 {
-                    char * errstr = malloc(strlen(fcs_user_errstr)+500);
+                    char * errstr = SMALLOC(errstr, strlen(fcs_user_errstr)+500);
                     sprintf(
                             errstr,
                             "Error in depth tests' order!\n%s\n",
@@ -1868,7 +1916,7 @@ break;
 
             if (ret != 0)
             {
-                char * errstr = malloc(strlen(fcs_user_errstr)+500);
+                char * errstr = SMALLOC(errstr, strlen(fcs_user_errstr)+500);
                 sprintf(
                     errstr,
                     "Error in the optimization scan's pruning!\n%s\n",
@@ -1894,6 +1942,27 @@ break;
         }
         break;
 
+        case FCS_OPT_FLARES_CHOICE: /* STRINGS=--flares-choice; */
+        {
+            PROCESS_OPT_ARG() ;
+
+            if (freecell_solver_user_set_flares_choice(
+                instance,
+                (*arg)
+            ) != 0)
+            {
+                char * errstr = SMALLOC(errstr, strlen(*arg)+500);
+                sprintf(
+                    errstr,
+                    "Unknown flares choice argument '%s'.\n",
+                    (*arg)
+                );
+                *error_string = errstr;
+
+                RET_ERROR_IN_ARG() ;
+            }
+        }
+        break;
 
         }
         /* OPT-PARSE-END */
