@@ -55,10 +55,11 @@ struct fcs_derived_state_struct
 {
     fcs_state_keyval_pair_t state;
     fcs_encoded_state_buffer_t key;
-    fcs_encoded_state_buffer_t parent;
+    fcs_dbm_record_t * parent;
     struct fcs_derived_state_struct * next;
-    fcs_bool_t is_reversible_move;
+    int core_irreversible_moves_count;
     fcs_fcc_move_t move;
+    int num_non_reversible_moves_including_prune;
     DECLARE_IND_BUF_T(indirect_stacks_buffer)
 };
 
@@ -69,6 +70,12 @@ typedef struct fcs_derived_state_struct fcs_derived_state_t;
 #define FREECELL2MOVE(idx) (idx+8)
 #define FOUND2MOVE(idx) ((idx)+8+4)
 
+#define CALC_SEQUENCES_ARE_BUILT_BY() ( \
+    (local_variant == FCS_DBM_VARIANT_BAKERS_DOZEN) \
+    ? FCS_SEQ_BUILT_BY_RANK \
+    : FCS_SEQ_BUILT_BY_ALTERNATE_COLOR \
+    )
+
 #ifdef INDIRECT_STACK_STATES
 
 #define COPY_INDIRECT_COLS() \
@@ -113,37 +120,59 @@ typedef struct fcs_derived_state_struct fcs_derived_state_t;
     COPY_INDIRECT_COLS() \
 }
 
-#define COMMIT_NEW_STATE(src, dest, is_reversible) \
+#define COMMIT_NEW_STATE_WITH_COUNT(src, dest, count) \
 { \
  \
-    ptr_new_state->parent = *(key); \
+    ptr_new_state->parent = parent_ptr; \
     ptr_new_state->move = MAKE_MOVE((src), (dest)); \
  \
-    ptr_new_state->is_reversible_move = (is_reversible); \
+    ptr_new_state->core_irreversible_moves_count = (count); \
     /* Finally, enqueue the new state. */ \
     ptr_new_state->next = (*derived_list); \
     (*derived_list) = ptr_new_state; \
  \
 }
 
+#define COMMIT_NEW_STATE(src, dest, is_reversible) \
+    COMMIT_NEW_STATE_WITH_COUNT(src, dest, ((is_reversible) ? 0 : 1))
+
+#ifdef FCS_FREECELL_ONLY
+#define SEQS_ARE_BUILT_BY_RANK() FALSE
+#else
+#define SEQS_ARE_BUILT_BY_RANK() (sequences_are_built_by == FCS_SEQ_BUILT_BY_RANK)
+#endif
+
 static GCC_INLINE int calc_foundation_to_put_card_on(
+        enum fcs_dbm_variant_type_t local_variant,
         fcs_state_t * my_ptr_state,
         fcs_card_t card
         )
 {
+#ifndef FCS_FREECELL_ONLY
+    /* needed by the macros. */
+    int sequences_are_built_by;
+#endif
     int deck;
 
+#ifndef FCS_FREECELL_ONLY
+    sequences_are_built_by = CALC_SEQUENCES_ARE_BUILT_BY();
+#endif
+
     for(deck=0;deck < INSTANCE_DECKS_NUM;deck++)
     {
-        if (fcs_foundation_value(*my_ptr_state, (deck<<2)+fcs_card_suit(card)) == fcs_card_card_num(card) - 1)
+        if (fcs_foundation_value(*my_ptr_state, (deck<<2)+fcs_card_suit(card)) == fcs_card_rank(card) - 1)
         {
             int other_deck_idx;
 
             for (other_deck_idx = 0 ; other_deck_idx < (INSTANCE_DECKS_NUM << 2) ; other_deck_idx++)
             {
                 if (fcs_foundation_value(*my_ptr_state, other_deck_idx)
-                        < fcs_card_card_num(card) - 2 -
-                        ((other_deck_idx&0x1) == (fcs_card_suit(card)&0x1))
+                        < fcs_card_rank(card) - 2 -
+                        (
+                            SEQS_ARE_BUILT_BY_RANK()
+                            ? 0
+                            : ((other_deck_idx&0x1) == (fcs_card_suit(card)&0x1))
+                        )
                    )
                 {
                     break;
@@ -186,8 +215,17 @@ static GCC_INLINE fcs_fcc_moves_list_item_t * fc_solve_fcc_alloc_moves_list_item
     return new_item;
 }
 
-/* Returns the additional number of cards moved to the foundation  */
+#define FROM_COL_IS_REVERSIBLE_MOVE() \
+                        ((cards_num <= 1) ? TRUE \
+                             : fcs_is_parent_card(card, fcs_col_get_card( \
+                                     col, cards_num-2) \
+                               ) \
+                        )
+
+
+/* Returns the number of amortized irreversible moves performed. */
 static GCC_INLINE int horne_prune(
+    enum fcs_dbm_variant_type_t local_variant,
     fcs_state_keyval_pair_t * init_state_kv_ptr,
     fcs_fcc_moves_seq_t * moves_seq,
     fcs_fcc_moves_seq_allocator_t * allocator
@@ -199,14 +237,23 @@ static GCC_INLINE int horne_prune(
     int dest_foundation;
     int num_cards_moved;
     fcs_card_t card;
+#ifndef FCS_FREECELL_ONLY
+    /* needed by the macros. */
+    int sequences_are_built_by;
+#endif
     /*  */
     fcs_fcc_move_t additional_moves[RANK_KING * 4 * DECKS_NUM];
     int count_moves_so_far = 0;
+    int count_additional_irrev_moves = 0;
+
+#ifndef FCS_FREECELL_ONLY
+    sequences_are_built_by = CALC_SEQUENCES_ARE_BUILT_BY();
+#endif
 
 #define the_state (init_state_kv_ptr->s)
     do {
         num_cards_moved = 0;
-        for( stack_idx=0 ; stack_idx < LOCAL_STACKS_NUM ; stack_idx++)
+        for ( stack_idx=0 ; stack_idx < LOCAL_STACKS_NUM ; stack_idx++ )
         {
             col = fcs_state_get_col(the_state, stack_idx);
             cards_num = fcs_col_len(col);
@@ -216,8 +263,12 @@ static GCC_INLINE int horne_prune(
                 card = fcs_col_get_card(col, cards_num-1);
 
                 if ((dest_foundation =
-                    calc_foundation_to_put_card_on(&the_state, card)) >= 0)
+                    calc_foundation_to_put_card_on(local_variant, &the_state, card)) >= 0)
                 {
+                    if (! FROM_COL_IS_REVERSIBLE_MOVE())
+                    {
+                        count_additional_irrev_moves++;
+                    }
                     /* We can safely move it. */
                     num_cards_moved++;
 
@@ -232,13 +283,13 @@ static GCC_INLINE int horne_prune(
         }
 
         /* Now check the same for the free cells */
-        for( fc=0 ; fc < LOCAL_FREECELLS_NUM ; fc++)
+        for ( fc=0 ; fc < LOCAL_FREECELLS_NUM ; fc++)
         {
             card = fcs_freecell_card(the_state, fc);
-            if (fcs_card_card_num(card) != 0)
+            if (fcs_card_is_valid(card))
             {
                 if ((dest_foundation =
-                    calc_foundation_to_put_card_on(&the_state, card)) >= 0)
+                    calc_foundation_to_put_card_on(local_variant, &the_state, card)) >= 0)
                 {
                     num_cards_moved++;
 
@@ -305,12 +356,14 @@ static GCC_INLINE int horne_prune(
         moves_seq->count += count_moves_so_far;
     }
 
-    return count_moves_so_far;
+    return count_moves_so_far + count_additional_irrev_moves;
 }
 
 static GCC_INLINE fcs_bool_t instance_solver_thread_calc_derived_states(
+    enum fcs_dbm_variant_type_t local_variant,
     fcs_state_keyval_pair_t * init_state_kv_ptr,
     fcs_encoded_state_buffer_t * key,
+    fcs_dbm_record_t * parent_ptr,
     fcs_derived_state_t * * derived_list,
     fcs_derived_state_t * * derived_list_recycle_bin,
     fcs_compact_allocator_t * derived_list_allocator,
@@ -334,7 +387,7 @@ static GCC_INLINE fcs_bool_t instance_solver_thread_calc_derived_states(
     int empty_stack_idx = -1;
 
 #ifndef FCS_FREECELL_ONLY
-    sequences_are_built_by = FCS_SEQ_BUILT_BY_ALTERNATE_COLOR;
+    sequences_are_built_by = CALC_SEQUENCES_ARE_BUILT_BY();
 #endif
 
 #define the_state (init_state_kv_ptr->s)
@@ -368,7 +421,7 @@ static GCC_INLINE fcs_bool_t instance_solver_thread_calc_derived_states(
             for (deck=0 ; deck < INSTANCE_DECKS_NUM ; deck++)
             {
 
-                if (fcs_foundation_value(the_state, deck*4+suit) == fcs_card_card_num(card) - 1)
+                if (fcs_foundation_value(the_state, deck*4+suit) == fcs_card_rank(card) - 1)
                 {
                     /* We can put it there */
                     BEGIN_NEW_STATE()
@@ -381,7 +434,8 @@ static GCC_INLINE fcs_bool_t instance_solver_thread_calc_derived_states(
 
                     fcs_increment_foundation(new_state, deck*4+suit);
 
-                    COMMIT_NEW_STATE(COL2MOVE(stack_idx), FOUND2MOVE(suit), FALSE)
+
+                    COMMIT_NEW_STATE_WITH_COUNT(COL2MOVE(stack_idx), FOUND2MOVE(suit), (FROM_COL_IS_REVERSIBLE_MOVE() ? 1 : 2))
                 }
             }
         }
@@ -392,16 +446,16 @@ static GCC_INLINE fcs_bool_t instance_solver_thread_calc_derived_states(
     }
 
 #define fc_idx stack_idx
-    /* Move freecell stack cards to foundations. */
+    /* Move freecell cards to foundations. */
     for (fc_idx=0 ; fc_idx < LOCAL_FREECELLS_NUM ; fc_idx++)
     {
         card = fcs_freecell_card(the_state, fc_idx);
         suit = fcs_card_suit(card);
-        if (fcs_card_card_num(card) != 0)
+        if (fcs_card_is_valid(card))
         {
             for(deck=0;deck<INSTANCE_DECKS_NUM;deck++)
             {
-                if (fcs_foundation_value(the_state, deck*4+suit) == fcs_card_card_num(card) - 1)
+                if (fcs_foundation_value(the_state, deck*4+suit) == fcs_card_rank(card) - 1)
                 {
                     BEGIN_NEW_STATE()
 
@@ -422,12 +476,15 @@ static GCC_INLINE fcs_bool_t instance_solver_thread_calc_derived_states(
         }
     }
 
+    int cards_num_min_limit =
+        ( (local_variant == FCS_DBM_VARIANT_BAKERS_DOZEN) ? 1 : 0 );
+
     /* Move stack card on top of a parent */
     for (stack_idx=0;stack_idx<LOCAL_STACKS_NUM;stack_idx++)
     {
         col = fcs_state_get_col(the_state, stack_idx);
         cards_num = fcs_col_len(col);
-        if (cards_num > 0)
+        if (cards_num > cards_num_min_limit)
         {
             card = fcs_col_get_card(col, cards_num-1);
 
@@ -459,12 +516,6 @@ static GCC_INLINE fcs_bool_t instance_solver_thread_calc_derived_states(
                             fcs_col_pop_top(new_src_col);
                             fcs_col_push_card(new_dest_col, card);
                         }
-#define FROM_COL_IS_REVERSIBLE_MOVE() \
-                        ((cards_num <= 1) ? TRUE \
-                             : fcs_is_parent_card(card, fcs_col_get_card( \
-                                     col, cards_num-2) \
-                               ) \
-                        )
 
                         COMMIT_NEW_STATE(
                             COL2MOVE(stack_idx), COL2MOVE(ds),
@@ -480,7 +531,7 @@ static GCC_INLINE fcs_bool_t instance_solver_thread_calc_derived_states(
     for (fc_idx=0 ; fc_idx < LOCAL_FREECELLS_NUM ; fc_idx++)
     {
         card = fcs_freecell_card(the_state, fc_idx);
-        if (fcs_card_card_num(card) != 0)
+        if (fcs_card_is_valid(card))
         {
             for (ds=0;ds<LOCAL_STACKS_NUM;ds++)
             {
@@ -515,7 +566,8 @@ static GCC_INLINE fcs_bool_t instance_solver_thread_calc_derived_states(
         }
     }
 
-    if (empty_stack_idx >= 0)
+    if ((local_variant != FCS_DBM_VARIANT_BAKERS_DOZEN)
+        && (empty_stack_idx >= 0))
     {
         /* Stack Card to Empty Stack */
         for(stack_idx=0;stack_idx<LOCAL_STACKS_NUM;stack_idx++)
@@ -555,7 +607,7 @@ static GCC_INLINE fcs_bool_t instance_solver_thread_calc_derived_states(
         for (fc_idx=0;fc_idx<LOCAL_FREECELLS_NUM;fc_idx++)
         {
             card = fcs_freecell_card(the_state, fc_idx);
-            if (fcs_card_card_num(card) == 0)
+            if (fcs_card_is_empty(card))
             {
                 continue;
             }
@@ -585,7 +637,7 @@ static GCC_INLINE fcs_bool_t instance_solver_thread_calc_derived_states(
         {
             col = fcs_state_get_col(the_state, stack_idx);
             cards_num = fcs_col_len(col);
-            if (cards_num > 0)
+            if (cards_num > cards_num_min_limit)
             {
                 card = fcs_col_get_card(col, cards_num-1);
                 /* Let's move it */
@@ -611,17 +663,27 @@ static GCC_INLINE fcs_bool_t instance_solver_thread_calc_derived_states(
     }
 #undef fc_idx
 
-    /* Perform Horne's Prune on all the states. */
-    if (perform_horne_prune)
+    /* Perform Horne's Prune on all the states,
+     * or just set their num irreversible moves counts.
+     * */
     {
         fcs_derived_state_t * derived_iter;
 
         for (derived_iter = (*derived_list);
-            derived_iter ;
-            derived_iter = derived_iter->next
-        )
+             derived_iter ;
+             derived_iter = derived_iter->next
+            )
         {
-            horne_prune(&(derived_iter->state), NULL, NULL);
+            derived_iter->num_non_reversible_moves_including_prune =
+            (
+                derived_iter->core_irreversible_moves_count
+                +
+                (
+                    perform_horne_prune
+                    ? horne_prune(local_variant, &(derived_iter->state), NULL, NULL)
+                    : 0
+                )
+            );
         }
     }
 
