@@ -83,11 +83,10 @@ static fc_solve_delta_stater_t * fc_solve_delta_stater_alloc(
 #endif
         )
 {
-    fc_solve_delta_stater_t * self;
     int col_idx;
     int max_num_cards;
 
-    self = malloc(sizeof(*self));
+    fc_solve_delta_stater_t * self = SMALLOC1(self);
 
 #ifndef FCS_FREECELL_ONLY
     self->sequences_are_built_by = sequences_are_built_by;
@@ -418,7 +417,7 @@ static void fc_solve_delta_stater_decode(
         foundations[i] = 14;
     }
 
-#define PROCESS_CARD(card) { if (fcs_card_card_num(card) < foundations[fcs_card_suit(card)]) { foundations[fcs_card_suit(card)] = fcs_card_card_num(card); } }
+#define PROCESS_CARD(card) { if (fcs_card_rank(card) < foundations[fcs_card_suit(card)]) { foundations[fcs_card_suit(card)] = fcs_card_rank(card); } }
 
     num_freecells = self->num_freecells;
     bits_per_orig_cards_in_column = self->bits_per_orig_cards_in_column;
@@ -483,16 +482,11 @@ static void fc_solve_delta_stater_decode(
 
             for ( i = 0 ; i < num_cards_in_seq ; i++)
             {
-                fcs_card_t new_card;
-                int suit_bit;
-
-                new_card = fc_solve_empty_card;
-                suit_bit = fc_solve_bit_reader_read(bit_r, 1);
-
-                fcs_card_set_num(new_card, fcs_card_card_num(last_card)-1);
-                fcs_card_set_suit(new_card,
+                int suit_bit = fc_solve_bit_reader_read(bit_r, 1);
+                fcs_card_t new_card = fcs_make_card(
+                    fcs_card_rank(last_card)-1,
                     ((suit_bit << 1) |
-                        ((fcs_card_suit(last_card) & 0x1) ^ 0x1)
+                     ((fcs_card_suit(last_card) & 0x1) ^ 0x1)
                     )
                 );
 
@@ -513,6 +507,7 @@ static void fc_solve_delta_stater_decode(
 }
 
 static GCC_INLINE void fc_solve_delta_stater_decode_into_state_proto(
+        enum fcs_dbm_variant_type_t local_variant,
         fc_solve_delta_stater_t * delta_stater,
         const fcs_uchar_t * const enc_state,
         fcs_state_keyval_pair_t * ret
@@ -604,6 +599,7 @@ static char * prepare_state_str(const char * proto)
  * The char * returned is malloc()ed and should be free()ed.
  */
 DLLEXPORT char * fc_solve_user_INTERNAL_delta_states_enc_and_dec(
+        enum fcs_dbm_variant_type_t local_variant,
         const char * init_state_str_proto,
         const char * derived_state_str_proto
         )
