@@ -6,7 +6,9 @@
 #include "fc_pro_iface_pos.h"
 
 #include "inline.h"
+#include "alloc_wrap.h"
 
+#if 0
 static const char * ranks_map = "0A23456789TJQK";
 
 static char * rank_to_string(int rank, char * buf)
@@ -25,7 +27,7 @@ static char * suit_to_string(int suit, char * buf)
     return buf;
 }
 
-static char * card_to_string(Card card, char * buf)
+static char * card_to_string(fcs_card_t card, char * buf)
 {
     rank_to_string(card&0x0F,buf);
     suit_to_string(card>>4,buf+1);
@@ -33,81 +35,14 @@ static char * card_to_string(Card card, char * buf)
     return buf;
 }
 
-char * fc_solve_fc_pro_position_to_string(Position * pos, int num_freecells)
-{
-    int a, stack;
-    char buffer[4000], temp[4][20];
-    char * s_end;
-
-    buffer[0] = '\0';
-    s_end = buffer;
-
-    for(a=0;a<4;a++)
-    {
-        if (pos->foundations[a] != 0)
-        {
-            break;
-        }
-    }
-    if (a < 4)
-    {
-        s_end += sprintf(s_end, "Foundations:");
-        for(a=0;a<4;a++)
-        {
-            if (pos->foundations[a] != 0)
-            {
-                suit_to_string(a, temp[0]);
-                rank_to_string(pos->foundations[a], temp[1]);
-                s_end += sprintf(
-                    s_end,
-                    " %s-%s", temp[0], temp[1]
-                    );
-            }
-        }
-        *s_end = '\n';
-        s_end++;
-    }
-    s_end += sprintf(s_end, "Freecells:");
-    for(a=0;a<num_freecells;a++)
-    {
-        if (pos->hold[a] == 0)
-        {
-            s_end += sprintf(s_end, " -");
-        }
-        else
-        {
-            s_end += sprintf(s_end, " %s", card_to_string(pos->hold[a], temp[0]));
-        }
-    }
-    *s_end = '\n';
-    s_end++;
-
-    for(stack=0;stack<8;stack++)
-    {
-        for(a=0;a<pos->tableau[stack].count;a++)
-        {
-            s_end +=
-                sprintf(
-                    s_end,
-                    "%s%s",
-                    ((a == 0)? "" : " "),
-                    card_to_string(pos->tableau[stack].cards[a], temp[0])
-                );
-        }
-        *s_end = '\n';
-        s_end++;
-    }
-    *s_end = '\0';
-
-    return strdup(buffer);
-}
+#endif
 
 static GCC_INLINE int Cvtf89(int fcn)
 {
     return (fcn >= 7) ? (fcn+3) : fcn;
 }
 
-char * moves_processed_render_move(fcs_extended_move_t move, char * string)
+char * fc_solve_moves_processed_render_move(fcs_extended_move_t move, char * string)
 {
     switch(fcs_move_get_type(move.move))
     {
@@ -174,66 +109,69 @@ char * moves_processed_render_move(fcs_extended_move_t move, char * string)
 }
 
 #define MOVES_PROCESSED_GROW_BY 32
-static void moves_processed_add_new_move(moves_processed_t * moves, fcs_extended_move_t new_move)
+static GCC_INLINE void moves_processed_add_new_move(fcs_moves_processed_t * const moves, const fcs_extended_move_t new_move)
 {
     if (! ((++moves->num_moves) & (MOVES_PROCESSED_GROW_BY - 1)))
     {
         moves->moves =
-            realloc(
-                moves->moves,
-                (
-                    sizeof(moves->moves[0])
-                    * (moves->num_moves + MOVES_PROCESSED_GROW_BY)
-                )
-            );
+            SREALLOC (moves->moves, moves->num_moves + MOVES_PROCESSED_GROW_BY);
     }
     moves->moves[moves->num_moves-1] = new_move;
 }
 
-moves_processed_t * moves_processed_gen(Position * orig, int num_freecells, void * instance)
+void fc_solve_moves_processed_gen(
+    fcs_moves_processed_t * const ret,
+    fcs_state_keyval_pair_t * const orig,
+    const int num_freecells,
+    const fcs_moves_sequence_t * const moves_seq
+)
 {
-    Position pos;
-    moves_processed_t * ret;
+    fcs_state_keyval_pair_t pos_proto;
+    DECLARE_IND_BUF_T(indirect_stacks_buffer)
+
+    fcs_kv_state_t orig_pass;
+    orig_pass.key = &(orig->s);
+    orig_pass.val = &(orig->info);
+
+    fcs_kv_state_t pos_pass;
+    pos_pass.key = &(pos_proto.s);
+    pos_pass.val = &(pos_proto.info);
+
+    fcs_duplicate_kv_state(&pos_pass, &orig_pass);
+    for (int i = 0 ; i < 8 ; i++)
+    {
+        fcs_copy_stack(pos_proto.s, pos_proto.info, i, indirect_stacks_buffer);
+    }
+
+#define pos (pos_proto.s)
     int virtual_stack_len[8];
 #ifndef NDEBUG
     int virtual_freecell_len[12];
 #endif
     int i, j, move_idx, num_back_end_moves;
-    fcs_move_t move, out_move;
+    fcs_move_t move, out_move, * next_move_ptr;
 
-    pos = *orig;
+    num_back_end_moves = moves_seq->num_moves;
+    next_move_ptr = moves_seq->moves;
 
-    num_back_end_moves = freecell_solver_user_get_moves_left(instance);
-
-    ret = malloc(sizeof(*ret));
     ret->num_moves = 0;
-    ret->moves = malloc(sizeof(ret->moves[0]) * MOVES_PROCESSED_GROW_BY);
+    ret->moves = SMALLOC(ret->moves, MOVES_PROCESSED_GROW_BY);
     ret->next_move_idx = 0;
 
     for(i=0;i<8;i++)
     {
-        virtual_stack_len[i] = orig->tableau[i].count;
+        fcs_cards_column_t col = fcs_state_get_col(pos, i);
+        virtual_stack_len[i] = fcs_col_len(col);
     }
 #ifndef NDEBUG
     for(i=0;i<num_freecells;i++)
     {
-        virtual_freecell_len[i] = (orig->hold[i] != 0) ? 1 : 0;
+        virtual_freecell_len[i] = (! fcs_freecell_is_empty(pos, i)) ? 1 : 0;
     }
 #endif
 
     for(move_idx=0; move_idx < num_back_end_moves ; move_idx ++)
     {
-#if 0
-        if (getenv("FCS_OUTPUT_INTERMEDIATE_POS"))
-        {
-            char * as_str;
-            as_str = fc_solve_fc_pro_position_to_string(&pos, num_freecells);
-            printf("state =\n<<<\n%s\n>>>\n\n", as_str);
-            free(as_str);
-        }
-#endif
-
-
         /*
          * Move safe cards to the foundations
          * */
@@ -244,32 +182,33 @@ moves_processed_t * moves_processed_gen(Position * orig, int num_freecells, void
                 /* Check the intermediate position validity */
                 char exists[4*13];
                 int rank, suit;
-                int fc, col, count, i;
-                Card card;
+                int fc, col, count;
+                fcs_card_t card;
 
                 memset(exists, '\0', sizeof(exists));
                 for (suit=0;suit<4;suit++)
                 {
-                    for(rank=1;rank<=pos.foundations[suit];rank++)
+                    for(rank=1;rank<=fcs_foundation_value(pos, suit);rank++)
                     {
                         exists[rank-1+suit*13] = 1;
                     }
                 }
                 for (col=0;col<8;col++)
                 {
-                    count = pos.tableau[col].count;
+                    fcs_cards_column_t col_col = fcs_state_get_col(pos, col);
+                    count = fcs_col_len(col_col);
                     for (i=0;i<count;i++)
                     {
-                        card = pos.tableau[col].cards[i];
-                        exists[(card & 0x0F)-1+(card >> 4)*13] = 1;
+                        card = fcs_col_get_card(col_col, i);
+                        exists[fcs_card_rank(card)-1+fcs_card_suit(card)*13] = 1;
                     }
                 }
                 for (fc=0;fc<num_freecells;fc++)
                 {
-                    card = pos.hold[fc];
-                    if (card != 0)
+                    card = fcs_freecell_card(pos, fc);
+                    if (! fcs_card_is_empty(card))
                     {
-                        exists[(card & 0x0F)-1+(card >> 4)*13] = 1;
+                        exists[fcs_card_rank(card)-1+fcs_card_suit(card)*13] = 1;
                     }
                 }
                 for (i=0;i<52;i++)
@@ -283,25 +222,43 @@ moves_processed_t * moves_processed_gen(Position * orig, int num_freecells, void
             }
 #endif
 
-            for(i=0;i<8;i++)
+#if 0
+    fcs_state_locs_struct_t locs;
+    fc_solve_init_locs(&locs);
+            printf("STATE=<<<\n%s\n>>>\n",
+                fc_solve_state_as_string(
+                    &pos,
+                    &pos_proto.info,
+                    &locs,
+                    4,
+                    8,
+                    1,
+                    TRUE,
+                    FALSE,
+                    TRUE
+                )
+            );
+#endif
+
+            for (i = 0 ; i < 8 ; i++)
             {
                 int rank, suit;
-                Card card;
+                fcs_card_t card;
 
-                if (pos.tableau[i].count > 0)
+                fcs_cards_column_t col = fcs_state_get_col(pos, i);
+                if (fcs_col_len(col) > 0)
                 {
-                    card = pos.tableau[i].cards[pos.tableau[i].count-1];
-                    rank = card & 0x0F;
-                    suit = card >> 4;
+                    card = fcs_col_get_card(col, fcs_col_len(col) - 1);
+                    rank = fcs_card_rank(card);
+                    suit = fcs_card_suit(card);
                     /* Check if we can safely move it */
-                    if ((pos.foundations[suit^0x1] >= rank-2) &&
-                        (pos.foundations[suit^0x1^0x2] >= rank-2) &&
-                        (pos.foundations[suit^0x2] >= rank-3) &&
-                        (pos.foundations[suit] == rank-1))
+                    if ((fcs_foundation_value(pos, suit^0x1) >= rank-2) &&
+                        (fcs_foundation_value(pos, suit^0x1^0x2) >= rank-2) &&
+                        (fcs_foundation_value(pos, suit^0x2) >= rank-3) &&
+                        (fcs_foundation_value(pos, suit) == rank-1))
                     {
-                        pos.foundations[suit]++;
-                        pos.tableau[i].count--;
-
+                        fcs_increment_foundation(pos, suit);
+                        fcs_col_pop_top(col);
                         /* An Automove. */
 
                         break;
@@ -315,21 +272,21 @@ moves_processed_t * moves_processed_gen(Position * orig, int num_freecells, void
             for(j=0;j<num_freecells;j++)
             {
                 int rank, suit;
-                Card card;
+                fcs_card_t card;
 
-                if (pos.hold[j] != 0)
+                if (! fcs_freecell_is_empty(pos, j))
                 {
-                    card = pos.hold[j];
-                    rank = card & 0x0F;
-                    suit = card >> 4;
+                    card = fcs_freecell_card(pos, j);
+                    rank = fcs_card_rank(card);
+                    suit = fcs_card_suit(card);
                     /* Check if we can safely move it */
-                    if ((pos.foundations[suit^0x1] >= rank-2) &&
-                        (pos.foundations[suit^0x1^0x2] >= rank-2) &&
-                        (pos.foundations[suit^0x2] >= rank-3) &&
-                        (pos.foundations[suit] == rank-1))
+                    if ((fcs_foundation_value(pos, suit^0x1) >= rank-2) &&
+                        (fcs_foundation_value(pos, suit^0x1^0x2) >= rank-2) &&
+                        (fcs_foundation_value(pos, suit^0x2) >= rank-3) &&
+                        (fcs_foundation_value(pos, suit) == rank-1))
                     {
-                        pos.foundations[suit]++;
-                        pos.hold[j] = 0;
+                        fcs_increment_foundation(pos, suit);
+                        fcs_empty_freecell(pos, j);
 
                         /* We've just done an auto-move */
                         break;
@@ -342,24 +299,23 @@ moves_processed_t * moves_processed_gen(Position * orig, int num_freecells, void
                 break;
             }
         }
-        freecell_solver_user_get_next_move(instance, &move);
+        move = *(next_move_ptr++);
 
         {
-            int src, dest, len;
-            Card card;
+            int src, dest;
+            fcs_card_t card;
             switch(fcs_move_get_type(move))
             {
                 case FCS_MOVE_TYPE_STACK_TO_FOUNDATION:
                     {
                         src = fcs_move_get_src_stack(move);
-                        assert(virtual_stack_len[src] >= pos.tableau[src].count);
-                        if (virtual_stack_len[src] == pos.tableau[src].count)
+                        fcs_cards_column_t col = fcs_state_get_col(pos, src);
+                        assert(virtual_stack_len[src] >= fcs_col_len(col));
+                        if (virtual_stack_len[src] == fcs_col_len(col))
                         {
-                            len = virtual_stack_len[src];
-                            card = pos.tableau[src].cards[len-1];
-                            pos.foundations[card >> 4]++;
+                            fcs_col_pop_card(col, card);
+                            fcs_increment_foundation(pos, fcs_card_suit(card));
                             virtual_stack_len[src]--;
-                            pos.tableau[src].count--;
                             {
                                 fcs_extended_move_t ext_move;
                                 ext_move.move = move;
@@ -380,7 +336,7 @@ moves_processed_t * moves_processed_gen(Position * orig, int num_freecells, void
                     {
                         src = fcs_move_get_src_freecell(move);
                         assert((virtual_freecell_len[src] == 1));
-                        if (pos.hold[src] == 0)
+                        if (fcs_freecell_is_empty(pos, src))
                         {
                             /* Do nothing */
                         }
@@ -394,8 +350,8 @@ moves_processed_t * moves_processed_gen(Position * orig, int num_freecells, void
 
                                 moves_processed_add_new_move(ret, ext_move);
                             }
-                            pos.foundations[pos.hold[src] >> 4]++;
-                            pos.hold[src] = 0;
+                            fcs_increment_foundation(pos, fcs_freecell_card_suit(pos, src));
+                            fcs_empty_freecell(pos, src);
                         }
 #ifndef NDEBUG
                         virtual_freecell_len[src] = 0;
@@ -408,7 +364,7 @@ moves_processed_t * moves_processed_gen(Position * orig, int num_freecells, void
                         src = fcs_move_get_src_freecell(move);
                         dest = fcs_move_get_dest_stack(move);
                         assert(virtual_freecell_len[src] == 1);
-                        if (pos.hold[src] == 0)
+                        if (fcs_freecell_is_empty(pos, src))
                         {
                             /* Do nothing */
                         }
@@ -422,8 +378,9 @@ moves_processed_t * moves_processed_gen(Position * orig, int num_freecells, void
 
                                 moves_processed_add_new_move(ret, ext_move);
                             }
-                            pos.tableau[dest].cards[pos.tableau[dest].count++] = pos.hold[src];
-                            pos.hold[src] = 0;
+                            fcs_cards_column_t dest_col = fcs_state_get_col(pos, dest);
+                            fcs_col_push_card(dest_col, fcs_freecell_card(pos, src));
+                            fcs_empty_freecell(pos, src);
                         }
 #ifndef NDEBUG
                         virtual_freecell_len[src] = 0;
@@ -437,8 +394,9 @@ moves_processed_t * moves_processed_gen(Position * orig, int num_freecells, void
                         src = fcs_move_get_src_stack(move);
                         dest = fcs_move_get_dest_freecell(move);
                         assert(virtual_stack_len[src] > 0);
-                        assert(pos.tableau[src].count <= virtual_stack_len[src]);
-                        if (pos.tableau[src].count < virtual_stack_len[src])
+                        fcs_cards_column_t col = fcs_state_get_col(pos, src);
+                        assert(fcs_col_len(col) <= virtual_stack_len[src]);
+                        if (fcs_col_len(col) < virtual_stack_len[src])
                         {
                             /* Do nothing */
                         }
@@ -453,7 +411,9 @@ moves_processed_t * moves_processed_gen(Position * orig, int num_freecells, void
 
                                 moves_processed_add_new_move(ret, ext_move);
                             }
-                            pos.hold[dest] = pos.tableau[src].cards[--pos.tableau[src].count];
+                            fcs_card_t temp_card;
+                            fcs_col_pop_card(col, temp_card);
+                            fcs_put_card_in_freecell(pos, dest, temp_card);
                         }
                         virtual_stack_len[src]--;
 #ifndef NDEBUG
@@ -464,18 +424,19 @@ moves_processed_t * moves_processed_gen(Position * orig, int num_freecells, void
 
                case FCS_MOVE_TYPE_STACK_TO_STACK:
                     {
-                        int num_cards, virt_num_cards;
-
                         src = fcs_move_get_src_stack(move);
                         dest = fcs_move_get_dest_stack(move);
-                        num_cards = fcs_move_get_num_cards_in_seq(move);
-                        assert(virtual_stack_len[src] >= pos.tableau[src].count);
-                        if (virtual_stack_len[src] > pos.tableau[src].count)
+                        int num_cards = fcs_move_get_num_cards_in_seq(move);
+                        fcs_cards_column_t src_col = fcs_state_get_col(pos, src);
+                        fcs_cards_column_t dest_col = fcs_state_get_col(pos, dest);
+                        int src_len = fcs_col_len(src_col);
+                        assert(virtual_stack_len[src] >= src_len);
+                        if (virtual_stack_len[src] > src_len)
                         {
 #ifndef min
 #define min(a,b) (((a)<(b))?(a):(b))
 #endif
-                            virt_num_cards = min((virtual_stack_len[src]-pos.tableau[src].count), num_cards);
+                            int virt_num_cards = min((virtual_stack_len[src]-src_len), num_cards);
 #undef min
                             virtual_stack_len[src] -= virt_num_cards;
                             virtual_stack_len[dest] += virt_num_cards;
@@ -490,17 +451,17 @@ moves_processed_t * moves_processed_gen(Position * orig, int num_freecells, void
                             {
                                 fcs_extended_move_t ext_move;
                                 ext_move.move = out_move;
-                                ext_move.to_empty_stack = (pos.tableau[dest].count == 0);
+                                ext_move.to_empty_stack = (fcs_col_len(dest_col) == 0);
                                 moves_processed_add_new_move(ret, ext_move);
                             }
                             for(i=0;i<num_cards;i++)
                             {
-                                pos.tableau[dest].cards[pos.tableau[dest].count+i] = pos.tableau[src].cards[pos.tableau[src].count-num_cards+i];
+                                fcs_col_push_col_card(dest_col, src_col, fcs_col_len(src_col)-num_cards+i);
+                            }
+                            for(i=0;i<num_cards;i++)
+                            {
+                                fcs_col_pop_top(src_col);
                             }
-                            pos.tableau[dest].count =
-                                (uchar)(pos.tableau[dest].count+num_cards);
-                            pos.tableau[src].count =
-                                (uchar)(pos.tableau[src].count - num_cards);
                             virtual_stack_len[dest] += num_cards;
                             virtual_stack_len[src] -= num_cards;
                         }
@@ -510,27 +471,6 @@ moves_processed_t * moves_processed_gen(Position * orig, int num_freecells, void
             }
         }
     }
-
-    return ret;
 }
 
-int moves_processed_get_moves_left(moves_processed_t * moves)
-{
-    return moves->num_moves- moves->next_move_idx;
-}
 
-int moves_processed_get_next_move(moves_processed_t * moves, fcs_extended_move_t * move)
-{
-    if (moves->next_move_idx == moves->num_moves)
-    {
-        return 1;
-    }
-    *move = moves->moves[moves->next_move_idx++];
-    return 0;
-}
-
-void moves_processed_free(moves_processed_t * moves)
-{
-    free(moves->moves);
-    free(moves);
-}
