@@ -37,43 +37,21 @@
 #include "fcs_cl.h"
 
 #include "fc_pro_iface_pos.h"
-#define FCS_WITHOUT_GET_BOARD
 #include "range_solvers_gen_ms_boards.h"
 #include "unused.h"
 #include "bool.h"
+#include "indirect_buffer.h"
+
 
 #define     SUIT(card)      ((card) % 4)
 #define     VALUE(card)     ((card) / 4)
 
-static GCC_INLINE void fc_pro_get_board(long gamenumber, Position * pos)
+static GCC_INLINE void fc_pro_get_board(long gamenumber, fcs_state_string_t state_string, fcs_state_keyval_pair_t * pos IND_BUF_T_PARAM(indirect_stacks_buffer))
 {
-    int  i, j;                /*  generic counters */
-    int  wLeft = 52;          /*  cards left to be chosen in shuffle */
-    CARD deck[52];            /* deck of 52 unique cards */
-    int col;
-    CARD card, suit;
-
-    memset(pos, '\0', sizeof(*pos));
-
-    /* shuffle cards */
-
-    for (i = 0; i < 52; i++)      /* put unique card in each deck loc. */
-    {
-        deck[i] = i;
-    }
-
-    for (i = 0; i < 52; i++)
-    {
-        j = microsoft_rand_rand(&gamenumber) % wLeft;
-        col = (i%8);
-        card = deck[j];
-        suit = SUIT(card);
-        pos->tableau[col].cards[pos->tableau[col].count++]
-            = (Card)((VALUE(card)+1)
-            + (((suit == 3) ? suit : ((suit+1)%3))<<4))
-            ;
-        deck[j] = deck[--wLeft];
-    }
+    get_board(gamenumber, state_string);
+    fc_solve_initial_user_state_to_c(
+        state_string, pos, 4, 8, 1, indirect_stacks_buffer
+    );
 }
 
 struct fc_solve_display_information_context_struct
@@ -96,21 +74,21 @@ typedef struct fc_solve_display_information_context_struct fc_solve_display_info
 
 static void my_iter_handler(
     void * user_instance,
-    int iter_num,
+    fcs_int_limit_t iter_num,
     int depth,
     void * ptr_state,
-    int parent_iter_num,
+    fcs_int_limit_t parent_iter_num,
     void * lp_context
     )
 {
     fc_solve_display_information_context_t * context;
     context = (fc_solve_display_information_context_t*)lp_context;
 
-    fprintf(stdout, "Iteration: %i\n", iter_num);
+    fprintf(stdout, "Iteration: %li\n", (long)iter_num);
     fprintf(stdout, "Depth: %i\n", depth);
     if (context->display_parent_iter_num)
     {
-        fprintf(stdout, "Parent Iteration: %i\n", parent_iter_num);
+        fprintf(stdout, "Parent Iteration: %li\n", (long)parent_iter_num);
     }
     fprintf(stdout, "\n");
 
@@ -158,7 +136,7 @@ static int cmd_line_callback(
 
     if ((!strcmp(argv[arg], "-i")) || (!strcmp(argv[arg], "--iter-output")))
     {
-        freecell_solver_user_set_iter_handler(
+        freecell_solver_user_set_iter_handler_long(
             instance,
             my_iter_handler,
             dc
@@ -292,21 +270,22 @@ static int read_int(FILE * f, int * dest)
 int main(int argc, char * argv[])
 {
     pack_item_t user;
-    /* char buffer[2048]; */
     int ret;
     int len;
     int board_num;
     int start_board, end_board, stop_at;
-    char * buffer;
+    const char * variant = "freecell";
     char temp_str[10];
     fcs_portable_time_t mytime;
+    char * buffer;
 
     fcs_int64_t total_num_iters = 0;
 
     char * error_string;
     int parser_ret;
+    fcs_bool_t variant_is_freecell;
 
-    int total_iterations_limit_per_board = -1;
+    fcs_int_limit_t total_iterations_limit_per_board = -1;
 
     char * binary_output_filename = NULL;
 
@@ -314,7 +293,8 @@ int main(int argc, char * argv[])
 
     int arg = 1, start_from_arg;
 
-    Position pos;
+    fcs_state_keyval_pair_t pos;
+    DECLARE_IND_BUF_T(indirect_stacks_buffer)
 
     if (argc < 4)
     {
@@ -328,7 +308,25 @@ int main(int argc, char * argv[])
 
     for (;arg < argc; arg++)
     {
-        if (!strcmp(argv[arg], "--binary-output-to"))
+        if (!strcmp(argv[arg], "--variant"))
+        {
+            arg++;
+            if (arg == argc)
+            {
+                fprintf(stderr, "--variant came without an argument!\n");
+                print_help();
+                exit(-1);
+            }
+            variant = argv[arg];
+
+            if (strlen(variant) > 50)
+            {
+                fprintf(stderr, "--variant's argument is too long!\n");
+                print_help();
+                exit(-1);
+            }
+        }
+        else if (!strcmp(argv[arg], "--binary-output-to"))
         {
             arg++;
             if (arg == argc)
@@ -348,7 +346,7 @@ int main(int argc, char * argv[])
                 print_help();
                 exit(-1);
             }
-            total_iterations_limit_per_board = atoi(argv[arg]);
+            total_iterations_limit_per_board = atol(argv[arg]);
         }
         else
         {
@@ -384,7 +382,7 @@ int main(int argc, char * argv[])
 
             print_int_wrapper(start_board);
             print_int_wrapper(end_board);
-            print_int_wrapper(total_iterations_limit_per_board);
+            print_int_wrapper(((int)total_iterations_limit_per_board));
         }
         else
         {
@@ -401,7 +399,11 @@ int main(int argc, char * argv[])
             }
             read_int_wrapper(start_board);
             read_int_wrapper(end_board);
-            read_int_wrapper(total_iterations_limit_per_board);
+            {
+                int val;
+                read_int_wrapper(val);
+                total_iterations_limit_per_board = (fcs_int_limit_t)val;
+            }
 
             fseek(in, 0, SEEK_END);
             file_len = ftell(in);
@@ -472,18 +474,44 @@ int main(int argc, char * argv[])
         return (-1);
     }
 
+    variant_is_freecell = (!strcmp(variant, "freecell"));
+    freecell_solver_user_apply_preset(user.instance, variant);
+
     ret = 0;
 
     for(board_num=start_board;board_num<=end_board;board_num++)
     {
-        fc_pro_get_board(board_num, &pos);
+#define BUF_SIZE 2000
+        if (variant_is_freecell)
+        {
+            buffer = calloc(BUF_SIZE, sizeof(buffer[0]));
+            fc_pro_get_board(board_num, buffer, &(pos)
+#ifdef INDIRECT_STACK_STATES
+                , indirect_stacks_buffer
+#endif
+                );
+        }
+        else
+        {
+            FILE * from_make_pysol;
+            char command[1000];
+            buffer = calloc(BUF_SIZE, sizeof(buffer[0]));
 
-        buffer = fc_solve_fc_pro_position_to_string(&pos, 4);
+            sprintf(command, "make_pysol_freecell_board.py -F -t %d %s",
+                    board_num,
+                    variant
+                   );
+
+            from_make_pysol = popen(command, "r");
+            fread(buffer, sizeof(buffer[0]), BUF_SIZE-1, from_make_pysol);
+            pclose(from_make_pysol);
+#undef BUF_SIZE
+        }
 #if 0
         printf("%s\n", buffer);
 #endif
 
-        freecell_solver_user_limit_iterations(user.instance, total_iterations_limit_per_board);
+        freecell_solver_user_limit_iterations_long(user.instance, total_iterations_limit_per_board);
 
         ret =
             freecell_solver_user_solve_board(
@@ -493,50 +521,68 @@ int main(int argc, char * argv[])
 
         free(buffer);
 
+        int num_iters;
+        int num_moves;
+        int num_fcpro_moves;
+        fcs_moves_processed_t fc_pro_moves;
+        fc_pro_moves.moves = NULL;
 
         if (ret == FCS_STATE_SUSPEND_PROCESS)
         {
             FCS_PRINT_INTRACTABLE_BOARD(mytime, board_num);
-            fflush(stdout);
 
-            print_int_wrapper(-1);
-            printf("%s", "[[Num FCS Moves]]=-1\n[[Num FCPro Moves]]=-1\n");
+            num_iters = num_moves = num_fcpro_moves = -1;
         }
         else if (ret == FCS_STATE_IS_NOT_SOLVEABLE)
         {
             FCS_PRINT_UNSOLVED_BOARD(mytime, board_num);
-            print_int_wrapper(-2);
-            printf("%s", "[[Num FCS Moves]]=-2\n[[Num FCPro Moves]]=-2\n");
+            num_iters = num_moves = num_fcpro_moves = -1;
         }
         else
         {
-            moves_processed_t * fc_pro_moves;
-            fcs_extended_move_t move;
+            num_iters = (int)freecell_solver_user_get_num_times_long(user.instance);
+            num_moves = freecell_solver_user_get_moves_left(user.instance);
 
-            print_int_wrapper(freecell_solver_user_get_num_times(user.instance));
-            printf("[[Num FCS Moves]]=%d\n",
-                    freecell_solver_user_get_moves_left(user.instance)
-                  );
-
-            fc_pro_moves = moves_processed_gen(&pos, 4, user.instance);
-
-            printf("[[Num FCPro Moves]]=%d\n[[Start]]\n",
-                    moves_processed_get_moves_left(fc_pro_moves)
-                  );
-
-            len = 0;
-            while (! moves_processed_get_next_move(fc_pro_moves, &move))
+            if (variant_is_freecell)
             {
-                moves_processed_render_move(move, temp_str);
-                printf("%s%c", temp_str,
-                        ((((++len) % 10) == 0) ? '\n' : ' ')
-                    );
-            }
-            moves_processed_free(fc_pro_moves);
-            printf("\n%s\n", "[[End]]");
-        }
+                fcs_moves_sequence_t moves_seq;
 
-        total_num_iters += freecell_solver_user_get_num_times(user.instance);
+                freecell_solver_user_get_moves_sequence(user.instance, &moves_seq);
+                fc_solve_moves_processed_gen(&fc_pro_moves, &pos, 4, &moves_seq);
+
+                num_fcpro_moves = fc_solve_moves_processed_get_moves_left(&fc_pro_moves);
+
+                if (moves_seq.moves)
+                {
+                    free (moves_seq.moves);
+                    moves_seq.moves = NULL;
+                }
+            }
+            else
+            {
+                num_fcpro_moves = num_moves;
+            }
+        }
+        print_int_wrapper(num_iters);
+        printf("[[Num Iters]]=%d\n[[Num FCS Moves]]=%d\n[[Num FCPro Moves]]=%d\n", num_iters, num_moves, num_fcpro_moves);
+
+        printf("%s\n", "[[Start]]");
+        if (fc_pro_moves.moves)
+        {
+            fcs_extended_move_t move;
+            len = 0;
+            while (! fc_solve_moves_processed_get_next_move(&fc_pro_moves, &move))
+            {
+                fc_solve_moves_processed_render_move(move, temp_str);
+                printf("%s%c", temp_str,
+                    ((((++len) % 10) == 0) ? '\n' : ' ')
+                );
+            }
+            fc_solve_moves_processed_free(&fc_pro_moves);
+        }
+        printf("\n%s\n", "[[End]]");
+        fflush(stdout);
+        total_num_iters += freecell_solver_user_get_num_times_long(user.instance);
 
         if (board_num % stop_at == 0)
         {
@@ -549,7 +595,6 @@ int main(int argc, char * argv[])
 
         }
 
-
         freecell_solver_user_recycle(user.instance);
     }
 
