@@ -115,6 +115,7 @@ static GCC_INLINE void fc_solve_fcc_release_moves_seq(
 }
 
 static void perform_FCC_brfs(
+    enum fcs_dbm_variant_type_t local_variant,
     /* The first state in the game, from which all states are encoded. */
     fcs_state_keyval_pair_t * init_state,
     /* The start state. */
@@ -171,6 +172,7 @@ static void perform_FCC_brfs(
     fcs_meta_compact_allocator_t * meta_alloc
 )
 {
+    void * tree_recycle_bin = NULL;
     fcs_dbm_queue_item_t * queue_head, * queue_tail, * queue_recycle_bin, * new_item, * extracted_item;
     fcs_compact_allocator_t queue_allocator, derived_list_allocator;
     fcs_derived_state_t * derived_list, * derived_list_recycle_bin,
@@ -209,7 +211,7 @@ static void perform_FCC_brfs(
 #endif
     );
 
-    traversed_states = fc_solve_kaz_tree_create(fc_solve_compare_encoded_states, NULL, meta_alloc);
+    traversed_states = fc_solve_kaz_tree_create(fc_solve_compare_encoded_states, NULL, meta_alloc, &tree_recycle_bin);
 #if 0
     found_new_start_points = fc_solve_kaz_tree_create(fc_solve_compare_encoded_states, NULL);
 #endif
@@ -247,29 +249,24 @@ static void perform_FCC_brfs(
         }
 
         /* Handle the min_by_sorting scan. */
-        if (cache_does_key_exist(
-            does_state_exist_in_any_FCC_cache,
-            &(extracted_item->key)
-        ))
-        {
-            *is_min_by_sorting_new = FALSE;
-            goto free_resources;
-        }
-        else
-        {
-            cache_insert(does_state_exist_in_any_FCC_cache, &(extracted_item->key), NULL, '\0');
-        }
-
-        if (! running_min_was_assigned)
+        if ((! running_min_was_assigned) ||
+            (memcmp(&(extracted_item->key), &running_min,
+                    sizeof(running_min)) < 0)
+        )
         {
             running_min_was_assigned = TRUE;
             running_min = extracted_item->key;
-        }
-        else
-        {
-            if (memcmp(&(extracted_item->key), &running_min, sizeof(running_min)) < 0)
+            if (cache_does_key_exist(
+                    does_state_exist_in_any_FCC_cache,
+                    &(running_min)
+                    ))
             {
-                running_min = extracted_item->key;
+                *is_min_by_sorting_new = FALSE;
+                goto free_resources;
+            }
+            else
+            {
+                cache_insert(does_state_exist_in_any_FCC_cache, &(running_min), NULL, '\0');
             }
         }
 
@@ -308,8 +305,10 @@ static void perform_FCC_brfs(
 #endif
 
         instance_solver_thread_calc_derived_states(
+            local_variant,
             &state,
             &(extracted_item->key),
+            NULL,
             &derived_list,
             &derived_list_recycle_bin,
             &derived_list_allocator,
@@ -341,12 +340,13 @@ static void perform_FCC_brfs(
                 derived_iter = next_derived_iter
         )
         {
-            fcs_bool_t is_reversible = derived_iter->is_reversible_move;
+            fcs_bool_t is_reversible = (derived_iter->core_irreversible_moves_count == 0);
             unsigned char extra_move;
 
 
             fcs_init_and_encode_state(
                 delta_stater,
+                local_variant,
                 &(derived_iter->state),
                 &(new_item->key)
             );
