@@ -37,17 +37,8 @@
 #include <limits.h>
 #include <signal.h>
 
-/*
- * Define FCS_DBM_USE_RWLOCK to use the pthread FCFS RWLock which appears
- * to improve CPU utilisations of the various worker threads and possibly
- * overall performance.
- * #define FCS_DBM_USE_RWLOCK 1 */
-
-#ifdef FCS_DBM_USE_RWLOCK
-#include <pthread/rwlock_fcfs.h>
-#endif
-
 #include "config.h"
+
 #undef FCS_RCS_STATES
 
 #include "bool.h"
@@ -56,10 +47,11 @@
 
 #include "dbm_calc_derived.h"
 
-#include "delta_states.c"
+#include "delta_states_any.h"
 
 #include "dbm_common.h"
 #include "dbm_solver.h"
+#include "dbm_move_to_string.h"
 
 #include "dbm_lru_cache.h"
 #include "fcc_brfs.h"
@@ -95,37 +87,19 @@ typedef struct {
     dict_t * does_min_by_absolute_depth_exist;
 } fcs_fcc_collection_by_depth;
 
-#define FCC_DEPTH (RANK_KING * 4 * DECKS_NUM * 2)
 typedef struct {
     int curr_depth;
     long max_num_elements_in_cache;
-    fcs_fcc_collection_by_depth FCCs_by_depth[FCC_DEPTH];
+    fcs_fcc_collection_by_depth FCCs_by_depth[MAX_FCC_DEPTH];
     /* No need to reset when we ascend to a new depth.
      * TODO : make sure the pointers to the encoded states
      * inside the cache are always valid.
      * TODO : might be easier to reset during every ascension.
      * */
     fcs_lru_cache_t cache;
+    void * tree_recycle_bin;
 } fcs_fcc_solver_state;
 
-static const pthread_mutex_t initial_mutex_constant =
-    PTHREAD_MUTEX_INITIALIZER
-    ;
-
-#ifdef FCS_DBM_USE_RWLOCK
-typedef pthread_rwlock_fcfs_t * fcs_lock_t;
-#define FCS_LOCK(lock) pthread_rwlock_fcfs_gain_write(lock)
-#define FCS_UNLOCK(lock) pthread_rwlock_fcfs_release(lock)
-#define FCS_INIT_LOCK(lock) ((lock) = pthread_rwlock_fcfs_alloc())
-#define FCS_DESTROY_LOCK(lock) pthread_rwlock_fcfs_destroy(lock)
-#else
-typedef pthread_mutex_t fcs_lock_t;
-#define FCS_LOCK(lock) pthread_mutex_lock(&(lock))
-#define FCS_UNLOCK(lock) pthread_mutex_unlock(&(lock))
-#define FCS_INIT_LOCK(lock) ((lock) = initial_mutex_constant)
-#define FCS_DESTROY_LOCK(lock) {}
-#endif
-
 typedef struct {
     fcs_fcc_solver_state solver_state;
     long count_num_processed;
@@ -135,16 +109,19 @@ typedef struct {
     FILE * out_fh;
     long num_FCCs_processed_for_depth;
     long num_unique_FCCs_for_depth;
+    enum fcs_dbm_variant_type_t variant;
 } fcs_dbm_solver_instance_t;
 
 static GCC_INLINE void instance_init(
     fcs_dbm_solver_instance_t * instance,
+    enum fcs_dbm_variant_type_t local_variant,
     long max_processed_positions_count,
     long positions_milestone_step,
     long FCCs_per_depth_milestone_step,
     FILE * out_fh
 )
 {
+    instance->variant = local_variant;
     instance->count_num_processed = 0;
     instance->max_processed_positions_count = max_processed_positions_count;
     instance->positions_milestone_step = positions_milestone_step;
@@ -185,16 +162,18 @@ static GCC_INLINE void init_solver_state(
         meta_alloc
     );
 
-    for (i = 0 ; i < FCC_DEPTH ; i++)
+    solver_state->tree_recycle_bin = NULL;
+
+    for (i = 0 ; i < MAX_FCC_DEPTH ; i++)
     {
         fcs_fcc_collection_by_depth * fcc = &(solver_state->FCCs_by_depth[i]);
         fcc->queue = NULL;
         fcc->queue_recycle_bin = NULL;
         fc_solve_compact_allocator_init( &(fcc->queue_allocator), meta_alloc );
         fcc->does_min_by_sorting_exist
-            = fc_solve_kaz_tree_create(fc_solve_compare_encoded_states, NULL, meta_alloc);
+            = fc_solve_kaz_tree_create(fc_solve_compare_encoded_states, NULL, meta_alloc, &(solver_state->tree_recycle_bin));
         fcc->does_min_by_absolute_depth_exist
-            = fc_solve_kaz_tree_create(fc_solve_compare_encoded_states, NULL, meta_alloc);
+            = fc_solve_kaz_tree_create(fc_solve_compare_encoded_states, NULL, meta_alloc, &(solver_state->tree_recycle_bin));
     }
 }
 
@@ -249,7 +228,7 @@ static GCC_INLINE void solver_state_free(
 )
 {
     int depth;
-    for (depth = 0; depth < FCC_DEPTH ; depth++)
+    for (depth = 0; depth < MAX_FCC_DEPTH ; depth++)
     {
         solver_state__free_dcc_depth(solver_state, depth, moves_list_allocator);
     }
@@ -296,6 +275,16 @@ static GCC_INLINE void instance_print_processed_FCCs(
     );
 }
 
+static GCC_INLINE void instance_print_reached(
+    fcs_dbm_solver_instance_t * instance
+    )
+{
+    instance_time_printf(
+        instance,
+        "Reached %li positions\n", instance->count_num_processed
+        );
+}
+
 static int instance_run_solver(
     fcs_dbm_solver_instance_t * instance,
     long max_num_elements_in_cache,
@@ -305,6 +294,7 @@ static int instance_run_solver(
     fcs_meta_compact_allocator_t * meta_alloc
 )
 {
+    enum fcs_dbm_variant_type_t local_variant;
     fc_solve_delta_stater_t * delta;
     fcs_encoded_state_buffer_t init_state_enc;
     fcs_fcc_solver_state * solver_state;
@@ -317,6 +307,7 @@ static int instance_run_solver(
     long FCCs_per_depth_milestone_step;
     fcs_fcc_moves_seq_t ret_moves_seq;
 
+    local_variant = instance->variant;
     /* Initialize the state. */
     solver_state = &(instance->solver_state);
     init_solver_state(solver_state, max_num_elements_in_cache, meta_alloc);
@@ -336,7 +327,7 @@ static int instance_run_solver(
     );
 
     fcs_init_and_encode_state(
-        delta, init_state, &init_state_enc
+        delta, local_variant, init_state, &init_state_enc
     );
 
     ret = FCC_IMPOSSIBLE;
@@ -381,7 +372,7 @@ static int instance_run_solver(
     /* Now: iterate over the depths and generate new states. */
     for (curr_depth=0
          ;
-         (ret == FCC_IMPOSSIBLE) && (curr_depth < FCC_DEPTH )
+         (ret == FCC_IMPOSSIBLE) && (curr_depth < MAX_FCC_DEPTH )
          ;
          solver_state->curr_depth = ++curr_depth,
          fcc_stage++
@@ -392,7 +383,7 @@ static int instance_run_solver(
         instance->num_unique_FCCs_for_depth = 0;
 
         do_next_fcc_start_points_exist
-            = fc_solve_kaz_tree_create(fc_solve_compare_encoded_states, NULL, meta_alloc);
+            = fc_solve_kaz_tree_create(fc_solve_compare_encoded_states, NULL, meta_alloc, &(solver_state->tree_recycle_bin));
 
         while ((ret == FCC_IMPOSSIBLE) && fcc_stage->queue)
         {
@@ -419,6 +410,7 @@ static int instance_run_solver(
 
             /* Now scan the new fcc */
             perform_FCC_brfs(
+                local_variant,
                 init_state,
                 fcc->min_by_absolute_depth,
                 &(fcc->moves_seq_to_min_by_absolute_depth),
@@ -451,10 +443,7 @@ static int instance_run_solver(
             {
                 if ((instance->count_num_processed += num_new_positions) >= next_count_num_processed_landmark)
                 {
-                    instance_time_printf(
-                        instance,
-                        "Reached %li positions\n", instance->count_num_processed
-                    );
+                    instance_print_reached(instance);
                     next_count_num_processed_landmark = instance->count_num_processed;
                     next_count_num_processed_landmark += (STEP - (next_count_num_processed_landmark % STEP));
                     if (instance->count_num_processed >=
@@ -524,6 +513,7 @@ static int instance_run_solver(
                 /*  TODO : convert horne_prune to fcs_fcc_moves_seq_t . */
                 num_additional_moves =
                     horne_prune(
+                        local_variant,
                         &state,
                         &(start_point_iter->moves_seq),
                         moves_list_allocator
@@ -560,6 +550,7 @@ static int instance_run_solver(
 
                 fcs_init_and_encode_state(
                     delta,
+                    local_variant,
                     &(state),
                     &enc_state
                     );
@@ -687,55 +678,19 @@ typedef struct {
 
 #define USER_STATE_SIZE 2000
 
-static const char * move_to_string(unsigned char move, char * move_buffer)
-{
-    int iter, inspect;
-    char * s;
-
-    s = move_buffer;
-
-    for (iter=0 ; iter < 2 ; iter++)
-    {
-        inspect = (move & 0xF);
-        move >>= 4;
-
-        if (inspect < 8)
-        {
-            s += sprintf(s, "Column %d", inspect);
-        }
-        else
-        {
-            inspect -= 8;
-            if (inspect < 4)
-            {
-                s += sprintf(s, "Freecell %d", inspect);
-            }
-            else
-            {
-                inspect -= 4;
-                s += sprintf(s, "Foundation %d", inspect);
-            }
-        }
-        if (iter == 0)
-        {
-            s += sprintf(s, "%s", " -> ");
-        }
-    }
-
-    return move_buffer;
-}
-
 static fcs_dbm_solver_instance_t * global_instance_ptr;
 
 static void command_signal_handler(int signal_num GCC_UNUSED)
 {
     instance_print_processed_FCCs( global_instance_ptr );
+    instance_print_reached( global_instance_ptr );
 }
 
 int main(int argc, char * argv[])
 {
     /* Temporarily #if'ed away until we finish working on instance_run_solver
      * */
+    enum fcs_dbm_variant_type_t local_variant;
     fcs_dbm_solver_instance_t instance;
     long caches_delta;
     long max_processed_positions_count = LONG_MAX;
@@ -755,6 +710,7 @@ int main(int argc, char * argv[])
     fcs_meta_compact_allocator_t meta_alloc;
     DECLARE_IND_BUF_T(init_indirect_stacks_buffer)
 
+    local_variant = FCS_DBM_VARIANT_2FC_FREECELL;
     caches_delta = 1000000;
     num_threads = 2;
     FCCs_per_depth_milestone_step = 10000;
@@ -866,6 +822,7 @@ int main(int argc, char * argv[])
 
     instance_init(
         &instance,
+        local_variant,
         max_processed_positions_count,
         positions_milestone_step,
         FCCs_per_depth_milestone_step,
@@ -896,7 +853,7 @@ int main(int argc, char * argv[])
     moves_list_allocator.allocator = &(moves_list_compact_alloc);
     init_moves_seq.moves_list = NULL;
     init_moves_seq.count = 0;
-    horne_prune(&init_state, &init_moves_seq, &moves_list_allocator);
+    horne_prune(local_variant, &init_state, &init_moves_seq, &moves_list_allocator);
 
 #ifndef WIN32
     signal(SIGUSR1, command_signal_handler);
