@@ -34,6 +34,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "alloc_wrap.h"
 #include "portable_int64.h"
 #include "portable_time.h"
 
@@ -70,21 +71,21 @@ typedef struct fc_solve_display_information_context_struct fc_solve_display_info
 
 static void my_iter_handler(
     void * user_instance,
-    int iter_num,
+    fcs_int_limit_t iter_num,
     int depth,
     void * ptr_state,
-    int parent_iter_num,
+    fcs_int_limit_t parent_iter_num,
     void * lp_context
     )
 {
     fc_solve_display_information_context_t * context;
     context = (fc_solve_display_information_context_t*)lp_context;
 
-    fprintf(stdout, "Iteration: %i\n", iter_num);
+    fprintf(stdout, "Iteration: %li\n", (long)iter_num);
     fprintf(stdout, "Depth: %i\n", depth);
     if (context->display_parent_iter_num)
     {
-        fprintf(stdout, "Parent Iteration: %i\n", parent_iter_num);
+        fprintf(stdout, "Parent Iteration: %li\n", (long)parent_iter_num);
     }
     fprintf(stdout, "\n");
 
@@ -133,7 +134,7 @@ static int cmd_line_callback(
 
     if ((!strcmp(argv[arg], "-i")) || (!strcmp(argv[arg], "--iter-output")))
     {
-        freecell_solver_user_set_iter_handler(
+        freecell_solver_user_set_iter_handler_long(
             instance,
             my_iter_handler,
             dc
@@ -195,8 +196,6 @@ static const char * known_parameters[] = {
 
 #define BINARY_OUTPUT_NUM_INTS 16
 
-#define print_int_wrapper(i) { }
-
 static void print_help(void)
 {
     printf("\n%s",
@@ -222,7 +221,7 @@ typedef struct {
     int stop_at;
 } context_t;
 
-static int total_iterations_limit_per_board = -1;
+static fcs_int_limit_t total_iterations_limit_per_board = -1;
 
 static fcs_int64_t total_num_iters = 0;
 
@@ -242,7 +241,7 @@ typedef struct
 
 typedef struct
 {
-    int num_iters;
+    fcs_int_limit_t num_iters;
     int num_finished_boards;
 } response_t;
 
@@ -275,7 +274,7 @@ static int worker_func(int idx, worker_t w, void * instance)
         {
             get_board(board_num, state_string);
 
-            freecell_solver_user_limit_iterations(instance, total_iterations_limit_per_board);
+            freecell_solver_user_limit_iterations_long(instance, total_iterations_limit_per_board);
 
             ret =
                 freecell_solver_user_solve_board(
@@ -287,7 +286,6 @@ static int worker_func(int idx, worker_t w, void * instance)
             {
                 FCS_PRINT_INTRACTABLE_BOARD(mytime, board_num);
                 fflush(stdout);
-                print_int_wrapper(-1);
             }
             else if (ret == FCS_STATE_FLARES_PLAN_ERROR)
             {
@@ -304,14 +302,9 @@ static int worker_func(int idx, worker_t w, void * instance)
             {
                 FCS_PRINT_UNSOLVED_BOARD(mytime, board_num);
                 fflush(stdout);
-                print_int_wrapper(-2);
-            }
-            else
-            {
-                print_int_wrapper(freecell_solver_user_get_num_times(user.instance));
             }
 
-            total_num_iters_temp += freecell_solver_user_get_num_times(instance);
+            total_num_iters_temp += freecell_solver_user_get_num_times_long(instance);
 
             /*  TODO : implement at the master. */
 #if 0
@@ -372,7 +365,7 @@ int main(int argc, char * argv[])
                 print_help();
                 exit(-1);
             }
-            total_iterations_limit_per_board = atoi(argv[arg]);
+            total_iterations_limit_per_board = (fcs_int_limit_t)atol(argv[arg]);
         }
         else if (!strcmp(argv[arg], "--num-workers"))
         {
@@ -451,7 +444,7 @@ int main(int argc, char * argv[])
         return -1;
     }
 
-    workers = malloc(sizeof(workers[0])*num_workers);
+    workers = SMALLOC(workers, num_workers);
 
     for ( idx = 0 ; idx < num_workers ; idx++)
     {
