@@ -56,6 +56,8 @@
 
 #include "inline.h"
 #include "likely.h"
+#include "count.h"
+#include "alloc_wrap.h"
 
 /*
     General use of this interface:
@@ -79,13 +81,11 @@
     a guideline for the user.
 */
 
-const double fc_solve_default_befs_weights[5] = {0.5,0,0.3,0,0.2};
-
-
+const double fc_solve_default_befs_weights[FCS_NUM_BEFS_WEIGHTS] = {0.5, 0, 0.3, 0, 0.2, 0};
 
 static GCC_INLINE void normalize_befs_weights(
-    fc_solve_soft_thread_t * soft_thread
-    )
+    fc_solve_state_weighting_t * const weighting
+)
 {
     /* Normalize the Best-First-Search Weights, so the sum of all of them would be 1. */
 
@@ -94,11 +94,11 @@ static GCC_INLINE void normalize_befs_weights(
     int weight_idx;
 
     sum = 0;
-#define my_befs_weights soft_thread->method_specific.befs.meth.befs.befs_weights
+#define my_befs_weights weighting->befs_weights
     for (
         weight_idx = 0
             ;
-        weight_idx < (sizeof(my_befs_weights)/sizeof(my_befs_weights[0]))
+        weight_idx < COUNT(my_befs_weights)
             ;
         weight_idx++
     )
@@ -113,7 +113,7 @@ static GCC_INLINE void normalize_befs_weights(
     {
         sum = 1;
     }
-    for(weight_idx=0;weight_idx<(sizeof(my_befs_weights)/sizeof(my_befs_weights[0]));weight_idx++)
+    for (weight_idx=0 ; weight_idx < COUNT(my_befs_weights) ; weight_idx++)
     {
         my_befs_weights[weight_idx] /= sum;
     }
@@ -121,7 +121,7 @@ static GCC_INLINE void normalize_befs_weights(
 
 static GCC_INLINE void soft_thread_clean_soft_dfs(
     fc_solve_soft_thread_t * soft_thread
-    )
+)
 {
     int max_depth;
     int dfs_max_depth;
@@ -129,16 +129,16 @@ static GCC_INLINE void soft_thread_clean_soft_dfs(
     /* Check if a Soft-DFS-type scan was called in the first place */
     if (!(
             FC_SOLVE_IS_DFS(soft_thread)
-        && soft_thread->method_specific.soft_dfs.soft_dfs_info
-       ))
+        && DFS_VAR(soft_thread, soft_dfs_info)
+    ))
     {
         /* If not - do nothing */
         return;
     }
 
-    soft_dfs_info = soft_thread->method_specific.soft_dfs.soft_dfs_info;
-    max_depth = soft_thread->method_specific.soft_dfs.depth;
-    dfs_max_depth = soft_thread->method_specific.soft_dfs.dfs_max_depth;
+    soft_dfs_info = DFS_VAR(soft_thread, soft_dfs_info);
+    max_depth = DFS_VAR(soft_thread, depth);
+    dfs_max_depth = DFS_VAR(soft_thread, dfs_max_depth);
     /* De-allocate the Soft-DFS specific stacks */
     {
         int depth;
@@ -162,9 +162,9 @@ static GCC_INLINE void soft_thread_clean_soft_dfs(
 
         free(soft_dfs_info);
 
-        soft_thread->method_specific.soft_dfs.soft_dfs_info = NULL;
+        DFS_VAR(soft_thread, soft_dfs_info) = NULL;
 
-        soft_thread->method_specific.soft_dfs.dfs_max_depth = 0;
+        DFS_VAR(soft_thread, dfs_max_depth) = 0;
 
     }
 }
@@ -177,7 +177,7 @@ extern void fc_solve_free_soft_thread_by_depth_test_array(fc_solve_soft_thread_t
          depth_idx < soft_thread->by_depth_tests_order.num ;
          depth_idx++)
     {
-        free (soft_thread->by_depth_tests_order.by_depth_tests[depth_idx].tests_order.tests);
+        fc_solve_free_tests_order(&(soft_thread->by_depth_tests_order.by_depth_tests[depth_idx].tests_order));
     }
 
     soft_thread->by_depth_tests_order.num = 0;
@@ -189,15 +189,15 @@ extern void fc_solve_free_soft_thread_by_depth_test_array(fc_solve_soft_thread_t
 }
 
 static GCC_INLINE void free_instance_soft_thread_callback(
-        fc_solve_soft_thread_t * soft_thread
-        )
+    fc_solve_soft_thread_t * soft_thread
+)
 {
     fcs_bool_t is_scan_befs_or_bfs = FALSE;
     switch (soft_thread->method)
     {
         case FCS_METHOD_A_STAR:
             fc_solve_PQueueFree(
-                    &(soft_thread->method_specific.befs.meth.befs.pqueue)
+                    &(BEFS_VAR(soft_thread, pqueue))
             );
             /* Fall-through. */
         case FCS_METHOD_BFS:
@@ -216,66 +216,55 @@ static GCC_INLINE void free_instance_soft_thread_callback(
     }
 }
 
+static GCC_INLINE void accumulate_tests_by_ptr(
+    int * tests_order,
+    fcs_tests_order_t * st_tests_order
+)
+{
+
+    for ( int i=0 ; i < st_tests_order->num_groups ; i++)
+    {
+        for (int j=0; j < st_tests_order->groups[i].num ; j++)
+        {
+            *tests_order |= (1 << (st_tests_order->groups[i].tests[j]));
+        }
+    }
+}
+
 static GCC_INLINE void accumulate_tests_order(
     fc_solve_soft_thread_t * soft_thread,
     void * context
-    )
+)
 {
-    int * tests_order = (int *)context;
-    int i;
-    fcs_tests_order_t * st_tests_order;
-
-    st_tests_order = &(soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order);
-
-    for ( i=0 ; i < st_tests_order->num ; i++)
-    {
-        *tests_order |= (1 << (st_tests_order->tests[i] & FCS_TEST_ORDER_NO_FLAGS_MASK));
-    }
-
-    return;
+    accumulate_tests_by_ptr((int *)context, &(soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order));
 }
 
 static GCC_INLINE void determine_scan_completeness(
     fc_solve_soft_thread_t * soft_thread,
     void * global_tests_order
-    )
+)
 {
     int tests_order = 0;
-    int i;
-    fcs_tests_order_t * st_tests_order;
 
-    st_tests_order = &(soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order);
+    accumulate_tests_by_ptr(&tests_order, &(soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order));
 
-    for ( i=0 ; i < st_tests_order->num ; i++ )
-    {
-        tests_order |= (1 << (st_tests_order->tests[i] & FCS_TEST_ORDER_NO_FLAGS_MASK));
-    }
     STRUCT_SET_FLAG_TO(soft_thread, FCS_SOFT_THREAD_IS_A_COMPLETE_SCAN,
         (tests_order == *(int *)global_tests_order)
     );
 }
 
-enum
-{
-    FOREACH_SOFT_THREAD_CLEAN_SOFT_DFS,
-    FOREACH_SOFT_THREAD_FREE_INSTANCE,
-    FOREACH_SOFT_THREAD_ACCUM_TESTS_ORDER,
-    FOREACH_SOFT_THREAD_DETERMINE_SCAN_COMPLETENESS
-};
 
-static void foreach_soft_thread(
+void fc_solve_foreach_soft_thread(
     fc_solve_instance_t * instance,
     int callback_choice,
     void * context
-    )
+)
 {
     int ht_idx;
     fc_solve_hard_thread_t * hard_thread;
 
     for(ht_idx = 0 ; ht_idx<=instance->num_hard_threads; ht_idx++)
     {
-        ST_LOOP_DECLARE_VARS();
-
         if (ht_idx < instance->num_hard_threads)
         {
             hard_thread = &(instance->hard_threads[ht_idx]);
@@ -294,103 +283,70 @@ static void foreach_soft_thread(
             switch (callback_choice)
             {
                 case FOREACH_SOFT_THREAD_CLEAN_SOFT_DFS:
-                    soft_thread_clean_soft_dfs(
-                        soft_thread
-                    );
-                    break;
+                    soft_thread_clean_soft_dfs( soft_thread );
+                break;
 
                 case FOREACH_SOFT_THREAD_FREE_INSTANCE:
-                    free_instance_soft_thread_callback(
-                        soft_thread
-                    );
-                    break;
+                    free_instance_soft_thread_callback( soft_thread );
+                break;
 
                 case FOREACH_SOFT_THREAD_ACCUM_TESTS_ORDER:
-                    accumulate_tests_order(
-                        soft_thread,
-                        context
-                    );
-                    break;
+                    accumulate_tests_order( soft_thread, context );
+                break;
 
                 case FOREACH_SOFT_THREAD_DETERMINE_SCAN_COMPLETENESS:
-                    determine_scan_completeness(
-                        soft_thread,
-                        context
-                    );
-                    break;
+                    determine_scan_completeness( soft_thread, context );
+                break;
             }
         }
     }
 }
 
-
-
-
-static GCC_INLINE void clean_soft_dfs(
-        fc_solve_instance_t * instance
-        )
+static GCC_INLINE void clean_soft_dfs( fc_solve_instance_t * instance )
 {
-    foreach_soft_thread(instance, FOREACH_SOFT_THREAD_CLEAN_SOFT_DFS, NULL);
+    fc_solve_foreach_soft_thread(instance, FOREACH_SOFT_THREAD_CLEAN_SOFT_DFS, NULL);
 }
 
 
 static GCC_INLINE void init_soft_thread(
-        fc_solve_hard_thread_t * hard_thread,
-        fc_solve_soft_thread_t * soft_thread
-        )
+    fc_solve_hard_thread_t * hard_thread,
+    fc_solve_soft_thread_t * soft_thread
+)
 {
     soft_thread->hard_thread = hard_thread;
 
     soft_thread->id = (hard_thread->instance->next_soft_thread_id)++;
 
-    soft_thread->method_specific.soft_dfs.dfs_max_depth = 0;
+    DFS_VAR(soft_thread, dfs_max_depth) = 0;
 
     soft_thread->by_depth_tests_order.num = 1;
     soft_thread->by_depth_tests_order.by_depth_tests =
-        malloc(sizeof(soft_thread->by_depth_tests_order.by_depth_tests[0]));
+        SMALLOC1(soft_thread->by_depth_tests_order.by_depth_tests);
 
     soft_thread->by_depth_tests_order.by_depth_tests[0].max_depth = INT_MAX;
-    soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order.num = 0;
+    soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order.num_groups = 0;
     soft_thread->by_depth_tests_order.by_depth_tests[0]
-        .tests_order.tests = NULL;
+        .tests_order.groups = NULL;
 
     /* Initialize all the Soft-DFS stacks to NULL */
-    soft_thread->method_specific.soft_dfs.soft_dfs_info = NULL;
+    DFS_VAR(soft_thread, soft_dfs_info) = NULL;
 
-    soft_thread->method_specific.soft_dfs.tests_by_depth_array.num_units = 0;
-    soft_thread->method_specific.soft_dfs.tests_by_depth_array.by_depth_units = NULL;
+    DFS_VAR(soft_thread, tests_by_depth_array).num_units = 0;
+    DFS_VAR(soft_thread, tests_by_depth_array).by_depth_units = NULL;
 
-    soft_thread->method_specific.befs.tests_list = NULL;
+    BEFS_M_VAR(soft_thread, tests_list) = NULL;
 
     /* The default solving method */
     soft_thread->method = FCS_METHOD_SOFT_DFS;
+    soft_thread->super_method_type = FCS_SUPER_METHOD_DFS;
 
-    soft_thread->method_specific.befs.befs_positions_by_rank = NULL;
+    BEFS_M_VAR(soft_thread, befs_positions_by_rank) = NULL;
 
-    soft_thread->method_specific.soft_dfs.rand_seed = 24;
+    DFS_VAR(soft_thread, rand_seed) = 24;
 
     soft_thread->num_times_step = NUM_TIMES_STEP;
 
-#if 0
-    {
-        char * no_use;
-        fc_solve_apply_tests_order(soft_thread, "[01][23456789]", &no_use);
-    }
-#else
-    soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order.num =
-        soft_thread->hard_thread->instance->instance_tests_order.num;
-    /* Bound the maximal number up to the next product of
-     * TESTS_ORDER_GROW_BY .
-     * */
-    soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order.tests =
-        malloc(sizeof(soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order.tests[0]) *
-            ((soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order.num & (~(TESTS_ORDER_GROW_BY - 1)))+TESTS_ORDER_GROW_BY)
-        );
-    memcpy(soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order.tests,
-        soft_thread->hard_thread->instance->instance_tests_order.tests,
-        sizeof(soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order.tests[0]) * soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order.num
-        );
-#endif
+    soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order = tests_order_dup(&(soft_thread->hard_thread->instance->instance_tests_order));
 
     fc_solve_reset_soft_thread(soft_thread);
 
@@ -400,9 +356,9 @@ static GCC_INLINE void init_soft_thread(
 }
 
 void fc_solve_instance__init_hard_thread(
-        fc_solve_instance_t * instance,
-        fc_solve_hard_thread_t * hard_thread
-        )
+    fc_solve_instance_t * const instance,
+    fc_solve_hard_thread_t * const hard_thread
+)
 {
     hard_thread->instance = instance;
 
@@ -422,7 +378,7 @@ void fc_solve_instance__init_hard_thread(
     hard_thread->prelude_idx = 0;
 
     fc_solve_reset_hard_thread(hard_thread);
-    fc_solve_compact_allocator_init(&(hard_thread->allocator));
+    fc_solve_compact_allocator_init(&(hard_thread->allocator), instance->meta_alloc);
 
     fcs_move_stack_init(hard_thread->reusable_move_stack);
 }
@@ -436,11 +392,11 @@ void fc_solve_instance__init_hard_thread(
     Afterwards fc_solve_init_instance() should be called in order
     to really prepare it for solving.
   */
-fc_solve_instance_t * fc_solve_alloc_instance(void)
+fc_solve_instance_t * fc_solve_alloc_instance(fcs_meta_compact_allocator_t * const meta_alloc)
 {
-    fc_solve_instance_t * instance;
+    fc_solve_instance_t * instance = SMALLOC1(instance);
 
-    instance = malloc(sizeof(fc_solve_instance_t));
+    instance->meta_alloc = meta_alloc;
 
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_INDIRECT)
     instance->num_indirect_prev_states = 0;
@@ -459,16 +415,16 @@ fc_solve_instance_t * fc_solve_alloc_instance(void)
     instance->trim_states_in_collection_from = -1;
     instance->effective_trim_states_in_collection_from = LONG_MAX;
 
-    instance->instance_tests_order.num = 0;
-    instance->instance_tests_order.tests = NULL;
+    instance->instance_tests_order.num_groups = 0;
+    instance->instance_tests_order.groups = NULL;
 
     instance->list_of_vacant_states = NULL;
 
 
     STRUCT_CLEAR_FLAG(instance, FCS_RUNTIME_OPT_TESTS_ORDER_WAS_SET );
 
-    instance->opt_tests_order.num = 0;
-    instance->opt_tests_order.tests = NULL;
+    instance->opt_tests_order.num_groups = 0;
+    instance->opt_tests_order.groups = NULL;
 
     instance->num_hard_threads = 0;
     instance->hard_threads = NULL;
@@ -522,59 +478,7 @@ fc_solve_instance_t * fc_solve_alloc_instance(void)
 
 #undef DEFAULT_MAX_NUM_ELEMENTS_IN_CACHE
 
-static GCC_INLINE void free_instance_hard_thread_callback(fc_solve_hard_thread_t * hard_thread)
-{
-    if (likely(hard_thread->prelude_as_string))
-    {
-        free (hard_thread->prelude_as_string);
-    }
-    if (likely(hard_thread->prelude))
-    {
-        free (hard_thread->prelude);
-    }
-    fcs_move_stack_static_destroy(hard_thread->reusable_move_stack);
 
-    free(hard_thread->soft_threads);
-
-    if (likely(hard_thread->allocator.packs))
-    {
-        fc_solve_compact_allocator_finish(&(hard_thread->allocator));
-        hard_thread->allocator.packs = NULL;
-    }
-}
-
-/*
-    This function is the last function that should be called in the
-    sequence of operations on instance, and it is meant for de-allocating
-    whatever memory was allocated by alloc_instance().
-  */
-void fc_solve_free_instance(fc_solve_instance_t * instance)
-{
-    HT_LOOP_DECLARE_VARS();
-
-    foreach_soft_thread(instance, FOREACH_SOFT_THREAD_FREE_INSTANCE, NULL);
-
-    HT_LOOP_START()
-    {
-        free_instance_hard_thread_callback(hard_thread);
-    }
-
-    free(instance->hard_threads);
-    if (instance->optimization_thread)
-    {
-        free_instance_hard_thread_callback(instance->optimization_thread);
-        free(instance->optimization_thread);
-    }
-
-    free(instance->instance_tests_order.tests);
-
-    if (STRUCT_QUERY_FLAG(instance, FCS_RUNTIME_OPT_TESTS_ORDER_WAS_SET))
-    {
-        free(instance->opt_tests_order.tests);
-    }
-
-    free(instance);
-}
 
 
 
@@ -586,24 +490,18 @@ enum
 };
 
 static GCC_INLINE int compile_prelude(
-    fc_solve_hard_thread_t * hard_thread
-    )
+    fc_solve_hard_thread_t * const hard_thread
+)
 {
-    char * p_quota, * p_scan, * p;
-    char * string;
-    int last_one = 0;
+    fcs_bool_t last_one = FALSE;
     int num_items = 0;
-    fcs_prelude_item_t * prelude;
-    ST_LOOP_DECLARE_VARS();
+    fcs_prelude_item_t * prelude = NULL;
 
-    prelude = NULL;
-    string = hard_thread->prelude_as_string;
-
-    p = string;
+    const char * p  = hard_thread->prelude_as_string;
 
     while (! last_one)
     {
-        p_quota = p;
+        const int p_quota = atoi(p);
         while((*p) && isdigit(*p))
         {
             p++;
@@ -613,27 +511,27 @@ static GCC_INLINE int compile_prelude(
             free(prelude);
             return FCS_COMPILE_PRELUDE_NO_AT_SIGN;
         }
-        *p = '\0';
         p++;
-        p_scan = p;
+        const char * const p_scan = p;
         while((*p) && ((*p) != ','))
         {
             p++;
         }
         if ((*p) == '\0')
         {
-            last_one = 1;
+            last_one = TRUE;
         }
-        *p = '\0';
+        char * const p_scan_copy = strndup(p_scan, p-p_scan);
         p++;
 
         ST_LOOP_START()
         {
-            if (soft_thread->name && (!strcmp(soft_thread->name, p_scan)))
+            if (soft_thread->name && (!strcmp(soft_thread->name, p_scan_copy)))
             {
                 break;
             }
         }
+        free (p_scan_copy);
         if (ST_LOOP_FINISHED())
         {
             free(prelude);
@@ -642,14 +540,14 @@ static GCC_INLINE int compile_prelude(
 #define PRELUDE_GROW_BY 16
         if (! (num_items & (PRELUDE_GROW_BY-1)))
         {
-            prelude = realloc(prelude, sizeof(prelude[0]) * (num_items+PRELUDE_GROW_BY));
+            prelude = SREALLOC(prelude, num_items+PRELUDE_GROW_BY);
         }
         prelude[num_items].scan_idx = ST_LOOP_INDEX();
-        prelude[num_items].quota = atoi(p_quota);
+        prelude[num_items].quota = p_quota;
         num_items++;
     }
 
-    hard_thread->prelude = realloc(prelude, sizeof(prelude[0]) * num_items);
+    hard_thread->prelude = SREALLOC(prelude, num_items);
     hard_thread->prelude_num_items = num_items;
     hard_thread->prelude_idx = 0;
 
@@ -659,14 +557,15 @@ static GCC_INLINE int compile_prelude(
 
 void fc_solve_init_instance(fc_solve_instance_t * instance)
 {
-    HT_LOOP_DECLARE_VARS();
-
     /* Initialize the state packs */
     HT_LOOP_START()
     {
         if (hard_thread->prelude_as_string)
         {
-            compile_prelude(hard_thread);
+            if (!hard_thread->prelude)
+            {
+                compile_prelude(hard_thread);
+            }
         }
         hard_thread->num_times_left_for_soft_thread =
             hard_thread->soft_threads[0].num_times_step;
@@ -674,20 +573,19 @@ void fc_solve_init_instance(fc_solve_instance_t * instance)
 
     {
         int total_tests = 0;
-        foreach_soft_thread(
+        fc_solve_foreach_soft_thread(
             instance,
             FOREACH_SOFT_THREAD_ACCUM_TESTS_ORDER,
             &total_tests
         );
-        foreach_soft_thread(
+        fc_solve_foreach_soft_thread(
             instance,
             FOREACH_SOFT_THREAD_DETERMINE_SCAN_COMPLETENESS,
             &total_tests
         );
         if (!STRUCT_QUERY_FLAG(
                 instance, FCS_RUNTIME_OPT_TESTS_ORDER_WAS_SET
-            )
-        )
+        ))
         {
             /*
              *
@@ -695,20 +593,28 @@ void fc_solve_init_instance(fc_solve_instance_t * instance)
              * to a valid tests order.
              *
              * */
-            int bit_idx, num_tests = 0;
-            int * tests = malloc(sizeof(total_tests)*8*sizeof(tests[0]));
+            int num_tests = 0;
+            int * tests = SMALLOC(tests, sizeof(total_tests)*8);
 
-            for(bit_idx=0; total_tests != 0; bit_idx++, total_tests >>= 1)
+            for (int bit_idx=0 ;
+                total_tests != 0 ;
+                bit_idx++, total_tests >>= 1)
             {
                 if ((total_tests & 0x1) != 0)
                 {
                     tests[num_tests++] = bit_idx;
                 }
             }
-            tests = realloc(tests, num_tests*sizeof(tests[0]));
-            instance->opt_tests_order.tests = tests;
-            instance->opt_tests_order.num =
+            tests = SREALLOC(tests,
+                ((num_tests & (~(TESTS_ORDER_GROW_BY - 1)))+TESTS_ORDER_GROW_BY)
+            );
+            instance->opt_tests_order.num_groups = 1;
+            instance->opt_tests_order.groups =
+                SMALLOC( instance->opt_tests_order.groups, TESTS_ORDER_GROW_BY);
+            instance->opt_tests_order.groups[0].tests = tests;
+            instance->opt_tests_order.groups[0].num =
                 num_tests;
+            instance->opt_tests_order.groups[0].shuffling_type = FCS_NO_SHUFFLING;
             STRUCT_TURN_ON_FLAG(instance, FCS_RUNTIME_OPT_TESTS_ORDER_WAS_SET);
         }
     }
@@ -720,13 +626,14 @@ void fc_solve_init_instance(fc_solve_instance_t * instance)
 #if (FCS_RCS_CACHE_STORAGE == FCS_RCS_CACHE_STORAGE_JUDY)
         cache->states_values_to_keys_map = ((Pvoid_t) NULL);
 #elif (FCS_RCS_CACHE_STORAGE == FCS_RCS_CACHE_STORAGE_KAZ_TREE)
-        cache->kaz_tree = fc_solve_kaz_tree_create(fc_solve_compare_lru_cache_keys, NULL);
+        cache->kaz_tree = fc_solve_kaz_tree_create(fc_solve_compare_lru_cache_keys, NULL, instance->meta_alloc);
 #else
 #error Unknown FCS_RCS_CACHE_STORAGE
 #endif
 
         fc_solve_compact_allocator_init(
-            &(cache->states_values_to_keys_allocator)
+            &(cache->states_values_to_keys_allocator),
+            instance->meta_alloc
         );
         cache->lowest_pri = NULL;
         cache->highest_pri = NULL;
@@ -752,6 +659,7 @@ void fc_solve_init_instance(fc_solve_instance_t * instance)
             || \
        (FCS_STACK_STORAGE == FCS_STACK_STORAGE_LIBREDBLACK_TREE) \
     )
+
 static int fcs_stack_compare_for_comparison_with_context(
     const void * v_s1,
     const void * v_s2,
@@ -759,8 +667,7 @@ static int fcs_stack_compare_for_comparison_with_context(
     const
 #endif
     void * context GCC_UNUSED
-
-    )
+)
 {
     return fc_solve_stack_compare_for_comparison(v_s1, v_s2);
 }
@@ -772,14 +679,14 @@ static int fcs_stack_compare_for_comparison_with_context(
 /* A hash calculation function for use in glib's hash */
 static guint fc_solve_glib_hash_stack_hash_function (
     gconstpointer key
-    )
+)
 {
     guint hash_value_int;
     /* Calculate the hash value for the stack */
     /* This hash function was ripped from the Perl source code.
      * (It is not derived work however). */
     const char * s_ptr = (char*)key;
-    const char * s_end = s_ptr+fcs_col_len((fcs_card_t *)key)+1;
+    const char * const s_end = s_ptr+fcs_col_len((fcs_card_t *)key)+1;
     hash_value_int = 0;
     while (s_ptr < s_end)
     {
@@ -809,17 +716,8 @@ static gint fc_solve_glib_hash_stack_compare (
  * */
 extern void fc_solve_trace_solution(
     fc_solve_instance_t * instance
-    )
+)
 {
-    /*
-        Trace the solution.
-    */
-    fcs_collectible_state_t * s1;
-    int move_idx;
-    fcs_move_stack_t * stack;
-    fcs_internal_move_t * moves;
-    fcs_move_stack_t * solution_moves_ptr;
-
     fcs_internal_move_t canonize_move = fc_solve_empty_move;
     fcs_int_move_set_type(canonize_move, FCS_MOVE_TYPE_CANONIZE);
 
@@ -831,9 +729,9 @@ extern void fc_solve_trace_solution(
 
     fcs_move_stack_init(instance->solution_moves);
 
-    s1 = instance->final_state;
+    fcs_collectible_state_t * s1 = instance->final_state;
 
-    solution_moves_ptr = &(instance->solution_moves);
+    fcs_move_stack_t * const solution_moves_ptr = &(instance->solution_moves);
     /* Retrace the step from the current state to its parents */
     while (FCS_S_PARENT(s1) != NULL)
     {
@@ -846,10 +744,9 @@ extern void fc_solve_trace_solution(
 
         /* Merge the move stack */
         {
-
-            stack = FCS_S_MOVES_TO_PARENT(s1);
-            moves = stack->moves;
-            for(move_idx=stack->num_moves-1;move_idx>=0;move_idx--)
+            const fcs_move_stack_t * const stack = FCS_S_MOVES_TO_PARENT(s1);
+            const fcs_internal_move_t * const moves = stack->moves;
+            for (int move_idx=stack->num_moves-1 ; move_idx >= 0 ; move_idx--)
             {
                 fcs_move_stack_push(solution_moves_ptr, moves[move_idx]);
             }
@@ -869,18 +766,17 @@ extern void fc_solve_trace_solution(
 
 static GCC_INLINE fcs_state_t * rcs_states_get_state(
     fc_solve_instance_t * instance,
-    fcs_collectible_state_t * state)
+    fcs_collectible_state_t * state
+)
 {
-    return
-        (
-         (state == instance->tree_new_state)
-         ? instance->tree_new_state_key
-         : fc_solve_lookup_state_key_from_val(
-             instance,
-             state
-             )
-        );
-
+    return (
+            (state == instance->tree_new_state)
+            ? instance->tree_new_state_key
+            : fc_solve_lookup_state_key_from_val(
+                instance,
+                state
+            )
+    );
 }
 
 static int fc_solve_rcs_states_compare(const void * void_a, const void * void_b, void * param)
@@ -913,7 +809,7 @@ static int fc_solve_rcs_states_compare(const void * void_a, const void * void_b,
 void fc_solve_start_instance_process_with_board(
     fc_solve_instance_t * instance,
     fcs_state_keyval_pair_t * init_state
-    )
+)
 {
     fcs_state_keyval_pair_t * state_copy_ptr;
 
@@ -964,21 +860,21 @@ void fc_solve_start_instance_process_with_board(
     /* Initialize the data structure that will manage the state collection */
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_LIBREDBLACK_TREE)
     instance->tree = rbinit(
-            fc_solve_state_extra_info_compare_with_context,
-            NULL
-            );
+        fc_solve_state_extra_info_compare_with_context,
+        NULL
+    );
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_LIBAVL2_TREE)
 
     instance->tree = fcs_libavl2_states_tree_create(
-            STATE_STORAGE_TREE_COMPARE(),
-            STATE_STORAGE_TREE_CONTEXT(),
-            NULL
-            );
+        STATE_STORAGE_TREE_COMPARE(),
+        STATE_STORAGE_TREE_CONTEXT(),
+        NULL
+    );
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_KAZ_TREE)
 
     instance->tree = fc_solve_kaz_tree_create(
-            STATE_STORAGE_TREE_COMPARE(), STATE_STORAGE_TREE_CONTEXT()
-            );
+        STATE_STORAGE_TREE_COMPARE(), STATE_STORAGE_TREE_CONTEXT()
+    );
 
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_GLIB_TREE)
     instance->tree = g_tree_new(fc_solve_state_compare);
@@ -988,22 +884,23 @@ void fc_solve_start_instance_process_with_board(
     instance->hash = g_hash_table_new(
         fc_solve_hash_function,
         fc_solve_state_compare_equal
-        );
+    );
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_INTERNAL_HASH)
-     fc_solve_hash_init(
-            &(instance->hash),
+    fc_solve_hash_init(
+        instance->meta_alloc,
+        &(instance->hash),
 #ifdef FCS_INLINED_HASH_COMPARISON
-            FCS_INLINED_HASH__STATES
+        FCS_INLINED_HASH__STATES
 #else
 #ifdef FCS_WITH_CONTEXT_VARIABLE
-            fc_solve_state_compare_with_context,
+        fc_solve_state_compare_with_context,
 
-            NULL
+        NULL
 #else
-            fc_solve_state_compare
+        fc_solve_state_compare
 #endif
 #endif
-       );
+    );
 #ifdef FCS_RCS_STATES
      instance->hash.instance = instance;
 #endif
@@ -1026,36 +923,37 @@ void fc_solve_start_instance_process_with_board(
        collection */
 #if FCS_STACK_STORAGE == FCS_STACK_STORAGE_INTERNAL_HASH
     fc_solve_hash_init(
-            &(instance->stacks_hash ),
+        instance->meta_alloc,
+        &(instance->stacks_hash ),
 #ifdef FCS_INLINED_HASH_COMPARISON
-            FCS_INLINED_HASH__COLUMNS
+        FCS_INLINED_HASH__COLUMNS
 #else
 #ifdef FCS_WITH_CONTEXT_VARIABLE
-            fcs_stack_compare_for_comparison_with_context,
-            NULL
+        fcs_stack_compare_for_comparison_with_context,
+        NULL
 #else
-            fc_solve_stack_compare_for_comparison
+        fc_solve_stack_compare_for_comparison
 #endif
 #endif
-        );
+    );
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_LIBAVL2_TREE)
     instance->stacks_tree = fcs_libavl2_stacks_tree_create(
-            fcs_stack_compare_for_comparison_with_context,
-            NULL,
-            NULL
-            );
+        fcs_stack_compare_for_comparison_with_context,
+        NULL,
+        NULL
+    );
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_LIBREDBLACK_TREE)
     instance->stacks_tree = rbinit(
         fcs_stack_compare_for_comparison_with_context,
         NULL
-        );
+    );
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_GLIB_TREE)
     instance->stacks_tree = g_tree_new(fc_solve_stack_compare_for_comparison);
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_GLIB_HASH)
     instance->stacks_hash = g_hash_table_new(
         fc_solve_glib_hash_stack_hash_function,
         fc_solve_glib_hash_stack_compare
-        );
+    );
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_GOOGLE_DENSE_HASH)
     instance->stacks_hash = fc_solve_columns_google_hash_new();
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_JUDY)
@@ -1077,7 +975,7 @@ void fc_solve_start_instance_process_with_board(
         NULL,
         NULL,
         &(instance->db)
-        );
+    );
 #endif
 
     {
@@ -1091,14 +989,12 @@ void fc_solve_start_instance_process_with_board(
             instance->hard_threads,
             &pass,
             &no_use
-            );
+        );
 
     }
 
     instance->current_hard_thread = instance->hard_threads;
     {
-        HT_LOOP_DECLARE_VARS();
-
         HT_LOOP_START()
         {
             if (hard_thread->prelude != NULL)
@@ -1130,9 +1026,7 @@ void fc_solve_start_instance_process_with_board(
     This function should be called after the user has retrieved the
     results generated by the scan as it will destroy them.
   */
-void fc_solve_finish_instance(
-    fc_solve_instance_t * instance
-    )
+void fc_solve_finish_instance( fc_solve_instance_t * instance )
 {
     /* De-allocate the state collection */
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_LIBREDBLACK_TREE)
@@ -1224,7 +1118,7 @@ void fc_solve_finish_instance(
 
 fc_solve_soft_thread_t * fc_solve_new_soft_thread(
     fc_solve_hard_thread_t * hard_thread
-    )
+)
 {
     fc_solve_soft_thread_t * ret;
 
@@ -1235,12 +1129,12 @@ fc_solve_soft_thread_t * fc_solve_new_soft_thread(
         return NULL;
     }
 
-    hard_thread->soft_threads = realloc(hard_thread->soft_threads, sizeof(hard_thread->soft_threads[0])*(hard_thread->num_soft_threads+1));
-
+    hard_thread->soft_threads =
+        SREALLOC(hard_thread->soft_threads, hard_thread->num_soft_threads+1);
     init_soft_thread(
-            hard_thread,
-            (ret = &(hard_thread->soft_threads[hard_thread->num_soft_threads]))
-            );
+        hard_thread,
+        (ret = &(hard_thread->soft_threads[hard_thread->num_soft_threads]))
+    );
 
     hard_thread->num_soft_threads++;
 
