@@ -34,13 +34,19 @@
 extern "C" {
 #endif
 
+#include <math.h>
+
 #include "config.h"
 #include "state.h"
 #include "move.h"
 #include "fcs_enums.h"
 
+#include "rate_state.h"
 #include "inline.h"
 #include "unused.h"
+#include "fcs_limit.h"
+#include "likely.h"
+#include "count.h"
 
 #include "indirect_buffer.h"
 #include "rand.h"
@@ -108,7 +114,7 @@ extern "C" {
 
 #include "pqueue.h"
 
-#include "alloc.h"
+#include "meta_alloc.h"
 
 /*
  * This is a linked list item that is used to implement a queue for the BFS
@@ -149,50 +155,75 @@ struct fc_solve_soft_thread_struct;
 
 
 typedef void (*fc_solve_solve_for_state_test_t)(
-        struct fc_solve_soft_thread_struct *,
-        fcs_kv_state_t *,
-        fcs_derived_states_list_t *
-        );
+    struct fc_solve_soft_thread_struct *,
+    fcs_kv_state_t *,
+    fcs_derived_states_list_t *
+);
 
 typedef struct fc_solve_hard_thread_struct fc_solve_hard_thread_t;
 
 #include "move_funcs_maps.h"
 
 /* HT_LOOP == hard threads' loop - macros to abstract it. */
-#define HT_LOOP_DECLARE_VARS() \
-    fc_solve_hard_thread_t * hard_thread, * end_hard_thread
-
 #define HT_LOOP_START() \
-    for ( end_hard_thread = ( \
-              (hard_thread = instance->hard_threads) \
-              + instance->num_hard_threads \
-          ) ; \
+    fc_solve_hard_thread_t * hard_thread = instance->hard_threads; \
+    fc_solve_hard_thread_t * const end_hard_thread = hard_thread+instance->num_hard_threads; \
+    for ( ; \
          hard_thread < end_hard_thread ;  \
          hard_thread++ \
-         )
+    )
 
 /* ST_LOOP == soft threads' loop - macros to abstract it. */
-#define ST_LOOP_DECLARE_VARS() \
-    fc_solve_soft_thread_t * soft_thread, * end_soft_thread
-
 #define ST_LOOP_START() \
-    for ( end_soft_thread = ( \
-              (soft_thread = hard_thread->soft_threads) \
-              + hard_thread->num_soft_threads \
-          ) ; \
+    fc_solve_soft_thread_t * soft_thread = hard_thread->soft_threads; \
+    fc_solve_soft_thread_t * const end_soft_thread = soft_thread + hard_thread->num_soft_threads; \
+    for ( ; \
          soft_thread < end_soft_thread ;  \
          soft_thread++ \
-         )
+    )
 
 #define ST_LOOP_FINISHED() (soft_thread == end_soft_thread)
 
 #define ST_LOOP_INDEX() (soft_thread - hard_thread->soft_threads)
 
 #define TESTS_ORDER_GROW_BY 16
+
+typedef struct
+{
+    fcs_bool_t should_go_over_stacks;
+    double max_sequence_move_factor,
+           cards_under_sequences_factor,
+           seqs_over_renegade_cards_factor,
+           depth_factor,
+           num_cards_not_on_parents_factor;
+
+    double num_cards_out_lookup_table[14];
+    /*
+     * The BeFS weights of the different BeFS tests. Those
+     * weights determine the commulative priority of the state.
+     * */
+    double befs_weights[FCS_NUM_BEFS_WEIGHTS];
+} fc_solve_state_weighting_t;
+
+typedef enum
+{
+    FCS_NO_SHUFFLING,
+    FCS_RAND,
+    FCS_WEIGHTING,
+} fcs_tests_group_type_t;
+
 typedef struct
 {
     int num;
     int * tests;
+    fcs_tests_group_type_t shuffling_type;
+    fc_solve_state_weighting_t weighting;
+} fcs_tests_order_group_t;
+
+typedef struct
+{
+    int num_groups;
+    fcs_tests_order_group_t * groups;
 } fcs_tests_order_t;
 
 typedef struct
@@ -218,18 +249,12 @@ typedef struct {
      * decks_num can be 1 or 2 .
      * */
 
-#define DECLARE_GAME_PARAMS() \
-    fcs_game_type_params_t game_params
-
 #define SET_INSTANCE_GAME_PARAMS(inst) \
-    game_params = (inst)->game_params
+    const fcs_game_type_params_t game_params = (inst)->game_params
 
 #define SET_GAME_PARAMS() \
     SET_INSTANCE_GAME_PARAMS(instance)
 
-#define DECLARE_AND_SET_GAME_PARAMS() \
-    fcs_game_type_params_t game_params = instance->game_params
-
 #ifndef HARD_CODED_NUM_FREECELLS
     fcs_game_limit_t freecells_num;
 #define INSTANCE_FREECELLS_NUM (instance->game_params.freecells_num)
@@ -365,7 +390,7 @@ typedef struct
 #error unknown FCS_RCS_CACHE_STORAGE
 #endif
     fcs_compact_allocator_t states_values_to_keys_allocator;
-    long count_elements_in_cache, max_num_elements_in_cache;
+    fcs_int_limit_t count_elements_in_cache, max_num_elements_in_cache;
 
     fcs_cache_key_info_t * lowest_pri, * highest_pri;
 
@@ -377,13 +402,13 @@ typedef struct
 typedef void * fcs_instance_debug_iter_output_context_t;
 
 typedef void (*fcs_instance_debug_iter_output_func_t)(
-        fcs_instance_debug_iter_output_context_t,
-        int iter_num,
-        int depth,
-        void * instance,
-        fcs_kv_state_t * state,
-        int parent_iter_num
-        );
+    fcs_instance_debug_iter_output_context_t,
+    fcs_int_limit_t iter_num,
+    int depth,
+    void * instance,
+    fcs_kv_state_t * state,
+    fcs_int_limit_t parent_iter_num
+);
 
 
 struct fc_solve_instance_struct
@@ -399,7 +424,7 @@ struct fc_solve_instance_struct
      * size is calculated based on the number.
      * */
     fcs_standalone_state_ptrs_t * indirect_prev_states;
-    int num_indirect_prev_states;
+    fcs_int_limit_t num_indirect_prev_states;
 #endif
 
     /*
@@ -411,7 +436,7 @@ struct fc_solve_instance_struct
 
     /* The number of states that were checked by the solving algorithm.
      * Badly named, should be renamed to num_iters or num_checked_states */
-    int num_times;
+    fcs_int_limit_t num_times;
 
     /*
      * Like max_num_times only defaults to MAX_INT if below zero so it will
@@ -419,8 +444,8 @@ struct fc_solve_instance_struct
      *
      * Normally should be used instead.
      * */
-    int effective_max_num_times, effective_max_num_states_in_collection;
-    long effective_trim_states_in_collection_from;
+    fcs_int_limit_t effective_max_num_times, effective_max_num_states_in_collection;
+    fcs_int_limit_t effective_trim_states_in_collection_from;
     /*
      * tree is the balanced binary tree that is used to store and index
      * the checked states.
@@ -490,12 +515,12 @@ struct fc_solve_instance_struct
      *
      * It gives a rough estimate of the memory occupied by the instance.
      * */
-    int active_num_states_in_collection, num_states_in_collection;
+    fcs_int_limit_t active_num_states_in_collection, num_states_in_collection;
 
     /*
      * A limit to the above.
      * */
-    int max_num_states_in_collection;
+    fcs_int_limit_t max_num_states_in_collection;
 
     int num_hard_threads;
     struct fc_solve_hard_thread_struct * hard_threads;
@@ -546,8 +571,8 @@ struct fc_solve_instance_struct
      *
      * */
     int max_depth;
-    int max_num_times;
-    long trim_states_in_collection_from;
+    fcs_int_limit_t max_num_times;
+    fcs_int_limit_t trim_states_in_collection_from;
 
     /*
      * The debug_iter_output variables provide a programmer programmable way
@@ -588,6 +613,10 @@ struct fc_solve_instance_struct
      * */
     fcs_move_stack_t solution_moves;
 
+    /*
+     * The meta allocator - see meta_alloc.h.
+     * */
+    fcs_meta_compact_allocator_t * meta_alloc;
 };
 
 typedef struct fc_solve_instance_struct fc_solve_instance_t;
@@ -616,21 +645,21 @@ struct fc_solve_hard_thread_struct
      *
      * Thus, the soft thread switching should be done based on this variable
      * */
-    int num_times;
+    fcs_int_limit_t num_times;
 
     /*
      * The maximal limit for num_times.
      * */
-    int max_num_times;
+    fcs_int_limit_t max_num_times;
 
-    int num_times_step;
+    fcs_int_limit_t num_times_step;
 
     /*
      * This is the number of iterations that still have to be done for
      * soft_threads[st_idx]. It is reset to (st_idx+1)->num_times_step
      * when st_idx is incremented.
      * */
-    int num_times_left_for_soft_thread;
+    fcs_int_limit_t num_times_left_for_soft_thread;
 
     /*
      * These variables are used to compute the MD5 checksum of a state
@@ -667,7 +696,7 @@ struct fc_solve_hard_thread_struct
     dll_ind_buf_t indirect_stacks_buffer;
 #endif
 
-    int prelude_num_items;
+    fcs_int_limit_t prelude_num_items;
     int prelude_idx;
     fcs_prelude_item_t * prelude;
 
@@ -681,12 +710,16 @@ struct fc_solve_hard_thread_struct
      * */
     int num_soft_threads_finished;
 
-   char * prelude_as_string;
+    char * prelude_as_string;
 };
 
 
 /********************************************/
 
+typedef struct {
+    int idx;
+    pq_rating_t rating;
+} fcs_rating_with_index_t;
 
 typedef struct
 {
@@ -696,7 +729,7 @@ typedef struct
     int tests_list_index;
     int test_index;
     int derived_states_random_indexes_max_size;
-    int * derived_states_random_indexes;
+    fcs_rating_with_index_t * derived_states_random_indexes;
     char * positions_by_rank;
     fcs_game_limit_t num_vacant_stacks;
     fcs_game_limit_t num_vacant_freecells;
@@ -728,7 +761,8 @@ enum
 typedef struct {
     fc_solve_solve_for_state_test_t * tests;
     int num_tests;
-    int to_randomize;
+    int shuffling_type;
+    fc_solve_state_weighting_t weighting;
 } fcs_tests_list_t;
 
 typedef struct {
@@ -746,6 +780,15 @@ typedef struct {
     fcs_tests_by_depth_unit_t * by_depth_units;
 } fcs_tests_by_depth_array_t;
 
+enum FCS_SUPER_METHOD_TYPE
+{
+    FCS_SUPER_METHOD_DFS,
+    FCS_SUPER_METHOD_BEFS_BRFS,
+};
+
+typedef enum FCS_SUPER_METHOD_TYPE fcs_super_method_type_t;
+
+
 struct fc_solve_soft_thread_struct
 {
     fc_solve_hard_thread_t * hard_thread;
@@ -768,12 +811,17 @@ struct fc_solve_soft_thread_struct
 
 
     /*
-     * The method (i.e: DFS, Soft-DFS, BFS or BeFS) that is used by this
+     * The method (i.e: Soft-DFS, Random-DFS, BFS or BeFS) that is used by this
      * instance.
      *
      * */
     int method;
 
+    /* The super-method type - can be  */
+    fcs_super_method_type_t super_method_type;
+
+    fc_solve_seq_cards_power_type_t initial_cards_under_sequences_value;
+
     union
     {
         struct
@@ -860,13 +908,7 @@ struct fc_solve_soft_thread_struct
                      * The priority queue of the BeFS scan
                      * */
                     PQUEUE pqueue;
-                    double initial_cards_under_sequences_value;
-
-                    /*
-                     * The BeFS weights of the different BeFS tests. Those
-                     * weights determine the commulative priority of the state.
-                     * */
-                    double befs_weights[5];
+                    fc_solve_state_weighting_t weighting;
                 } befs;
             } meth;
         } befs;
@@ -912,12 +954,16 @@ struct fc_solve_soft_thread_struct
     fcs_bool_t enable_pruning;
 };
 
+#define DFS_VAR(soft_thread,var) (soft_thread)->method_specific.soft_dfs.var
+#define BEFS_VAR(soft_thread,var) (soft_thread)->method_specific.befs.meth.befs.var
+/* M is Methods-common. */
+#define BEFS_M_VAR(soft_thread,var) (soft_thread)->method_specific.befs.var
+#define BRFS_VAR(soft_thread,var) (soft_thread)->method_specific.befs.meth.brfs.var
+
 typedef struct fc_solve_soft_thread_struct fc_solve_soft_thread_t;
 
 #define FC_SOLVE_IS_DFS(soft_thread) \
-    ((soft_thread->method == FCS_METHOD_SOFT_DFS) ||  \
-     (soft_thread->method == FCS_METHOD_RANDOM_DFS) \
-    )
+    ((soft_thread)->super_method_type == FCS_SUPER_METHOD_DFS)
 
 /*
  * An enum that specifies the meaning of each BeFS weight.
@@ -927,33 +973,30 @@ typedef struct fc_solve_soft_thread_struct fc_solve_soft_thread_t;
 #define FCS_BEFS_WEIGHT_CARDS_UNDER_SEQUENCES 2
 #define FCS_BEFS_WEIGHT_SEQS_OVER_RENEGADE_CARDS 3
 #define FCS_BEFS_WEIGHT_DEPTH 4
+#define FCS_BEFS_WEIGHT_NUM_CARDS_NOT_ON_PARENTS 5
 
-fc_solve_instance_t * fc_solve_alloc_instance(void);
+fc_solve_instance_t * fc_solve_alloc_instance(fcs_meta_compact_allocator_t * meta_alloc);
 
 extern void fc_solve_init_instance(
     fc_solve_instance_t * instance
-    );
-
-extern void fc_solve_free_instance(
-    fc_solve_instance_t * instance
-    );
+);
 
 extern void fc_solve_finish_instance(
     fc_solve_instance_t * instance
-    );
+);
 
 extern void fc_solve_start_instance_process_with_board(
     fc_solve_instance_t * instance,
     fcs_state_keyval_pair_t * init_state_val
-    );
+);
 
 extern int fc_solve_befs_or_bfs_do_solve(
     fc_solve_soft_thread_t * soft_thread
-    );
+);
 
 extern void fc_solve_increase_dfs_max_depth(
     fc_solve_soft_thread_t * soft_thread
-    );
+);
 
 static GCC_INLINE void * memdup(void * src, size_t mysize)
 {
@@ -970,30 +1013,180 @@ static GCC_INLINE void * memdup(void * src, size_t mysize)
     return dest;
 }
 
+static GCC_INLINE int update_col_cards_under_sequences(
+#ifndef FCS_FREECELL_ONLY
+    const int sequences_are_built_by,
+#endif
+    const fcs_cards_column_t col,
+    int d /* One less than cards_num of col. */
+)
+{
+    fcs_card_t this_card = fcs_col_get_card(col, d);
+    fcs_card_t prev_card = fcs_col_get_card(col, d-1);
+    for (; (d > 0) && ({ prev_card=fcs_col_get_card(col, d-1); fcs_is_parent_card(this_card, prev_card); }) ; d--, this_card = prev_card)
+    {
+    }
+    return d;
+}
+
+static GCC_INLINE void fc_solve_soft_thread_update_initial_cards_val(
+    fc_solve_soft_thread_t * soft_thread
+)
+{
+    fc_solve_instance_t * const instance = soft_thread->hard_thread->instance;
+#ifndef FCS_FREECELL_ONLY
+    const int sequences_are_built_by = GET_INSTANCE_SEQUENCES_ARE_BUILT_BY(instance);
+#endif
+    fcs_kv_state_t pass;
+
+    pass.key = &(instance->state_copy_ptr->s);
+    pass.val = &(instance->state_copy_ptr->info);
+
+    fc_solve_seq_cards_power_type_t cards_under_sequences = 0;
+    for (int a = 0 ; a < INSTANCE_STACKS_NUM ; a++)
+    {
+        const fcs_cards_column_t col = fcs_state_get_col(*pass.key, a);
+        cards_under_sequences += FCS_SEQS_OVER_RENEGADE_POWER(update_col_cards_under_sequences(
+#ifndef FCS_FREECELL_ONLY
+                sequences_are_built_by,
+#endif
+                col,
+                fcs_col_len(col)-1
+        )
+        );
+    }
+    soft_thread->initial_cards_under_sequences_value = cards_under_sequences;
+
+    return;
+}
+
+#define BEFS_MAX_DEPTH 20000
+
+extern const double fc_solve_default_befs_weights[FCS_NUM_BEFS_WEIGHTS];
+
+#ifdef FCS_FREECELL_ONLY
+#define is_filled_by_any_card() 1
+#else
+#define is_filled_by_any_card() (INSTANCE_EMPTY_STACKS_FILL == FCS_ES_FILLED_BY_ANY_CARD)
+#endif
+static GCC_INLINE void fc_solve_initialize_befs_rater(
+    fc_solve_soft_thread_t * const soft_thread,
+    fc_solve_state_weighting_t * weighting
+)
+{
+    double * const befs_weights = weighting->befs_weights;
+    double normalized_befs_weights[COUNT(weighting->befs_weights)];
+
+    /* Normalize the BeFS Weights, so the sum of all of them would be 1. */
+    double sum = 0;
+    for (int i = 0 ; i < FCS_NUM_BEFS_WEIGHTS; i++)
+    {
+        if (befs_weights[i] < 0)
+        {
+            befs_weights[i] = fc_solve_default_befs_weights[i];
+        }
+        sum += befs_weights[i];
+    }
+    if (sum < 1e-6)
+    {
+        sum = 1;
+    }
+    for (int i=0 ; i < FCS_NUM_BEFS_WEIGHTS ; i++)
+    {
+        normalized_befs_weights[i] = ((befs_weights[i] /= sum) * INT_MAX);
+    }
+#define pass (*raw_pass_raw)
+#define ptr_state_key (raw_pass_raw->key)
+
+#ifndef HARD_CODED_NUM_STACKS
+    fc_solve_hard_thread_t * const hard_thread = soft_thread->hard_thread;
+    fc_solve_instance_t * const instance = hard_thread->instance;
+#endif
+
+#if ((!defined(HARD_CODED_NUM_FREECELLS)) || (!defined(HARD_CODED_NUM_STACKS)) || (!defined(HARD_CODED_NUM_DECKS)))
+    SET_GAME_PARAMS();
+#endif
+
+#ifndef FCS_FREECELL_ONLY
+    fcs_bool_t bool_unlimited_sequence_move = INSTANCE_UNLIMITED_SEQUENCE_MOVE;
+#define unlimited_sequence_move bool_unlimited_sequence_move
+#else
+#define unlimited_sequence_move FALSE
+#endif
+
+    double num_cards_out_factor =
+        normalized_befs_weights[FCS_BEFS_WEIGHT_CARDS_OUT] / (LOCAL_DECKS_NUM*52);
+
+    double out_sum = 0.0;
+    for (int i=0 ; i <= 13 ; i++, out_sum += num_cards_out_factor)
+    {
+        weighting->num_cards_out_lookup_table[i] = out_sum;
+    }
+
+    weighting->max_sequence_move_factor =
+        normalized_befs_weights[FCS_BEFS_WEIGHT_MAX_SEQUENCE_MOVE] /
+        (is_filled_by_any_card()
+         ? (unlimited_sequence_move
+            ? (LOCAL_FREECELLS_NUM+INSTANCE_STACKS_NUM)
+            : ((LOCAL_FREECELLS_NUM+1)<<(INSTANCE_STACKS_NUM))
+         )
+         :
+           (unlimited_sequence_move
+            ? LOCAL_FREECELLS_NUM
+            : 1
+           )
+        );
+
+
+    weighting->cards_under_sequences_factor =
+        normalized_befs_weights[FCS_BEFS_WEIGHT_CARDS_UNDER_SEQUENCES] / soft_thread->initial_cards_under_sequences_value;
+
+    weighting->seqs_over_renegade_cards_factor =
+        normalized_befs_weights[FCS_BEFS_WEIGHT_SEQS_OVER_RENEGADE_CARDS] / FCS_SEQS_OVER_RENEGADE_POWER(LOCAL_DECKS_NUM*(13*4));
+
+    weighting->depth_factor =
+        normalized_befs_weights[FCS_BEFS_WEIGHT_DEPTH] / BEFS_MAX_DEPTH;
+
+    weighting->num_cards_not_on_parents_factor =
+        normalized_befs_weights[FCS_BEFS_WEIGHT_NUM_CARDS_NOT_ON_PARENTS] / (LOCAL_DECKS_NUM * 52);
+
+
+    weighting->should_go_over_stacks =
+    (
+        weighting->max_sequence_move_factor
+        || weighting->cards_under_sequences_factor
+        || weighting->seqs_over_renegade_cards_factor
+    );
+}
+
+#undef ptr_state_key
+#undef pass
+#undef unlimited_sequence_move
 
 static GCC_INLINE void fc_solve_soft_thread_init_soft_dfs(
     fc_solve_soft_thread_t * soft_thread
-    )
+)
 {
+    fc_solve_soft_thread_update_initial_cards_val(soft_thread);
     fc_solve_instance_t * instance = soft_thread->hard_thread->instance;
 
     fcs_state_keyval_pair_t * ptr_orig_state = instance->state_copy_ptr;
     /*
         Allocate some space for the states at depth 0.
     */
-    soft_thread->method_specific.soft_dfs.depth = 0;
+    DFS_VAR(soft_thread, depth) = 0;
 
     fc_solve_increase_dfs_max_depth(soft_thread);
 
-    soft_thread->method_specific.soft_dfs.soft_dfs_info[0].state
+    DFS_VAR(soft_thread, soft_dfs_info)[0].state
         = FCS_STATE_keyval_pair_to_collectible(ptr_orig_state);
 
     fc_solve_rand_init(
-            &(soft_thread->method_specific.soft_dfs.rand_gen),
-            soft_thread->method_specific.soft_dfs.rand_seed
+            &(DFS_VAR(soft_thread, rand_gen)),
+            DFS_VAR(soft_thread, rand_seed)
     );
 
-    if (! soft_thread->method_specific.soft_dfs.tests_by_depth_array.by_depth_units)
+    if (! DFS_VAR(soft_thread, tests_by_depth_array).by_depth_units)
     {
         fcs_tests_list_of_lists * tests_list_of_lists;
         fc_solve_solve_for_state_test_t * tests_list, * next_test;
@@ -1001,20 +1194,18 @@ static GCC_INLINE void fc_solve_soft_thread_init_soft_dfs(
         fcs_tests_by_depth_array_t * arr_ptr;
 
         int tests_order_num;
-        int * tests_order_tests;
-        int start_i;
+        fcs_tests_order_group_t * tests_order_groups;
         fcs_bool_t master_to_randomize =
             (soft_thread->method == FCS_METHOD_RANDOM_DFS)
             ;
-        fcs_bool_t do_first_iteration;
         int depth_idx;
         fcs_by_depth_tests_order_t * by_depth_tests_order;
 
-        arr_ptr = &(soft_thread->method_specific.soft_dfs.tests_by_depth_array);
+        arr_ptr = &(DFS_VAR(soft_thread, tests_by_depth_array));
         arr_ptr->by_depth_units =
-            malloc(
-                sizeof(arr_ptr->by_depth_units[0])
-                * (arr_ptr->num_units = soft_thread->by_depth_tests_order.num)
+            SMALLOC(
+                arr_ptr->by_depth_units,
+                (arr_ptr->num_units = soft_thread->by_depth_tests_order.num)
             );
 
         by_depth_tests_order =
@@ -1027,79 +1218,57 @@ static GCC_INLINE void fc_solve_soft_thread_init_soft_dfs(
             arr_ptr->by_depth_units[depth_idx].max_depth =
                 by_depth_tests_order[depth_idx].max_depth;
 
-            tests_order_tests = by_depth_tests_order[depth_idx].tests_order.tests;
+            tests_order_groups = by_depth_tests_order[depth_idx].tests_order.groups;
 
-            tests_order_num = by_depth_tests_order[depth_idx].tests_order.num;
+            tests_order_num = by_depth_tests_order[depth_idx].tests_order.num_groups;
 
             tests_list_of_lists =
                 &(arr_ptr->by_depth_units[depth_idx].tests);
 
             tests_list_of_lists->num_lists = 0;
             tests_list_of_lists->lists =
-                malloc(
-                        sizeof(tests_list_of_lists->lists[0]) * tests_order_num
-                      );
+                SMALLOC( tests_list_of_lists->lists, tests_order_num );
 
-            tests_list = malloc(sizeof(tests_list[0]) * tests_order_num);
-
-            start_i = 0;
-            while (start_i < tests_order_num)
+            for (int group_idx = 0 ; group_idx < tests_order_num ; group_idx++)
             {
-                int i;
-
-                do_first_iteration = TRUE;
-
-                for (i = start_i, next_test = tests_list ;
-                        (i < tests_order_num) &&
-                        (do_first_iteration ||
-                         ((!master_to_randomize) ||
-                          (
-                           /* We are still on a random group */
-                           (tests_order_tests[ i ] & FCS_TEST_ORDER_FLAG_RANDOM) &&
-                           /* A new random group did not start */
-                           (! (tests_order_tests[ i ] & FCS_TEST_ORDER_FLAG_START_RANDOM_GROUP))
-
-                          )
-                         )
-                        )
-                        ;
-                        i++)
+                int num = tests_order_groups[group_idx].num;
+                int * tests_order_tests = tests_order_groups[group_idx].tests;
+                tests_list = SMALLOC(tests_list, num);
+                next_test = tests_list;
+                for (int i = 0; i < num ; i++)
                 {
-                    do_first_iteration = FALSE;
-                    *(next_test++) =
-                        fc_solve_sfs_tests[
-                        tests_order_tests[i] & FCS_TEST_ORDER_NO_FLAGS_MASK
-                        ];
+                    *(next_test++) = fc_solve_sfs_tests[ tests_order_tests[i] ];
                 }
+                tests_list_struct_ptr =
+                    &(tests_list_of_lists->lists[tests_list_of_lists->num_lists++])
+                    ;
 
-                        tests_list_struct_ptr =
-                            &(tests_list_of_lists->lists[tests_list_of_lists->num_lists++])
-                            ;
+                tests_list_struct_ptr->tests = tests_list;
+                tests_list_struct_ptr->num_tests = num;
 
-                        ;
-                        tests_list_struct_ptr->tests =
-                            memdup(tests_list,
-                                    sizeof(tests_list[0])
-                                    * (tests_list_struct_ptr->num_tests = i-start_i)
-                                  );
+                tests_list_struct_ptr->shuffling_type =
+                    master_to_randomize
+                    ? tests_order_groups[group_idx].shuffling_type
+                    : FCS_NO_SHUFFLING;
 
-                        tests_list_struct_ptr->to_randomize =
-                            master_to_randomize &&
-                            (i > start_i) &&
-                            (tests_order_tests[ i-1 ] & FCS_TEST_ORDER_FLAG_RANDOM)
-                            ;
+                if (tests_list_struct_ptr->shuffling_type == FCS_WEIGHTING)
+                {
+                    tests_list_struct_ptr->weighting =
+                        tests_order_groups[group_idx].weighting;
 
-                        start_i = i;
+                    fc_solve_initialize_befs_rater(
+                        soft_thread,
+                        &(tests_list_struct_ptr->weighting)
+                    );
+                }
             }
 
             tests_list_of_lists->lists =
-                realloc(
+                SREALLOC(
                         tests_list_of_lists->lists,
-                        sizeof(tests_list_of_lists->lists[0]) *
                         tests_list_of_lists->num_lists
-                       );
+                );
 
-            free(tests_list);
         }
     }
 
@@ -1110,13 +1279,13 @@ extern int fc_solve_soft_dfs_do_solve(fc_solve_soft_thread_t * soft_thread);
 
 extern void fc_solve_soft_thread_init_befs_or_bfs(
     fc_solve_soft_thread_t * soft_thread
-    );
+);
 
 
 static GCC_INLINE int run_hard_thread(fc_solve_hard_thread_t * hard_thread)
 {
     fc_solve_soft_thread_t * soft_thread;
-    int num_times_started_at;
+    fcs_int_limit_t num_times_started_at;
     int ret;
     fc_solve_instance_t * instance = hard_thread->instance;
     int * st_idx_ptr = &(hard_thread->st_idx);
@@ -1189,11 +1358,9 @@ static GCC_INLINE int run_hard_thread(fc_solve_hard_thread_t * hard_thread)
          * that will abstract a scan. But it's not critical since
          * I don't support user-defined scans.
          * */
-        switch(soft_thread->method)
+        switch(soft_thread->super_method_type)
         {
-            case FCS_METHOD_SOFT_DFS:
-            case FCS_METHOD_HARD_DFS:
-            case FCS_METHOD_RANDOM_DFS:
+            case FCS_SUPER_METHOD_DFS:
 
             if (! STRUCT_QUERY_FLAG(soft_thread, FCS_SOFT_THREAD_INITIALIZED))
             {
@@ -1205,9 +1372,7 @@ static GCC_INLINE int run_hard_thread(fc_solve_hard_thread_t * hard_thread)
 
             break;
 
-            case FCS_METHOD_BFS:
-            case FCS_METHOD_A_STAR:
-            case FCS_METHOD_OPTIMIZE:
+            case FCS_SUPER_METHOD_BEFS_BRFS:
 
             if (! STRUCT_QUERY_FLAG(soft_thread, FCS_SOFT_THREAD_INITIALIZED))
             {
@@ -1268,7 +1433,7 @@ static GCC_INLINE int run_hard_thread(fc_solve_hard_thread_t * hard_thread)
              * */
             if (STRUCT_QUERY_FLAG(soft_thread, FCS_SOFT_THREAD_IS_A_COMPLETE_SCAN) &&
                     (! STRUCT_QUERY_FLAG(instance, FCS_RUNTIME_SCANS_SYNERGY))
-               )
+            )
             {
                 return FCS_STATE_IS_NOT_SOLVEABLE;
             }
@@ -1295,7 +1460,7 @@ static GCC_INLINE int run_hard_thread(fc_solve_hard_thread_t * hard_thread)
                     )
                 )
             )
-           )
+        )
         {
             return ret;
         }
@@ -1306,12 +1471,12 @@ static GCC_INLINE int run_hard_thread(fc_solve_hard_thread_t * hard_thread)
 
 extern void fc_solve_trace_solution(
     fc_solve_instance_t * instance
-    );
+);
 
 extern void fc_solve_instance__init_hard_thread(
-        fc_solve_instance_t * instance,
-        fc_solve_hard_thread_t * hard_thread
-        );
+    fc_solve_instance_t * instance,
+    fc_solve_hard_thread_t * hard_thread
+);
 
 extern void fc_solve_free_soft_thread_by_depth_test_array(fc_solve_soft_thread_t * soft_thread);
 
@@ -1319,23 +1484,31 @@ static GCC_INLINE fcs_tests_order_t tests_order_dup(fcs_tests_order_t * orig)
 {
     fcs_tests_order_t ret;
 
-    ret.num = orig->num;
+    ret.num_groups = orig->num_groups;
 
-    ret.tests =
-        malloc(sizeof(ret.tests[0]) *
-            ((ret.num & (~(TESTS_ORDER_GROW_BY - 1)))+TESTS_ORDER_GROW_BY)
+    ret.groups = memdup(orig->groups, sizeof(orig->groups[0]) *
+                        ((ret.num_groups & (~(TESTS_ORDER_GROW_BY - 1)))+TESTS_ORDER_GROW_BY)
+    );
+
+    for (int i = 0 ; i < ret.num_groups ; i++)
+    {
+        ret.groups[i].tests = memdup(
+            ret.groups[i].tests,
+            sizeof(ret.groups[i].tests[0]) *
+            ((ret.groups[i].num & (~(TESTS_ORDER_GROW_BY - 1)))+TESTS_ORDER_GROW_BY)
         );
-    memcpy(ret.tests, orig->tests, sizeof(ret.tests[0]) * ret.num);
+    }
 
     return ret;
 }
+
 /*
     This function optimizes the solution path using a BFS scan on the
     states in the solution path.
 */
 static GCC_INLINE int fc_solve_optimize_solution(
     fc_solve_instance_t * instance
-    )
+)
 {
     fc_solve_soft_thread_t * soft_thread;
     fc_solve_hard_thread_t * old_hard_thread, * optimization_thread;
@@ -1346,7 +1519,7 @@ static GCC_INLINE int fc_solve_optimize_solution(
     {
         instance->optimization_thread =
             optimization_thread =
-            malloc(sizeof(*optimization_thread));
+            SMALLOC1(optimization_thread);
 
         fc_solve_instance__init_hard_thread(instance, optimization_thread);
 
@@ -1374,7 +1547,7 @@ static GCC_INLINE int fc_solve_optimize_solution(
 
         soft_thread->by_depth_tests_order.num = 1;
         soft_thread->by_depth_tests_order.by_depth_tests =
-            malloc(sizeof(soft_thread->by_depth_tests_order.by_depth_tests[0]));
+            SMALLOC1(soft_thread->by_depth_tests_order.by_depth_tests);
 
         soft_thread->by_depth_tests_order.by_depth_tests[0].max_depth = INT_MAX;
         soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order =
@@ -1382,6 +1555,7 @@ static GCC_INLINE int fc_solve_optimize_solution(
     }
 
     soft_thread->method = FCS_METHOD_OPTIMIZE;
+    soft_thread->super_method_type = FCS_SUPER_METHOD_BEFS_BRFS;
 
     STRUCT_TURN_ON_FLAG(soft_thread, FCS_SOFT_THREAD_IS_A_COMPLETE_SCAN);
 
@@ -1397,15 +1571,13 @@ static GCC_INLINE int fc_solve_optimize_solution(
 
     STRUCT_TURN_ON_FLAG(instance, FCS_RUNTIME_IN_OPTIMIZATION_THREAD);
 
-    return
-        fc_solve_befs_or_bfs_do_solve(
-            soft_thread
-            );
+    return fc_solve_befs_or_bfs_do_solve( soft_thread );
 }
+
 /* Resume a solution process that was stopped in the middle */
 static GCC_INLINE int fc_solve_resume_instance(
     fc_solve_instance_t * instance
-    )
+)
 {
     int ret = FCS_STATE_SUSPEND_PROCESS;
     fc_solve_hard_thread_t * hard_thread;
@@ -1422,7 +1594,7 @@ static GCC_INLINE int fc_solve_resume_instance(
         ret =
             fc_solve_befs_or_bfs_do_solve(
                 &(instance->optimization_thread->soft_threads[0])
-                );
+            );
     }
     else
     {
@@ -1467,7 +1639,7 @@ static GCC_INLINE int fc_solve_resume_instance(
                         )
                     )
 
-                   )
+                )
                 {
                     goto end_of_hard_threads_loop;
                 }
@@ -1522,7 +1694,7 @@ static GCC_INLINE int fc_solve_resume_instance(
   */
 static GCC_INLINE void fc_solve_unresume_instance(
     fc_solve_instance_t * instance GCC_UNUSED
-    )
+)
 {
     /*
      * Do nothing - since finish_instance() can take care of solution_states
@@ -1534,24 +1706,22 @@ static GCC_INLINE void fc_solve_unresume_instance(
 
 
 static GCC_INLINE fc_solve_soft_thread_t *
-    fc_solve_instance_get_first_soft_thread(
-        fc_solve_instance_t * instance
-        )
+fc_solve_instance_get_first_soft_thread(
+    fc_solve_instance_t * instance
+)
 {
     return &(instance->hard_threads[0].soft_threads[0]);
 }
 
 extern fc_solve_soft_thread_t * fc_solve_new_soft_thread(
     fc_solve_hard_thread_t * const hard_thread
-    );
+);
 
 static GCC_INLINE fc_solve_soft_thread_t * fc_solve_new_hard_thread(
     fc_solve_instance_t * const instance
-    )
+)
 {
     fc_solve_hard_thread_t * ret;
-    HT_LOOP_DECLARE_VARS();
-
     /* Make sure we are not exceeding the maximal number of soft threads
      * for an instance. */
     if (instance->next_soft_thread_id == MAX_NUM_SCANS)
@@ -1560,17 +1730,13 @@ static GCC_INLINE fc_solve_soft_thread_t * fc_solve_new_hard_thread(
     }
 
     instance->hard_threads =
-        realloc(
-            instance->hard_threads,
-            (sizeof(instance->hard_threads[0]) * (instance->num_hard_threads+1))
-            );
+        SREALLOC( instance->hard_threads, instance->num_hard_threads+1 );
 
-    /* Since we realloc()ed the hard_threads, their addresses changed,
+    /* Since we SREALLOC()ed the hard_threads, their addresses changed,
      * so we need to update it.
      * */
     HT_LOOP_START()
     {
-        ST_LOOP_DECLARE_VARS();
         ST_LOOP_START()
         {
             soft_thread->hard_thread = hard_thread;
@@ -1591,7 +1757,7 @@ static GCC_INLINE fc_solve_soft_thread_t * fc_solve_new_hard_thread(
  * recycle_hard_thread() */
 static GCC_INLINE void fc_solve_reset_hard_thread(
     fc_solve_hard_thread_t * const hard_thread
-    )
+)
 {
     hard_thread->num_times = 0;
     hard_thread->max_num_times = INT_MAX;
@@ -1600,7 +1766,7 @@ static GCC_INLINE void fc_solve_reset_hard_thread(
 
 static GCC_INLINE void fc_solve_reset_soft_thread(
     fc_solve_soft_thread_t * const soft_thread
-    )
+)
 {
     STRUCT_CLEAR_FLAG(soft_thread, FCS_SOFT_THREAD_IS_FINISHED);
     STRUCT_CLEAR_FLAG(soft_thread, FCS_SOFT_THREAD_INITIALIZED);
@@ -1613,8 +1779,8 @@ static GCC_INLINE void fc_solve_release_tests_list(
 {
     if (is_scan_befs_or_bfs)
     {
-        free (soft_thread->method_specific.befs.tests_list);
-        soft_thread->method_specific.befs.tests_list = NULL;
+        free (BEFS_M_VAR(soft_thread, tests_list));
+        BEFS_M_VAR(soft_thread, tests_list) = NULL;
     }
     else
     {
@@ -1622,7 +1788,7 @@ static GCC_INLINE void fc_solve_release_tests_list(
         int unit_idx;
         fcs_tests_by_depth_array_t * arr;
 
-        arr = &(soft_thread->method_specific.soft_dfs.tests_by_depth_array);
+        arr = &(DFS_VAR(soft_thread, tests_by_depth_array));
         for (unit_idx = 0 ; unit_idx < arr->num_units ; unit_idx++)
         {
             if (arr->by_depth_units[unit_idx].tests.lists)
@@ -1647,10 +1813,8 @@ static GCC_INLINE void fc_solve_release_tests_list(
 
 static GCC_INLINE void fc_solve_instance__recycle_hard_thread(
     fc_solve_hard_thread_t * const hard_thread
-    )
+)
 {
-    ST_LOOP_DECLARE_VARS();
-
     fc_solve_reset_hard_thread(hard_thread);
     fc_solve_compact_allocator_recycle(&(hard_thread->allocator));
 
@@ -1658,9 +1822,7 @@ static GCC_INLINE void fc_solve_instance__recycle_hard_thread(
     {
         if (soft_thread->method == FCS_METHOD_A_STAR)
         {
-            fc_solve_PQueueFree(
-                &(soft_thread->method_specific.befs.meth.befs.pqueue)
-            );
+            fc_solve_PQueueFree( &(BEFS_VAR(soft_thread, pqueue)) );
         }
 
         fc_solve_reset_soft_thread(soft_thread);
@@ -1671,7 +1833,7 @@ static GCC_INLINE void fc_solve_instance__recycle_hard_thread(
 
 static GCC_INLINE void fc_solve_recycle_instance(
     fc_solve_instance_t * const instance
-        )
+)
 {
     int ht_idx;
 
@@ -1693,14 +1855,12 @@ static GCC_INLINE void fc_solve_recycle_instance(
     STRUCT_CLEAR_FLAG(instance, FCS_RUNTIME_IN_OPTIMIZATION_THREAD);
 }
 
-extern const double fc_solve_default_befs_weights[5];
-
 
 #ifdef FCS_RCS_STATES
 fcs_state_t * fc_solve_lookup_state_key_from_val(
-        fc_solve_instance_t * instance,
-        fcs_collectible_state_t * ptr_state_val
-        );
+    fc_solve_instance_t * instance,
+    fcs_collectible_state_t * ptr_state_val
+);
 
 extern int fc_solve_compare_lru_cache_keys(
     const void * void_a, const void * void_b, void * param
@@ -1708,12 +1868,83 @@ extern int fc_solve_compare_lru_cache_keys(
 
 #endif
 
+enum
+{
+    FOREACH_SOFT_THREAD_CLEAN_SOFT_DFS,
+    FOREACH_SOFT_THREAD_FREE_INSTANCE,
+    FOREACH_SOFT_THREAD_ACCUM_TESTS_ORDER,
+    FOREACH_SOFT_THREAD_DETERMINE_SCAN_COMPLETENESS
+};
+extern void fc_solve_foreach_soft_thread(
+    fc_solve_instance_t * instance,
+    int callback_choice,
+    void * context
+);
+
+static GCC_INLINE void free_instance_hard_thread_callback(fc_solve_hard_thread_t * hard_thread)
+{
+    if (likely(hard_thread->prelude_as_string))
+    {
+        free (hard_thread->prelude_as_string);
+    }
+    if (likely(hard_thread->prelude))
+    {
+        free (hard_thread->prelude);
+    }
+    fcs_move_stack_static_destroy(hard_thread->reusable_move_stack);
+
+    free(hard_thread->soft_threads);
+
+    fc_solve_compact_allocator_finish(&(hard_thread->allocator));
+}
+/*
+    This function is the last function that should be called in the
+    sequence of operations on instance, and it is meant for de-allocating
+    whatever memory was allocated by alloc_instance().
+  */
+
+static GCC_INLINE void fc_solve_free_tests_order(fcs_tests_order_t * tests_order)
+{
+    for (int group_idx = 0 ; group_idx < tests_order->num_groups ; group_idx++)
+    {
+        free (tests_order->groups[group_idx].tests);
+    }
+    free (tests_order->groups);
+    tests_order->groups = NULL;
+    tests_order->num_groups = 0;
+}
+
+static GCC_INLINE void fc_solve_free_instance(fc_solve_instance_t * instance)
+{
+    fc_solve_foreach_soft_thread(instance, FOREACH_SOFT_THREAD_FREE_INSTANCE, NULL);
+
+    HT_LOOP_START()
+    {
+        free_instance_hard_thread_callback(hard_thread);
+    }
+
+    free(instance->hard_threads);
+
+    if (instance->optimization_thread)
+    {
+        free_instance_hard_thread_callback(instance->optimization_thread);
+        free(instance->optimization_thread);
+    }
+    fc_solve_free_tests_order( &(instance->instance_tests_order) );
+    if (STRUCT_QUERY_FLAG(instance, FCS_RUNTIME_OPT_TESTS_ORDER_WAS_SET))
+    {
+        fc_solve_free_tests_order( &(instance->opt_tests_order) );
+    }
+
+    free(instance);
+}
+
 #define DECLARE_MOVE_FUNCTION(name) \
 extern void name( \
         fc_solve_soft_thread_t * const soft_thread, \
         fcs_kv_state_t * const raw_ptr_state_raw, \
         fcs_derived_states_list_t * const derived_states_list \
-        )
+)
 
 #ifdef __cplusplus
 }
