@@ -42,6 +42,7 @@
 #include <stddef.h>
 #include <assert.h>
 
+#include "alloc_wrap.h"
 #include "inline.h"
 
 #define DICT_IMPLEMENTATION
@@ -276,13 +277,13 @@ static int verify_dict_has_node(dnode_t *nil, dnode_t *root, dnode_t *node)
 #ifdef NO_FC_SOLVE
 dict_t *dict_create(dictcount_t maxcount, dict_comp_t comp, void * context)
 #else
-dict_t *fc_solve_kaz_tree_create(dict_comp_t comp, void * context)
+dict_t *fc_solve_kaz_tree_create(dict_comp_t comp, void * context, fcs_meta_compact_allocator_t * meta_allocator)
 #endif
 {
-    dict_t *dict = (dict_t *) malloc(sizeof *dict);
+    dict_t *dict = (dict_t *) SMALLOC1(dict);
 
     if (dict)
-        dict_init(dict, comp);
+        dict_init(dict, comp, meta_allocator);
 
     dict->context = context;
 
@@ -393,7 +394,9 @@ void dict_free(dict_t *dict)
 #ifdef NO_FC_SOLVE
 dict_t *dict_init(dict_t *dict, dictcount_t maxcount, dict_comp_t comp)
 #else
-dict_t *dict_init(dict_t *dict, dict_comp_t comp)
+dict_t *dict_init(dict_t *dict, dict_comp_t comp,
+    fcs_meta_compact_allocator_t * meta_allocator
+    )
 #endif
 {
     dict->compare = comp;
@@ -402,7 +405,7 @@ dict_t *dict_init(dict_t *dict, dict_comp_t comp)
     dict->allocnode = dnode_alloc;
     dict->freenode = dnode_free;
 #else
-    fc_solve_compact_allocator_init(&(dict->dict_allocator));
+    fc_solve_compact_allocator_init(&(dict->dict_allocator), meta_allocator);
     dict->dict_recycle_bin = NULL;
 #endif
 
@@ -1256,7 +1259,7 @@ static void dnode_free(dnode_t *node, void *context)
 
 dnode_t *dnode_create(void *data)
 {
-    dnode_t *dnode = (dnode_t *) malloc(sizeof *dnode);
+    dnode_t *dnode = (dnode_t *) SMALLOC1(dnode)
     if (dnode) {
         dnode->data = data;
         dnode->parent = NULL;
@@ -1527,15 +1530,6 @@ static int comparef(const void *key1, const void *key2)
     return strcmp((const char *) key1, (const char *) key2);
 }
 
-static char *dupstring(char *str)
-{
-    int sz = strlen(str) + 1;
-    char *dup = (char *) malloc(sz);
-    if (dup)
-        memcpy(dup, str, sz);
-    return dup;
-}
-
 static dnode_t *new_node(void *c)
 {
     static dnode_t few[5];
@@ -1594,8 +1588,8 @@ static void construct(dict_t *d)
                     puts("what?");
                     break;
                 }
-                key = dupstring(tok1);
-                val = dupstring(tok2);
+                key = strdup(tok1);
+                val = strdup(tok2);
                 dn = dnode_create(val);
 
                 if (!key || !val || !dn) {
@@ -1667,8 +1661,8 @@ int main(void)
                     puts("what?");
                     break;
                 }
-                key = dupstring(tok1);
-                val = dupstring(tok2);
+                key = strdup(tok1);
+                val = strdup(tok2);
 
                 if (!key || !val) {
                     puts("out of memory");
