@@ -28,20 +28,22 @@
 #include "avl.h"
 
 #include "inline.h"
+#include "alloc_wrap.h"
 
+#ifdef WITH_AVL_BALANCE_FIELD
 static GCC_INLINE signed char avl_get_balance(struct avl_node * node)
 {
-    return (((signed char)(node->avl_mylink[0] & 0x3))-1);
+    return node->avl_balance;
 }
 
 static GCC_INLINE struct avl_node * avl_process_link(uintptr_t mylink)
 {
-    return (struct avl_node *)(mylink & (~((uintptr_t)0x3)));
+    return (struct avl_node *)(mylink);
 }
 
 static GCC_INLINE void avl_set_link(struct avl_node * node, int myindex, struct avl_node * val)
 {
-    node->avl_mylink[myindex] = (((uintptr_t)val) | (node->avl_mylink[myindex] & 0x3));
+    node->avl_mylink[myindex] = ((uintptr_t)val);
 }
 
 /*
@@ -50,12 +52,40 @@ static GCC_INLINE void avl_set_link(struct avl_node * node, int myindex, struct
  *
  * Without bounding this, the program fails.
  * */
-#define AVL_BOUND(x) (((x) >= 1) ? 1 : ((x) <= (-1)) ? -1 : 0)
+#define AVL_BOUND(x) (((x) >= 3) ? 3 : ((x) <= (-3)) ? -3 : x)
 static GCC_INLINE void avl_set_balance(struct avl_node * node, signed char balance)
 {
-    node->avl_mylink[0] &= (~0x3);
-    node->avl_mylink[0] |= (((uintptr_t)(AVL_BOUND(balance)+1))&0x3);
+    node->avl_balance = AVL_BOUND(balance);
 }
+#else
+static GCC_INLINE signed char avl_get_balance(struct avl_node * node)
+{
+    return (((signed char)(node->avl_mylink[0] & 0x7))-3);
+}
+
+static GCC_INLINE struct avl_node * avl_process_link(uintptr_t mylink)
+{
+    return (struct avl_node *)(mylink & (~((uintptr_t)0x7)));
+}
+
+static GCC_INLINE void avl_set_link(struct avl_node * node, int myindex, struct avl_node * val)
+{
+    node->avl_mylink[myindex] = (((uintptr_t)val) | (node->avl_mylink[myindex] & 0x7));
+}
+
+/*
+ * "Be conservative in what you do, be liberal in what you accept from others."
+ * -- http://www.joelonsoftware.com/items/2008/03/17.htm
+ *
+ * Without bounding this, the program fails.
+ * */
+#define AVL_BOUND(x) (((x) >= 3) ? 3 : ((x) <= (-3)) ? -3 : x)
+static GCC_INLINE void avl_set_balance(struct avl_node * node, signed char balance)
+{
+    node->avl_mylink[0] &= (~0x7);
+    node->avl_mylink[0] |= (((uintptr_t)(AVL_BOUND(balance)+3))&0x7);
+}
+#endif
 
 static GCC_INLINE void avl_increment_balance(struct avl_node * node)
 {
@@ -67,9 +97,6 @@ static GCC_INLINE void avl_decrement_balance(struct avl_node * node)
     avl_set_balance(node, avl_get_balance(node)-1);
 }
 
-#define NEXT(p) L((p), 0)
-#define SET_NEXT(p, val) SET_L((p), 0, (val))
-
 #define NODEPTR_SET_BALANCE(p, b) avl_set_balance((p), (b))
 #define NODEPTR_INC_BALANCE(p) avl_increment_balance(p)
 #define NODEPTR_DEC_BALANCE(p) avl_decrement_balance(p)
@@ -79,21 +106,21 @@ static GCC_INLINE void avl_decrement_balance(struct avl_node * node)
    and memory allocator |allocator|.
    Returns |NULL| if memory allocation failed. */
 struct avl_table *
-avl_create (avl_comparison_func *compare, void *param, fcs_meta_compact_allocator_t * meta_alloc)
+avl_create (avl_comparison_func *compare, void *param, fcs_meta_compact_allocator_t * meta_alloc, void * * common_recycle_bin)
 {
   struct avl_table *tree;
 
   assert (compare != NULL);
 
-  tree = malloc (sizeof *tree);
+  tree = SMALLOC1 (tree);
   if (tree == NULL)
     return NULL;
 
-  tree->avl_root = NULL;
+  SET_TREE_AVL_ROOT(tree, NULL);
   tree->avl_compare = compare;
   tree->avl_param = param;
   fc_solve_compact_allocator_init(&(tree->avl_allocator), meta_alloc);
-  tree->avl_recycle_bin = NULL;
+  tree->avl_recycle_bin = (struct avl_node * *)common_recycle_bin;
   tree->avl_count = 0;
   tree->avl_generation = 0;
 
@@ -108,7 +135,7 @@ avl_find (const struct avl_table *tree, const void *item)
   struct avl_node *p;
 
   assert (tree != NULL && item != NULL);
-  for (p = tree->avl_root; p != NULL; )
+  for (p = TREE_AVL_ROOT(tree); p != NULL; )
     {
       int cmp = tree->avl_compare (item, NODE_DATA_PTR(p), tree->avl_param);
 
@@ -141,8 +168,8 @@ avl_probe (struct avl_table *tree, void *item)
 
   assert (tree != NULL && item != NULL);
 
-  z = (struct avl_node *) &tree->avl_root;
-  y = tree->avl_root;
+  z = (struct avl_node *)&(tree->avl_proto_root);
+  y = TREE_AVL_ROOT(tree);
   dir = 0;
   for (q = z, p = y; p != NULL; q = p, p = L(p, dir))
     {
@@ -155,9 +182,9 @@ avl_probe (struct avl_table *tree, void *item)
       da[k++] = dir = cmp > 0;
     }
 
-  if ((n = tree->avl_recycle_bin) != NULL)
+  if ((n = *(tree->avl_recycle_bin)) != NULL)
   {
-      tree->avl_recycle_bin = NEXT(n);
+      *(tree->avl_recycle_bin) = AVL_NEXT(n);
   }
   else
   {
@@ -165,6 +192,8 @@ avl_probe (struct avl_table *tree, void *item)
           &(tree->avl_allocator), sizeof(*n)
           );
   }
+  avl_set_decommissioned_flag(n, 0);
+
   SET_L(q, dir, n);
   if (n == NULL)
     return NULL;
@@ -308,7 +337,7 @@ avl_delete (struct avl_table *tree, const void *item)
   assert (tree != NULL && item != NULL);
 
   k = 0;
-  p = (struct avl_node *) &tree->avl_root;
+  p = (struct avl_node *) &tree->avl_proto_root;
   for (cmp = -1; cmp != 0;
        cmp = tree->avl_compare (item, NODE_DATA_PTR(p), tree->avl_param))
     {
@@ -367,8 +396,8 @@ avl_delete (struct avl_table *tree, const void *item)
 #if 0
   tree->avl_alloc->libavl_free (tree->avl_alloc, p);
 #else
-  SET_NEXT(p, tree->avl_recycle_bin);
-  tree->avl_recycle_bin = p;
+  AVL_SET_NEXT(p, *(tree->avl_recycle_bin));
+  *(tree->avl_recycle_bin) = p;
 #endif
 
   assert (k > 0);
@@ -503,7 +532,7 @@ trav_refresh (struct avl_traverser *trav)
       struct avl_node *i;
 
       trav->avl_height = 0;
-      for (i = trav->avl_table->avl_root; i != node; )
+      for (i = TREE_AVL_ROOT(trav->avl_table); i != node; )
         {
           assert (trav->avl_height < AVL_MAX_HEIGHT);
           assert (i != NULL);
@@ -539,7 +568,7 @@ avl_t_first (struct avl_traverser *trav, struct avl_table *tree)
   trav->avl_height = 0;
   trav->avl_generation = tree->avl_generation;
 
-  x = tree->avl_root;
+  x = TREE_AVL_ROOT(tree);
   if (x != NULL)
     while (L(x, 0) != NULL)
       {
@@ -566,7 +595,7 @@ avl_t_last (struct avl_traverser *trav, struct avl_table *tree)
   trav->avl_height = 0;
   trav->avl_generation = tree->avl_generation;
 
-  x = tree->avl_root;
+  x = TREE_AVL_ROOT(tree);
   if (x != NULL)
     while (L(x, 1) != NULL)
       {
@@ -593,7 +622,7 @@ avl_t_find (struct avl_traverser *trav, struct avl_table *tree, void *item)
   trav->avl_table = tree;
   trav->avl_height = 0;
   trav->avl_generation = tree->avl_generation;
-  for (p = tree->avl_root; p != NULL; p = q)
+  for (p = TREE_AVL_ROOT(tree); p != NULL; p = q)
     {
       int cmp = tree->avl_compare (item, NODE_DATA_PTR(p), tree->avl_param);
 
@@ -843,8 +872,8 @@ avl_copy (const struct avl_table *org, avl_copy_func *copy,
   if (new->avl_count == 0)
     return new;
 
-  x = (const struct avl_node *) &org->avl_root;
-  y = (struct avl_node *) &new->avl_root;
+  x = (const struct avl_node *) &TREE_AVL_ROOT(org);
+  y = (struct avl_node *) &TREE_AVL_ROOT(new);
   for (;;)
     {
       while (L(x, 0) != NULL)
@@ -856,7 +885,7 @@ avl_copy (const struct avl_table *org, avl_copy_func *copy,
                                            sizeof *L(y, 0));
           if (L(y, 0) == NULL)
             {
-              if (y != (struct avl_node *) &new->avl_root)
+              if (y != (struct avl_node *) &TREE_AVL_ROOT(new))
                 {
                   y->avl_data = NULL;
                   SET_L(y, 1, (NULL));
@@ -928,7 +957,7 @@ avl_destroy (struct avl_table *tree, avl_item_func *destroy)
 
   if (destroy != NULL)
   {
-      for (p = tree->avl_root; p != NULL; p = q)
+      for (p = TREE_AVL_ROOT(tree); p != NULL; p = q)
           if (L(p, 0) == NULL)
           {
               q = L(p, 1);
