@@ -47,27 +47,27 @@
 
 static void my_iter_handler(
     void * user_instance,
-    int iter_num,
+    fcs_int_limit_t iter_num,
     int depth,
     void * ptr_state,
-    int parent_iter_num,
+    fcs_int_limit_t parent_iter_num,
     void * lp_context
     )
 {
     fc_solve_display_information_context_t * context;
     context = (fc_solve_display_information_context_t*)lp_context;
 
-    fprintf(stdout, "Iteration: %i\n", iter_num);
+    fprintf(stdout, "Iteration: %li\n", (long)iter_num);
     fprintf(stdout, "Depth: %i\n", depth);
-    fprintf(stdout, "Stored-States: %i\n",
-        freecell_solver_user_get_num_states_in_collection(user_instance)
+    fprintf(stdout, "Stored-States: %li\n",
+        freecell_solver_user_get_num_states_in_collection_long(user_instance)
         );
     fprintf(stdout, "Scan: %s\n",
         freecell_solver_user_get_current_soft_thread_name(user_instance)
         );
     if (context->display_parent_iter_num)
     {
-        fprintf(stdout, "Parent Iteration: %i\n", parent_iter_num);
+        fprintf(stdout, "Parent Iteration: %li\n", (long)parent_iter_num);
     }
     fprintf(stdout, "\n");
 
@@ -259,6 +259,7 @@ help_screen_t help_screens[] = {
 "     3. The number of cards under sequences.\n"
 "     4. The length of the sequences which are found over renegade cards.\n"
 "     5. The depth of the board in the solution.\n"
+"     6. Inverse of number of cards not above their parents.\n"
 "\n"
 "-seed [seed_number]\n"
 "     Set the seed for the random number generator used by the\n"
@@ -448,7 +449,7 @@ static int cmd_line_callback(
     else if (IS_ARG("-i") || IS_ARG("--iter-output"))
     {
 #define set_iter_handler() \
-        freecell_solver_user_set_iter_handler(   \
+        freecell_solver_user_set_iter_handler_long(   \
             instance,   \
             my_iter_handler,   \
             dc    \
@@ -516,7 +517,7 @@ static int cmd_line_callback(
     else if (IS_ARG("--reset"))
     {
         init_debug_context(dc);
-        freecell_solver_user_set_iter_handler(
+        freecell_solver_user_set_iter_handler_long(
             instance,
             NULL,
             NULL
@@ -553,15 +554,15 @@ static void command_signal_handler(int signal_num GCC_UNUSED)
     {
         fprintf(
             stderr,
-            "The number of iterations is %i\n",
-            freecell_solver_user_get_num_times(current_instance)
+            "The number of iterations is %li\n",
+            freecell_solver_user_get_num_times_long(current_instance)
             );
     }
     else if (command_num == 1)
     {
         if (debug_iter_output_on)
         {
-            freecell_solver_user_set_iter_handler(
+            freecell_solver_user_set_iter_handler_long(
                 current_instance,
                 NULL,
                 NULL
@@ -570,7 +571,7 @@ static void command_signal_handler(int signal_num GCC_UNUSED)
         }
         else
         {
-            freecell_solver_user_set_iter_handler(
+            freecell_solver_user_set_iter_handler_long(
                 current_instance,
                 my_iter_handler,
                 dc
@@ -586,6 +587,10 @@ static void command_signal_handler(int signal_num GCC_UNUSED)
     command_num = 0;
 }
 
+static void abort_signal_handler(int signal_num GCC_UNUSED)
+{
+    freecell_solver_user_limit_iterations_long(current_instance, 0);
+}
 
 static freecell_solver_str_t known_parameters[] = {
     "-h", "--help",
@@ -714,24 +719,25 @@ int main(int argc, char * argv[])
 #ifndef WIN32
     signal(SIGUSR1, command_signal_handler);
     signal(SIGUSR2, select_signal_handler);
+    signal(SIGABRT, abort_signal_handler);
 #endif
 
 #if 0
     {
-        int limit = 500;
-        freecell_solver_user_limit_iterations(instance, limit);
+        fcs_int_limit_t limit = 500;
+        freecell_solver_user_limit_iterations_long(instance, limit);
         ret = freecell_solver_user_solve_board(instance, user_state);
         while (ret == FCS_STATE_SUSPEND_PROCESS)
         {
             limit += 500;
-            freecell_solver_user_limit_iterations(instance, limit);
+            freecell_solver_user_limit_iterations_long(instance, limit);
             ret = freecell_solver_user_resume_solution(instance);
         }
     }
 #elif defined(FCS_TRACE_MEM)
     {
 #define STEP 100000
-        int limit = STEP;
+        fcs_int_limit_t limit = STEP;
         char stat_fn[1024], foo_str[1024];
         fcs_portable_time_t mytime;
         FILE * stat;
@@ -740,7 +746,7 @@ int main(int argc, char * argv[])
 
         snprintf(stat_fn, sizeof(stat_fn), "/proc/%ld/stat", (long)(getpid()));
 
-        freecell_solver_user_limit_iterations(instance, limit);
+        freecell_solver_user_limit_iterations_long(instance, limit);
         ret = freecell_solver_user_solve_board(instance, user_state);
         while (ret == FCS_STATE_SUSPEND_PROCESS)
         {
@@ -795,7 +801,7 @@ int main(int argc, char * argv[])
 
             fflush(stdout);
             limit += STEP;
-            freecell_solver_user_limit_iterations(instance, limit);
+            freecell_solver_user_limit_iterations_long(instance, limit);
             ret = freecell_solver_user_resume_solution(instance);
         }
     }
