@@ -37,6 +37,7 @@ extern "C" {
 #include "config.h"
 
 #include "fcs_move.h"
+#include "fcs_limit.h"
 
 #include "inline.h"
 #include "bool.h"
@@ -56,6 +57,9 @@ extern "C" {
 #define FCS_MAX_NUM_SCANS_BUCKETS 4
 #endif
 
+/* How many ranks there are - Ace to King == 13. */
+#define FCS_MAX_RANK 13
+
 #define FCS_CHAR_BIT_SIZE_LOG2 3
 #define MAX_NUM_SCANS (FCS_MAX_NUM_SCANS_BUCKETS * (sizeof(unsigned char) * 8))
 
@@ -67,7 +71,7 @@ extern "C" {
 
 typedef struct
 {
-    short card_num;
+    short rank;
     char suit;
     char flags;
 } fcs_card_t;
@@ -92,6 +96,13 @@ typedef struct fcs_struct_state_t fcs_state_t;
 
 typedef int fcs_locs_t;
 
+static GCC_INLINE fcs_card_t fcs_make_card(const int rank, const int suit)
+{
+    fcs_card_t ret = { .rank = rank, .suit = suit, .flags = 0 };
+
+    return ret;
+}
+
 #define fcs_state_get_col(state, col_idx) \
     (&((state).stacks[(col_idx)]))
 
@@ -101,8 +112,8 @@ typedef int fcs_locs_t;
 #define fcs_col_get_card(col, c) \
     ((col)->cards[(c)] )
 
-#define fcs_card_card_num(card) \
-    ( (card).card_num )
+#define fcs_card_rank(card) \
+    ( (card).rank )
 
 #define fcs_card_suit(card) \
     ((int)( (card).suit ))
@@ -121,8 +132,8 @@ typedef int fcs_locs_t;
 #define fcs_card_set_suit(card, d) \
     (card).suit = (d)
 
-#define fcs_card_set_num(card, num) \
-    (card).card_num = (num)
+#define fcs_card_set_rank(card, num) \
+    (card).rank = (num)
 
 #ifndef FCS_WITHOUT_CARD_FLIPPING
 #define fcs_card_set_flipped(card, flipped) \
@@ -238,6 +249,8 @@ typedef struct fcs_struct_state_t fcs_state_t;
 
 typedef char fcs_locs_t;
 
+#else
+#error DEBUG_STATES , COMPACT_STATES or INDIRECT_STACK_STATES are not defined.
 #endif /* #ifdef DEBUG_STATES -
           #elif defined COMPACT_STATES -
           #elif defined INDIRECT_STACK_STATES
@@ -256,14 +269,14 @@ typedef char fcs_locs_t;
 #ifdef FCS_FREECELL_ONLY
 
 #define fcs_is_parent_card(child, parent) \
-    ((fcs_card_card_num(child)+1 == fcs_card_card_num(parent)) && \
+    ((fcs_card_rank(child)+1 == fcs_card_rank(parent)) && \
             ((fcs_card_suit(child) & 0x1) != (fcs_card_suit(parent)&0x1)) \
     )
 
 #else
 
 #define fcs_is_parent_card(child, parent) \
-    ((fcs_card_card_num(child)+1 == fcs_card_card_num(parent)) && \
+    ((fcs_card_rank(child)+1 == fcs_card_rank(parent)) && \
     ((sequences_are_built_by == FCS_SEQ_BUILT_BY_RANK) ?   \
         1 :                                                          \
         ((sequences_are_built_by == FCS_SEQ_BUILT_BY_SUIT) ?   \
@@ -274,11 +287,11 @@ typedef char fcs_locs_t;
 
 #endif
 
-#define fcs_col_get_card_num(col, card_idx) \
-    fcs_card_card_num(fcs_col_get_card((col), (card_idx)))
+#define fcs_col_get_rank(col, card_idx) \
+    fcs_card_rank(fcs_col_get_card((col), (card_idx)))
 
-#define fcs_freecell_card_num(state, f) \
-    ( fcs_card_card_num(fcs_freecell_card((state),(f))) )
+#define fcs_freecell_rank(state, f) \
+    ( fcs_card_rank(fcs_freecell_card((state),(f))) )
 
 #define fcs_freecell_card_suit(state, f) \
     ( fcs_card_suit(fcs_freecell_card((state),(f))) )
@@ -308,6 +321,10 @@ typedef char fcs_locs_t;
 #define fcs_col_push_col_card(dest_col, src_col, card_idx) \
     fcs_col_push_card((dest_col), fcs_col_get_card((src_col), (card_idx)))
 
+#define fcs_card_is_empty(card) (fcs_card_rank(card) == 0)
+#define fcs_card_is_valid(card) (fcs_card_rank(card) != 0)
+#define fcs_freecell_is_empty(state, idx) (fcs_card_is_empty(fcs_freecell_card(state, idx)))
+
 #if defined(COMPACT_STATES) || defined(DEBUG_STATES)
 
 #define fcs_duplicate_state_extra(ptr_dest, ptr_src) \
@@ -335,13 +352,18 @@ typedef char fcs_locs_t;
  * INDIRECT_STACK_STATES */
 #if defined(COMPACT_STATES) || defined(INDIRECT_STACK_STATES)
 
+static GCC_INLINE fcs_card_t fcs_make_card(const int rank, const int suit)
+{
+    return ( (((fcs_card_t)rank) << 2) | ((fcs_card_t)suit) );
+}
+
 #define fcs_col_len(col) \
     ( ((col)[0]) )
 
 #define fcs_col_get_card(col, card_idx) \
     ((col)[(card_idx)+1])
 
-#define fcs_card_card_num(card) \
+#define fcs_card_rank(card) \
     ( (card) >> 2 )
 
 #define fcs_card_suit(card) \
@@ -352,7 +374,7 @@ typedef char fcs_locs_t;
     ( (card) >> 6 )
 #endif
 
-#define fcs_card_set_num(card, num) \
+#define fcs_card_set_rank(card, num) \
     (card) = ((fcs_card_t)(((card)&0x03)|((num)<<2)));
 
 #define fcs_card_set_suit(card, suit) \
@@ -398,7 +420,7 @@ struct fcs_state_extra_info_struct
     /*
      * The iteration in which this state was marked as visited
      * */
-    int visited_iter;
+    fcs_int_limit_t visited_iter;
 #endif
 
     /*
@@ -492,6 +514,13 @@ typedef struct {
     fcs_state_extra_info_t * val;
 } fcs_kv_state_t;
 
+static GCC_INLINE void
+FCS_STATE_keyval_pair_to_kv(fcs_kv_state_t * ret, fcs_state_keyval_pair_t * s)
+{
+    ret->key = &(s->s);
+    ret->val = &(s->info);
+}
+
 /*
  * This type is the struct that is collectible inside the hash.
  *
@@ -548,12 +577,7 @@ typedef fcs_state_keyval_pair_t fcs_collectible_state_t;
 #define FCS_STATE_kv_to_collectible(s) ((fcs_collectible_state_t *)((s)->key))
 #define FCS_STATE_kv_to_pass(s) FCS_STATE_kv_to_collectible(s)
 
-static GCC_INLINE void
-FCS_STATE_collectible_to_kv(fcs_kv_state_t * ret, fcs_collectible_state_t * s)
-{
-    ret->key = &(s->s);
-    ret->val = &(s->info);
-}
+#define FCS_STATE_collectible_to_kv(ret,s) FCS_STATE_keyval_pair_to_kv((ret),(s))
 
 #endif
 
@@ -642,18 +666,18 @@ extern char * fc_solve_card_perl2user(
     );
 
 /*
- * Converts a card_number from its internal representation to a string.
+ * Converts a rank from its internal representation to a string.
  *
- * num - the card number
+ * rank_idx - the rank
  * str - the string to output to.
- * card_num_is_null - a pointer to a bool that indicates whether
+ * rank_is_null - a pointer to a bool that indicates whether
  *      the card number is out of range or equal to zero
  * t - whether 10 should be printed as T or not.
  * */
-extern char * fc_solve_p2u_card_number(
-    int num,
+extern char * fc_solve_p2u_rank(
+    int rank_idx,
     char * str,
-    int * card_num_is_null,
+    fcs_bool_t * rank_is_null,
     int t
 #ifndef FCS_WITHOUT_CARD_FLIPPING
     , int flipped
@@ -665,7 +689,7 @@ extern char * fc_solve_p2u_card_number(
  * (e.g: "A", "K", "9") to its card number that can be used by
  * the program.
  * */
-extern int fc_solve_u2p_card_number(const char * string);
+extern int fc_solve_u2p_rank(const char * string);
 
 /*
  * This function converts a string containing a suit letter (that is
@@ -887,9 +911,9 @@ static GCC_INLINE int fc_solve_initial_user_state_to_c_proto(
                 str++;
                 while (*str == '-')
                     str++;
-                /* Workaround for fc_solve_u2p_card_number's willingness
+                /* Workaround for fc_solve_u2p_rank's willingness
                  * to designate the string '0' as 10. */
-                c = ((str[0] == '0') ? 0 : fc_solve_u2p_card_number(str));
+                c = ((str[0] == '0') ? 0 : fc_solve_u2p_rank(str));
                 while (
                         (*str != ' ') &&
                         (*str != '\t') &&
@@ -912,6 +936,13 @@ static GCC_INLINE int fc_solve_initial_user_state_to_c_proto(
             continue;
         }
 
+        /* Handle columns that start with an initial colon, so we can
+         * input states from -p -t mid-play. */
+        if ((*str) == ':')
+        {
+            str++;
+        }
+
         col = fcs_state_get_col(ret, s);
         for(c=0 ; c < MAX_NUM_CARDS_IN_A_STACK ; c++)
         {
@@ -1014,11 +1045,12 @@ static GCC_INLINE int fc_solve_check_state_validity(
     /* Mark the cards in the freecells */
     for(f=0;f<freecells_num;f++)
     {
-        if (fcs_freecell_card_num(*state, f) != 0)
+        card = fcs_freecell_card(*state, f);
+        if (fcs_card_is_valid(card))
         {
             cards
-                [fcs_freecell_card_suit(*state, f)]
-                [fcs_freecell_card_num(*state, f)] ++;
+                [fcs_card_suit(card)]
+                [fcs_card_rank(card)] ++;
         }
     }
 
@@ -1030,30 +1062,28 @@ static GCC_INLINE int fc_solve_check_state_validity(
         for(c=0;c<col_len;c++)
         {
             card = fcs_col_get_card(col,c);
-            if (fcs_card_card_num(card) == 0)
+            if (fcs_card_is_empty(card))
             {
                 *misplaced_card = fc_solve_empty_card;
                 return FCS_STATE_VALIDITY__EMPTY_SLOT;
             }
             cards
                 [fcs_card_suit(card)]
-                [fcs_card_card_num(card)] ++;
+                [fcs_card_rank(card)] ++;
 
         }
     }
 
     /* Now check if there are extra or missing cards */
 
-    for(d=0;d<4;d++)
+    for(int suit_idx = 0; suit_idx < 4; suit_idx ++)
     {
-        for(c=1;c<=13;c++)
+        for (int rank = 1; rank <= FCS_MAX_RANK ; rank++)
         {
-            if (cards[d][c] != decks_num)
+            if (cards[suit_idx][rank] != decks_num)
             {
-                *misplaced_card = fc_solve_empty_card;
-                fcs_card_set_suit(*misplaced_card, d);
-                fcs_card_set_num(*misplaced_card, c);
-                return ((cards[d][c] < decks_num)
+                *misplaced_card = fcs_make_card(rank, suit_idx);
+                return ((cards[suit_idx][rank] < decks_num)
                     ? FCS_STATE_VALIDITY__MISSING_CARD
                     : FCS_STATE_VALIDITY__EXTRA_CARD
                     )
@@ -1084,6 +1114,10 @@ enum
 #define min(a,b) ((a)<(b)?(a):(b))
 #endif
 
+#ifndef max
+#define max(a,b) ((a)>(b)?(a):(b))
+#endif
+
 #define FCS_WITH_CARD_COMPARE_LOOKUP
 
 static GCC_INLINE int fc_solve_card_compare(
@@ -1094,11 +1128,11 @@ static GCC_INLINE int fc_solve_card_compare(
 #ifdef FCS_WITH_CARD_COMPARE_LOOKUP
     return (*c1)-(*c2);
 #else
-    if (fcs_card_card_num(*c1) > fcs_card_card_num(*c2))
+    if (fcs_card_rank(*c1) > fcs_card_rank(*c2))
     {
         return 1;
     }
-    else if (fcs_card_card_num(*c1) < fcs_card_card_num(*c2))
+    else if (fcs_card_rank(*c1) < fcs_card_rank(*c2))
     {
         return -1;
     }
@@ -1162,4 +1196,15 @@ static GCC_INLINE int fc_solve_stack_compare_for_comparison(const void * const v
 
 #endif
 
+#ifdef DEBUG_STATES
+
+#define DEFINE_fc_solve_empty_card() \
+    fcs_card_t fc_solve_empty_card = {0,0};
+
+#elif defined(COMPACT_STATES) || defined (INDIRECT_STACK_STATES)
+
+#define DEFINE_fc_solve_empty_card() \
+    fcs_card_t fc_solve_empty_card = (fcs_card_t)0;
+
+#endif
 #endif /* FC_SOLVE__STATE_H */
