@@ -32,7 +32,6 @@
 
 #define FCS_COMPILE_DEBUG_FUNCTIONS
 #include "../card.c"
-#include "../card_compare_lookups.c"
 #include "../app_str.c"
 #include "../state.c"
 #include "../delta_states.c"
@@ -41,14 +40,7 @@
 
 static fcs_card_t make_card(int rank, int suit)
 {
-    fcs_card_t card;
-
-    card = fc_solve_empty_card;
-
-    fcs_card_set_num(card, rank);
-    fcs_card_set_suit(card, suit);
-
-    return card;
+    return fcs_make_card(rank, suit);
 }
 
 static int test_encode_and_decode(fc_solve_delta_stater_t * delta, fcs_state_keyval_pair_t * state, const char * expected_str, const char * blurb)
@@ -729,7 +721,7 @@ int main_tests()
 
 int main(int argc, char * argv[])
 {
-    plan_tests(25);
+    plan_tests(28);
     main_tests();
     return exit_status();
 }
