@@ -34,6 +34,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "alloc_wrap.h"
 #include "portable_time.h"
 #include "portable_int64.h"
 
@@ -45,23 +46,29 @@
 #include "range_solvers_gen_ms_boards.h"
 #include "output_to_file.h"
 
+#ifdef FCS_TRACE_MEM
+#include "portable_time.h"
+#include <unistd.h>
+#include <sys/types.h>
+#endif
+
 static void my_iter_handler(
     void * user_instance,
-    int iter_num,
+    fcs_int_limit_t iter_num,
     int depth,
     void * ptr_state,
-    int parent_iter_num,
+    fcs_int_limit_t parent_iter_num,
     void * lp_context
     )
 {
     fc_solve_display_information_context_t * context;
     context = (fc_solve_display_information_context_t*)lp_context;
 
-    fprintf(stdout, "Iteration: %i\n", iter_num);
+    fprintf(stdout, "Iteration: %li\n", (long)iter_num);
     fprintf(stdout, "Depth: %i\n", depth);
     if (context->display_parent_iter_num)
     {
-        fprintf(stdout, "Parent Iteration: %i\n", parent_iter_num);
+        fprintf(stdout, "Parent Iteration: %li\n", (long)parent_iter_num);
     }
     fprintf(stdout, "\n");
 
@@ -110,7 +117,7 @@ static int cmd_line_callback(
 
     if ((!strcmp(argv[arg], "-i")) || (!strcmp(argv[arg], "--iter-output")))
     {
-        freecell_solver_user_set_iter_handler(
+        freecell_solver_user_set_iter_handler_long(
             instance,
             my_iter_handler,
             dc
@@ -161,10 +168,6 @@ static int cmd_line_callback(
     return FCS_CMD_LINE_SKIP;
 }
 
-#if 0
-#define total_iterations_limit_per_board 100000
-#endif
-
 static const char * known_parameters[] = {
     "-i", "--iter-output",
     "-s", "--state-output",
@@ -259,7 +262,7 @@ int main(int argc, char * argv[])
     int parser_ret;
 
     fcs_bool_t was_total_iterations_limit_per_board_set = FALSE;
-    int total_iterations_limit_per_board = -1;
+    fcs_int_limit_t total_iterations_limit_per_board = -1;
 
     char * binary_output_filename = NULL;
     fcs_state_string_t state_string;
@@ -269,6 +272,12 @@ int main(int argc, char * argv[])
     char * solution_filename = NULL;
 
     int arg = 1, start_from_arg;
+#ifdef FCS_TRACE_MEM
+    int rss_found = 0;
+    long max_rss_delta_deal = -1;
+    long long int prev_rss = -1, max_rss_delta = -1;
+#endif
+
     if (argc < 4)
     {
         fprintf(stderr, "Not Enough Arguments!\n");
@@ -304,7 +313,7 @@ int main(int argc, char * argv[])
                 exit(-1);
             }
             was_total_iterations_limit_per_board_set = TRUE;
-            total_iterations_limit_per_board = atoi(argv[arg]);
+            total_iterations_limit_per_board = (fcs_int_limit_t)atol(argv[arg]);
         }
         else if (!strcmp(argv[arg], "--solutions-directory"))
         {
@@ -316,7 +325,10 @@ int main(int argc, char * argv[])
                 exit(-1);
             }
             solutions_directory = argv[arg];
-            solution_filename = malloc(strlen(solutions_directory) + 1024);
+            solution_filename = SMALLOC(
+                solution_filename,
+                strlen(solutions_directory) + 1024
+            );
         }
         else
         {
@@ -350,7 +362,7 @@ int main(int argc, char * argv[])
 
             print_int_wrapper(start_board);
             print_int_wrapper(end_board);
-            print_int_wrapper(total_iterations_limit_per_board);
+            print_int_wrapper((int)total_iterations_limit_per_board);
         }
         else
         {
@@ -367,7 +379,11 @@ int main(int argc, char * argv[])
             }
             read_int_wrapper(start_board);
             read_int_wrapper(end_board);
-            read_int_wrapper(total_iterations_limit_per_board);
+            {
+                int val;
+                read_int_wrapper(val);
+                total_iterations_limit_per_board = val;
+            }
 
             fseek(in, 0, SEEK_END);
             file_len = ftell(in);
@@ -446,7 +462,7 @@ int main(int argc, char * argv[])
 
         if (was_total_iterations_limit_per_board_set)
         {
-            freecell_solver_user_limit_iterations(user.instance, total_iterations_limit_per_board);
+            freecell_solver_user_limit_iterations_long(user.instance, total_iterations_limit_per_board);
         }
 
         ret =
@@ -479,7 +495,7 @@ int main(int argc, char * argv[])
         }
         else
         {
-            print_int_wrapper(freecell_solver_user_get_num_times(user.instance));
+            print_int_wrapper((int)freecell_solver_user_get_num_times_long(user.instance));
         }
 
         if (solutions_directory)
@@ -508,7 +524,7 @@ int main(int argc, char * argv[])
             output_fh = NULL;
         }
 
-        total_num_iters += freecell_solver_user_get_num_times(user.instance);
+        total_num_iters += freecell_solver_user_get_num_times_long(user.instance);
 
         if (board_num % stop_at == 0)
         {
@@ -516,8 +532,85 @@ int main(int argc, char * argv[])
             fflush(stdout);
         }
 
-
         freecell_solver_user_recycle(user.instance);
+
+#if 0
+#elif defined(FCS_TRACE_MEM)
+    {
+#define STEP 100000
+        long long int rss;
+        unsigned long long unsigned_foo;
+        char stat_fn[1024], foo_str[1024];
+        FILE * stat;
+
+        snprintf(stat_fn, sizeof(stat_fn), "/proc/%ld/stat", (long)(getpid()));
+
+        {
+            /* This was taken from:
+             *
+             * http://www.brokestream.com/procstat.html
+             * */
+            stat = fopen(stat_fn, "r");
+#define readone(foo) (fscanf(stat, "%lld ", &rss))
+#define readstr(foo) (fscanf(stat, "%1000s ", foo_str))
+#define readchar(foo) (fscanf(stat, "%c ", foo_str))
+#define readunsigned(foo) (fscanf(stat, "%llu ", &unsigned_foo))
+            readone(&pid);
+            readstr(tcomm);
+            readchar(&state);
+            readone(&ppid);
+            readone(&pgid);
+            readone(&sid);
+            readone(&tty_nr);
+            readone(&tty_pgrp);
+            readone(&flags);
+            readone(&min_flt);
+            readone(&cmin_flt);
+            readone(&maj_flt);
+            readone(&cmaj_flt);
+            readone(&utime);
+            readone(&stimev);
+            readone(&cutime);
+            readone(&cstime);
+            readone(&priority);
+            readone(&nicev);
+            readone(&num_threads);
+            readone(&it_real_value);
+            readunsigned(&start_time);
+            readone(&vsize);
+            readone(&rss);
+#undef readone
+#undef readunsigned
+#undef readchar
+#undef readstr
+
+            fclose(stat);
+
+            long long rss_delta = rss - prev_rss;
+            if (rss_found == 1)
+            {
+                max_rss_delta = rss_delta;
+                max_rss_delta_deal = board_num;
+            }
+            else if (rss_found == 2)
+            {
+                if (rss_delta > max_rss_delta)
+                {
+                    max_rss_delta = rss_delta;
+                    max_rss_delta_deal = board_num;
+                }
+                printf("Max RSS delta %lld encountered at deal %ld\n",
+                    max_rss_delta, max_rss_delta_deal
+                );
+            }
+            prev_rss = rss;
+            if (rss_found < 2)
+            {
+                rss_found++;
+            }
+        }
+    }
+#endif
     }
 
     freecell_solver_user_free(user.instance);
@@ -529,5 +622,8 @@ int main(int argc, char * argv[])
         fclose(binary_output.file);
     }
 
+    free (solution_filename);
+    solution_filename = NULL;
+
     return 0;
 }
