@@ -37,6 +37,7 @@
 #include <unistd.h>
 #endif
 
+#include "alloc_wrap.h"
 #include "portable_int64.h"
 #include "portable_time.h"
 
@@ -67,21 +68,21 @@ typedef struct fc_solve_display_information_context_struct fc_solve_display_info
 
 static void my_iter_handler(
     void * user_instance,
-    int iter_num,
+    fcs_int_limit_t iter_num,
     int depth,
     void * ptr_state,
-    int parent_iter_num,
+    fcs_int_limit_t parent_iter_num,
     void * lp_context
     )
 {
     fc_solve_display_information_context_t * context;
     context = (fc_solve_display_information_context_t*)lp_context;
 
-    fprintf(stdout, "Iteration: %i\n", iter_num);
+    fprintf(stdout, "Iteration: %li\n", (long)iter_num);
     fprintf(stdout, "Depth: %i\n", depth);
     if (context->display_parent_iter_num)
     {
-        fprintf(stdout, "Parent Iteration: %i\n", parent_iter_num);
+        fprintf(stdout, "Parent Iteration: %li\n", (long)parent_iter_num);
     }
     fprintf(stdout, "\n");
 
@@ -130,7 +131,7 @@ static int cmd_line_callback(
 
     if ((!strcmp(argv[arg], "-i")) || (!strcmp(argv[arg], "--iter-output")))
     {
-        freecell_solver_user_set_iter_handler(
+        freecell_solver_user_set_iter_handler_long(
             instance,
             my_iter_handler,
             dc
@@ -192,8 +193,6 @@ static const char * known_parameters[] = {
 
 #define BINARY_OUTPUT_NUM_INTS 16
 
-#define print_int_wrapper(i) { }
-
 static void print_help(void)
 {
     printf("\n%s",
@@ -229,7 +228,7 @@ typedef struct {
     int stop_at;
 } context_t;
 
-int total_iterations_limit_per_board = -1;
+fcs_int_limit_t total_iterations_limit_per_board = -1;
 
 fcs_int64_t total_num_iters = 0;
 static pthread_mutex_t total_num_iters_lock;
@@ -247,7 +246,7 @@ static void * worker_thread(void * void_context)
     int quota_end;
     int stop_at;
     fcs_portable_time_t mytime;
-    long total_num_iters_temp = 0;
+    fcs_int_limit_t total_num_iters_temp = 0;
     /* 52 cards of 3 chars (suit+rank+whitespace) each,
      * plus 8 newlines, plus one '\0' terminator*/
     fcs_state_string_t state_string;
@@ -306,11 +305,12 @@ static void * worker_thread(void * void_context)
         {
             quota_end = end_board+1;
         }
-        for(;board_num<quota_end;board_num++)
+
+        for ( ; board_num < quota_end ; board_num++ )
         {
             get_board(board_num, state_string);
 
-            freecell_solver_user_limit_iterations(user.instance, total_iterations_limit_per_board);
+            freecell_solver_user_limit_iterations_long(user.instance, total_iterations_limit_per_board);
 
             ret =
                 freecell_solver_user_solve_board(
@@ -322,7 +322,6 @@ static void * worker_thread(void * void_context)
             {
                 FCS_PRINT_INTRACTABLE_BOARD(mytime, board_num);
                 fflush(stdout);
-                print_int_wrapper(-1);
             }
             else if (ret == FCS_STATE_FLARES_PLAN_ERROR)
             {
@@ -339,14 +338,9 @@ static void * worker_thread(void * void_context)
             {
                 FCS_PRINT_UNSOLVED_BOARD(mytime, board_num);
 
-                print_int_wrapper(-2);
-            }
-            else
-            {
-                print_int_wrapper(freecell_solver_user_get_num_times(user.instance));
             }
 
-            total_num_iters_temp += freecell_solver_user_get_num_times(user.instance);
+            total_num_iters_temp += freecell_solver_user_get_num_times_long(user.instance);
             if (total_num_iters_temp >= update_total_num_iters_threshold)
             {
                 pthread_mutex_lock(&total_num_iters_lock);
@@ -423,7 +417,7 @@ int main(int argc, char * argv[])
                 print_help();
                 exit(-1);
             }
-            total_iterations_limit_per_board = atoi(argv[arg]);
+            total_iterations_limit_per_board = atol(argv[arg]);
         }
         else if (!strcmp(argv[arg], "--num-workers"))
         {
@@ -474,7 +468,7 @@ int main(int argc, char * argv[])
     context.arg = start_from_arg;
     context.stop_at = stop_at;
 
-    workers = malloc(sizeof(workers[0])*num_workers);
+    workers = SMALLOC(workers, num_workers);
 
     for ( idx = 0 ; idx < num_workers ; idx++)
     {
